<?php
/**
 * Community Builder (TM)
 * @version $Id: $
 * @package CommunityBuilder
 * @copyright (C) 2004-2022 www.joomlapolis.com / Lightning MultiCom SA - and its licensors, all rights reserved
 * @license http://www.gnu.org/licenses/old-licenses/gpl-2.0.html GNU/GPL version 2
 */

use CB\Database\Table\PluginTable;
use CB\Plugin\AntiSpam\CBAntiSpam;
use CB\Plugin\AntiSpam\Table\BlockTable;
use CBLib\Application\Application;
use CBLib\Language\CBTxt;
use Joomla\CMS\Router\Route;
use Joomla\CMS\User\User;
use Kunena\Forum\Libraries\Forum\Topic\KunenaTopicHelper;
use Kunena\Forum\Libraries\User\KunenaUser as Kunena6User;
use Kunena\Forum\Libraries\Forum\Topic\KunenaTopic;
use Kunena\Forum\Libraries\Forum\Message\KunenaMessage;

if ( ! ( defined( '_VALID_CB' ) || defined( '_JEXEC' ) || defined( '_VALID_MOS' ) ) ) { die( 'Direct Access to this location is not allowed.' ); }

class PlgKunenacbantispamkunenabot extends JPlugin
{

	/**
	 * Checks if CB and CB AntiSpam are installed and loaded
	 */
	private function loadAntiSpam(): ?PluginTable
	{
		global $_PLUGINS;

		static $plugin		=	null;

		if ( $plugin === null ) {
			if ( ( ! file_exists( JPATH_SITE . '/libraries/CBLib/CBLib/Core/CBLib.php' ) ) || ( ! file_exists( JPATH_ADMINISTRATOR . '/components/com_comprofiler/plugin.foundation.php' ) ) ) {
				$plugin		=	false;

				return null;
			}

			include_once( JPATH_ADMINISTRATOR . '/components/com_comprofiler/plugin.foundation.php' );

			cbimport( 'cb.html' );
			cbimport( 'language.front' );

			$_PLUGINS->loadPluginGroup( 'user' );

			$plugin			=	$_PLUGINS->getLoadedPlugin( 'user', 'cbantispam' );

			outputCbJs();
			outputCbTemplate();
		}

		return $plugin;
	}

	/**
	 * @param string              $context
	 * @param TableKunenaMessages $table
	 * @param bool                $isNew
	 * @return bool
	 */
	public function onKunenaBeforeSave( $context, $table, $isNew ): bool
	{
		if ( $context !== 'com_kunena.KunenaForumMessage' ) {
			return true;
		}

		$userId				=	(int) $table->userid;

		if ( $this->params->get( 'cbantispam_autoblock', false ) ) {
			if ( ! $this->loadAntiSpam() ) {
				return true;
			}

			$blocked		=	CBAntiSpam::getBlock( $userId );

			if ( $blocked ) {
				$table->setError( $blocked->renderBlockReason() );

				return false;
			}
		}

		if ( ! $this->params->get( 'cbantispam_links', false ) ) {
			return true;
		}

		$excludeCategories	=	$this->params->get( 'cbantispam_exclude_categories', '' );

		if ( $excludeCategories && in_array( (string) $table->catid, explode( ',', $excludeCategories ), true ) ) {
			return true;
		}

		$excludeUsers		=	$this->params->get( 'cbantispam_exclude_users', '' );

		if ( $excludeUsers && in_array( (string) $table->userid, explode( ',', $excludeUsers ), true ) ) {
			return true;
		}

		$excludeGroups		=	$this->params->get( 'cbantispam_exclude_usergroups', [] );

		if ( $excludeGroups && is_array( $excludeGroups ) ) {
			$userGroups		=	$this->getJoomlaUser( $userId )->getAuthorisedGroups();

			foreach ( $excludeGroups as $excludeGroup ) {
				if ( ! in_array( $excludeGroup, $userGroups, true ) ) {
					continue;
				}

				return true;
			}
		}

		$excludeAccess		=	$this->params->get( 'cbantispam_exclude_accesslevels', [] );

		if ( $excludeAccess && is_array( $excludeAccess ) ) {
			$userAccess		=	$this->getJoomlaUser( $userId )->getAuthorisedViewLevels();

			foreach ( $excludeAccess as $excludeAccessLevel ) {
				if ( ! in_array( $excludeAccessLevel, $userAccess, true ) ) {
					continue;
				}

				return true;
			}
		}

		$excludePlans		=	$this->params->get( 'cbantispam_exclude_subscriptions_plans', [] );

		if ( $excludePlans && is_array( $excludePlans ) ) {
			if ( ! $this->loadAntiSpam() ) {
				return true;
			}

			$active			=	false;

			foreach ( $excludePlans as $excludePlan ) {
				if ( $active ) {
					break;
				}

				$active		=	$this->hasActiveSubscription( $userId, $excludePlan );
			}

			switch ( (int) $this->params->get( 'cbantispam_exclude_subscriptions', 1 ) ) {
				case 2: // Exclude All Except Selected Plans
					if ( ! $active ) {
						return true;
					}
					break;
				case 1: // Exclude Selected Plans
				default:
					if ( $active ) {
						return true;
					}
					break;
			}
		}

		$postLimit			=	(int) $this->params->get( 'cbantispam_links_posts', 10 );

		if ( ! $postLimit ) {
			$postLimit		=	10;
		}

		$kunenaUser			=	$this->getKunenaUser( $userId );

		if ( ! $kunenaUser ) {
			return true;
		}

		if ( (int) $kunenaUser->posts >= $postLimit ) {
			return true;
		}

		if ( ! $this->loadAntiSpam() ) {
			return true;
		}

		$flagged			=	$this->findUrls( (string) $table->message );

		if ( ! $flagged ) {
			return true;
		}

		$denyMessage		=	$this->params->get( 'cbantispam_links_denied', 'ANTISPAM_KUNENA_DENY' );

		if ( $denyMessage === 'ANTISPAM_KUNENA_DENY' ) {
			$denyMessage	=	CBTxt::T( 'ANTISPAM_KUNENA_DENY', 'You do not have permission to post links at this time. Please include text only for your first few posts. Thank you for your understanding.' );
		} else {
			$denyMessage	=	CBTxt::T( $denyMessage );
		}

		$table->setError( $denyMessage );
		$this->blockUser( $userId );
		$this->sendNotification( $table );

		return false;
	}

	/**
	 * @param string $message
	 * @return string[]
	 */
	private function findUrls( string $message ): array
	{
		$flagged		=	[];

		// First match BBCode attempts as they're the easiest to detect a URL in
		if ( preg_match_all( '%(?:\[img=(.*?)].*?\[/img])|(?:\[img](.*?)\[/img])|(?:\[url=(.*?)].*?\[/url])|(?:\[url](.*?)\[/url])|(?:\[video=(.*?)].*?\[/video])|(?:\[video](.*?)\[/video])%i', $message, $maches, PREG_SET_ORDER ) ) {
			foreach ( $maches as $foundURL ) {
				$matchedUrl	=	$this->getMatchingURL( $foundURL );

				if ( in_array( $matchedUrl, $flagged, true ) ) {
					continue;
				}

				if ( $this->checkURLWhiteListed( $matchedUrl ) ) {
					continue;
				}

				$flagged[]	=	$matchedUrl;
			}
		}

		// Now try to find flat URLs that might convert to clickable links by Kunena
		switch ( $this->params->get( 'cbantispam_links_regexp', 'kunena' ) ) {
			case 'cb':
				if ( preg_match_all( '/(\b(?:(?:https?|ftp|file):\/\/|www\.|ftp\.)[-A-Z0-9+&@#\/%=~_|$?!:,.]*[A-Z0-9+&@#\/%=~_|$])|(\b[A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z]{2,}\b)/i', $message, $maches, PREG_SET_ORDER ) ) {
					foreach ( $maches as $foundURL ) {
						$matchedUrl	=	$foundURL[0];

						if ( in_array( $matchedUrl, $flagged, true ) ) {
							continue;
						}

						if ( $this->checkURLWhiteListed( $matchedUrl ) ) {
							continue;
						}

						$flagged[]	=	$matchedUrl;
					}
				}
				break;
			case 'custom':
				if ( preg_match_all( $this->params->get( 'cbantispam_links_regexp_custom', '' ), $message, $maches, PREG_SET_ORDER ) ) {
					foreach ( $maches as $foundURL ) {
						$matchedUrl	=	$this->getMatchingURL( $foundURL );

						if ( in_array( $matchedUrl, $flagged, true ) ) {
							continue;
						}

						if ( $this->checkURLWhiteListed( $matchedUrl ) ) {
							continue;
						}

						$flagged[]	=	$matchedUrl;
					}
				}
				break;
			case 'kunena':
			default:
				if ( preg_match_all( '/(https?:\/\/)?([\da-z.-]+)\.([a-z.]{2,6})([\/\w .-]*)(#?[\w .-]*)(\??[\w .-]*)(=?[\w .-]*)/i', $message, $maches, PREG_SET_ORDER ) ) {
					foreach ( $maches as $foundURL ) {
						$matchedUrl	=	$foundURL[0];

						if ( in_array( $matchedUrl, $flagged, true ) ) {
							continue;
						}

						if ( $this->checkURLWhiteListed( $matchedUrl ) ) {
							continue;
						}

						$flagged[]	=	$matchedUrl;
					}
				}
				break;
		}

		return $flagged;
	}

	/**
	 * @param int $userId
	 * @return void
	 */
	private function blockUser( int $userId ): void
	{
		if ( ! $this->params->get( 'cbantispam_autoblock', false ) ) {
			return;
		}

		$blockMethod		=	(int) $this->params->get( 'cbantispam_autoblock_method', 2 );

		if ( ! $blockMethod ) {
			return;
		}

		$attempt			=	CBAntiSpam::logAttempt( 'kunena', $this->params->get( 'cbantispam_autoblock_timeframe', '-1 HOUR' ) );
		$attemptLimit		=	(int) $this->params->get( 'cbantispam_autoblock_count', 5 );

		if ( ! $attemptLimit ) {
			$attemptLimit	=	5;
		}

		if ( $attempt->getCount() < $attemptLimit ) {
			return;
		}

		$block				=	new BlockTable();

		switch ( $blockMethod ) {
			case 1: // Block IP Address
				$block->setType( 'ip' );
				$block->setValue( Application::Input()->getRequestIP() );
				break;
			case 3: // Block User
				$block->setType( 'account' );
				$block->setValue( $userId );
				break;
			case 2: // Block Account
			default:
				$block->setType( 'user' );
				$block->setValue( $userId );
				break;
		}

		$block->setDuration( $this->params->get( 'cbantispam_autoblock_dur', '+1 HOUR' ) );
		$block->setReason( $this->params->get( 'cbantispam_autoblock_reason', 'Too many failed post attempts.' ) );

		$block->store();
	}

	/**
	 * @param TableKunenaMessages $table
	 * @return void
	 */
	private function sendNotification( $table ): void
	{
		global $_CB_framework, $_CB_database;

		if ( ! $this->params->get( 'cbantispam_links_notify', false ) ) {
			return;
		}

		$userId			=	(int) $table->userid;
		$message		=	(string) $table->message;
		$topicId		=	(int) $table->parent;

		$cbUser			=	CBuser::getInstance( $userId, false );

		$extras			=	[	'topic_id'			=>	$topicId,
								'topic_subject'		=>	htmlspecialchars( (string) $table->subject ),
								'topic_url'			=>	$this->getKunenaTopicURL( $topicId ),
								'post_message'		=>	htmlspecialchars( $message ),
								'ip_address'		=>	(string) Application::Input()->getRequestIP(),
								'block_url'			=>	$_CB_framework->pluginClassUrl( 'cbantispam', true, [ 'action' => 'block', 'func' => 'account', 'user' => $userId ] ),
								'reset_url'			=>	$_CB_framework->pluginClassUrl( 'cbantispam', true, [ 'action' => 'block', 'func' => 'reset', 'user' => $userId ] ),
								'user_url'			=>	$_CB_framework->viewUrl( 'userprofile', true, [ 'user' => $userId ] ),
							];

		$subject		=	$this->params->get( 'cbantispam_links_notify_subject', 'ANTISPAM_KUNENA_NOTIFY_SUB' );

		if ( $subject === 'ANTISPAM_KUNENA_NOTIFY_SUB' ) {
			$subject	=	CBTxt::T( 'ANTISPAM_KUNENA_NOTIFY_SUB', 'Forum Post Blocked' );
		} else {
			$subject	=	CBTxt::T( $subject );
		}

		$message		=	$this->params->get( 'cbantispam_links_notify_message', 'ANTISPAM_KUNENA_NOTIFY_MSG' );

		if ( $message === 'ANTISPAM_KUNENA_NOTIFY_MSG' ) {
			$message	=	CBTxt::T( 'ANTISPAM_KUNENA_NOTIFY_MSG', 'A forum post from <a href="[user_url]">[formatname]</a> ([user_id]) ([ip_address]) has been blocked with the following message[cb:if topic_id>"0"] in topic <a href="[topic_url]">[topic_subject]</a>[/cb:if]:<br/><br/><hr/><p>[post_message]</p><hr/><br/><br/><a href="[block_url]">Click here to block [username]</a><br/><a href="[reset_url]">Click here to force a password reset for [username]</a>' );
		} else {
			$message	=	CBTxt::T( $message );
		}

		$subject		=	$cbUser->replaceUserVars( $subject, true, false, $extras, false );
		$message		=	$cbUser->replaceUserVars( $message, false, false, $extras, false );

		$notification	=	new cbNotification();
		$fromName		=	$this->params->get( 'cbantispam_links_notify_from_name', '' );
		$fromAddress	=	$this->params->get( 'cbantispam_links_notify_from_address', '' );
		$recipients		=	[];

		switch ( $this->params->get( 'cbantispam_links_notify_to', 'moderators' ) ) {
			case 'specific_emails':
				$emails			=	$this->params->get( 'cbantispam_links_notify_emails', '' );

				if ( ! $emails ) {
					break;
				}

				$recipients		=	explode( ',', $emails );
				break;
			case 'specific_users':
				$users			=	$this->params->get( 'cbantispam_links_notify_users', '' );

				if ( ! $users ) {
					break;
				}

				$query			=	"SELECT DISTINCT " . $_CB_database->NameQuote( 'email' )
								.	"\n FROM " . $_CB_database->NameQuote( '#__users' )
								.	"\n WHERE " . $_CB_database->NameQuote( 'id' ) . " IN " . $_CB_database->safeArrayOfIntegers( explode( ',', $users ) );

				$_CB_database->setQuery( $query );
				$recipients		=	$_CB_database->loadResultArray();
				break;
			case 'moderators':
			default:
				$query			=	'SELECT DISTINCT u.' . $_CB_database->NameQuote( 'email' )
								.	"\n FROM " . $_CB_database->NameQuote( '#__users' ) . " AS u"
								.	"\n INNER JOIN " . $_CB_database->NameQuote( '#__comprofiler' ) . " AS c"
								.	' ON c.' . $_CB_database->NameQuote( 'id' ) . ' = u.' . $_CB_database->NameQuote( 'id' )
								.	"\n INNER JOIN " . $_CB_database->NameQuote( '#__user_usergroup_map' ) . " AS g"
								.	' ON g.' . $_CB_database->NameQuote( 'user_id' ) . ' = c.' . $_CB_database->NameQuote( 'id' )
								.	"\n WHERE g." . $_CB_database->NameQuote( 'group_id' ) . " IN " . $_CB_database->safeArrayOfIntegers( Application::CmsPermissions()->getGroupsOfViewAccessLevel( Application::Config()->getInt( 'moderator_viewaccesslevel', 3 ), true ) )
								.	"\n AND u." . $_CB_database->NameQuote( 'block' ) . " = 0"
								.	"\n AND c." . $_CB_database->NameQuote( 'confirmed' ) . " = 1"
								.	"\n AND c." . $_CB_database->NameQuote( 'approved' ) . " = 1";

				$_CB_database->setQuery( $query );
				$recipients		=	$_CB_database->loadResultArray();
				break;
		}

		foreach ( $recipients as $recipient ) {
			if ( ! $recipient ) {
				continue;
			}

			$notification->sendFromSystem( $recipient, $subject, $message, false, 1, null, null, null, [], false, $fromName, $fromAddress );
		}
	}

	/**
	 * @param int $userId
	 * @return User
	 */
	private function getJoomlaUser( int $userId ): User
	{
		static $cache		=	[];

		if ( array_key_exists( $userId, $cache ) ) {
			return $cache[$userId];
		}

		$user			=	JFactory::getUser( $userId );

		$cache[$userId]	=	$user;

		return $user;
	}

	/**
	 * @param int $userId
	 * @return null|KunenaUser|Kunena6User
	 */
	private function getKunenaUser( int $userId )
	{
		static $cache		=	[];

		if ( array_key_exists( $userId, $cache ) ) {
			return $cache[$userId];
		}

		$user				=	null;

		if ( cbforumsClass::getModel()->version >= 6 ) {
			if ( class_exists( '\Kunena\Forum\Libraries\User\KunenaUser' ) ) {
				$user		=	Kunena6User::getInstance( $userId );
			}
		} elseif ( class_exists( 'KunenaUser' ) ) {
			$user			=	KunenaUser::getInstance( $userId );
		}

		$cache[$userId]		=	$user;

		return $user;
	}

	/**
	 * @param int $topicId
	 * @return string
	 */
	private function getKunenaTopicURL( int $topicId ): string
	{
		if ( ! $topicId ) {
			return '';
		}

		$topicUrl				=	'';

		if ( cbforumsClass::getModel()->version >= 6 ) {
			if ( class_exists( '\Kunena\Forum\Libraries\Route\KunenaRoute' ) ) {
				$topicUrl		=	KunenaTopic::getInstance( $topicId )->getUri();

				if ( ! $topicUrl ) {
					$topicUrl	=	KunenaMessage::getInstance( $topicId )->getUri();
				}
			}
		} elseif ( class_exists( 'KunenaRoute' ) ) {
			$topicUrl			=	KunenaForumTopic::getInstance( $topicId )->getUri();

			if ( ! $topicUrl ) {
				$topicUrl		=	KunenaForumMessage::getInstance( $topicId )->getUri();
			}
		}

		if ( $topicUrl ) {
			$topicUrl			=	Route::_( $topicUrl, true, Route::TLS_IGNORE, true );
		}

		return $topicUrl;
	}

	/**
	 * @param string[] $matches
	 * @return string
	 */
	private function getMatchingURL( array $matches ): string
	{
		for ( $i = 1; $i <= 10; $i++ ) {
			$url	=	( $matches[$i] ?? '' );

			if ( ! $url ) {
				continue;
			}

			return $url;
		}

		return '';
	}

	/**
	 * @param string $url
	 * @return bool
	 */
	private function checkURLWhiteListed( string $url ): bool
	{
		if ( ! $url ) {
			return false;
		}

		if ( $this->params->get( 'cbantispam_exclude_internal', true ) && JUri::isInternal( $url ) ) {
			return true;
		}

		if ( in_array( $url, $this->getSubFormList( 'cbantispam_exclude_urls' ), true ) ) {
			return true;
		}

		$domain		=	preg_replace( '/^(?:(?:\w+\.)*)?(\w+)\..+$/', '\1', parse_url( $url, PHP_URL_HOST ) );

		if ( in_array( $domain, $this->getSubFormList( 'cbantispam_exclude_domains' ), true ) ) {
			return true;
		}

		return false;
	}

	/**
	 * @param string $name
	 * @return array
	 */
	private function getSubFormList( string $name ): array
	{
		static $cache		=	[];

		if ( array_key_exists( $name, $cache ) ) {
			return $cache[$name];
		}

		$data				=	$this->params->get( $name );

		if ( ! $data ) {
			$cache[$name]	=	[];

			return [];
		}

		$list				=	[];

		foreach ( $data as $row ) {
			$item			=	( array_values( (array) $row )[0] ?? '' );

			if ( ! $item ) {
				continue;
			}

			$list[]			=	$item;
		}

		$cache[$name]		=	$list;

		return $list;
	}

	/**
	 * @param int $userId
	 * @param int $planId
	 * @return bool
	 */
	private function hasActiveSubscription( int $userId, int $planId ): bool
	{
		static $cache				=	[];

		if ( array_key_exists( $userId, $cache ) ) {
			return in_array( $planId, $cache[$userId], true );
		}

		$cache[$userId]				=	[];

		foreach ( cbpaidSomethingMgr::getAllSomethingOfUser( CBuser::getUserDataInstance( $userId ) ) as $subscriptions ) {
			/** @var cbpaidSomething[] $subscriptions */
			foreach ( array_keys( $subscriptions ) as $k ) {
				$cache[$userId][]	=	$subscriptions[$k]->getPlan()->getInt( 'id', 0 );
			}
		}

		return in_array( $planId, $cache[$userId], true );
	}
}