<?php
/**
 * Community Builder (TM)
 * @version $Id: $
 * @package CommunityBuilder
 * @copyright (C) 2004-2022 www.joomlapolis.com / Lightning MultiCom SA - and its licensors, all rights reserved
 * @license http://www.gnu.org/licenses/old-licenses/gpl-2.0.html GNU/GPL version 2
 */

use CB\Plugin\AutoActions\Table\AutoActionTable;
use CB\Plugin\AutoActions\Table\BatchTable;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\Component\Scheduler\Administrator\Task\Status;
use Joomla\Component\Scheduler\Administrator\Event\ExecuteTaskEvent;
use Joomla\Component\Scheduler\Administrator\Traits\TaskPluginTrait;
use Joomla\Event\SubscriberInterface;

defined( '_JEXEC' ) or die;

class PlgTaskCbautoactionstaskbot extends CMSPlugin implements SubscriberInterface
{
	use TaskPluginTrait;

	/**
	 * @var string[]
	 */
	protected const TASKS_MAP	=	[
		'autoaction.run'		=>	[
			'langConstPrefix'	=>	'PLG_TASK_CBAUTOACTIONSTASKBOT_TASK_RUN',
			'form'				=>	'autoaction',
			'method'			=>	'runAutoAction',
		],
		'autoaction.trigger'	=>	[
			'langConstPrefix'	=>	'PLG_TASK_CBAUTOACTIONSTASKBOT_TASK_TRIGGER',
			'form'				=>	'autoaction',
			'method'			=>	'triggerAutoAction',
		],
		'autoaction.execute'	=>	[
			'langConstPrefix'	=>	'PLG_TASK_CBAUTOACTIONSTASKBOT_TASK_EXECUTE',
			'form'				=>	'autoaction',
			'method'			=>	'executeAutoAction',
		],
		'autoaction.batch'	=>	[
			'langConstPrefix'	=>	'PLG_TASK_CBAUTOACTIONSTASKBOT_TASK_BATCH',
			'form'				=>	'batch',
			'method'			=>	'executeBatch',
		],
	];

	/**
	 * @var bool
	 * @since 4.1.0
	 */
	protected $autoloadLanguage	=	true;

	/**
	 * @inheritDoc
	 *
	 * @return string[]
	 */
	public static function getSubscribedEvents(): array
	{
		return [
			'onTaskOptionsList'		=>	'advertiseRoutines',
			'onExecuteTask'			=>	'standardRoutineHandler',
			'onContentPrepareForm'	=>	'enhanceTaskItemForm',
		];
	}

	/**
	 * @return bool
	 * @throws Exception
	 */
	private function loadAutoActions(): bool
	{
		global $_PLUGINS;

		static $cache		=	null;

		if ( $cache === null ) {
			if ( ( ! file_exists( JPATH_SITE . '/libraries/CBLib/CBLib/Core/CBLib.php' ) ) || ( ! file_exists( JPATH_ADMINISTRATOR . '/components/com_comprofiler/plugin.foundation.php' ) ) ) {
				$this->logTask( Text::_( 'PLG_TASK_CBAUTOACTIONSTASKBOT_TASK_CB_NA' ), 'warning' );

				$cache		=	false;

				return false;
			}

			include_once( JPATH_ADMINISTRATOR . '/components/com_comprofiler/plugin.foundation.php' );

			cbimport( 'cb.plugins' );

			if ( ! $_PLUGINS->loadPluginGroup( 'user', 'cbautoactions' ) ) {
				$this->logTask( Text::_( 'PLG_TASK_CBAUTOACTIONSTASKBOT_TASK_AUTOACTIONS_NA' ), 'warning' );

				$cache		=	false;

				return false;
			}

			$cache			=	true;
		}

		return $cache;
	}

	/**
	 * @param int $id
	 * @return null|AutoActionTable
	 * @throws Exception
	 */
	private function getAutoAction( int $id ): ?AutoActionTable
	{
		if ( ! $this->loadAutoActions() ) {
			return null;
		}

		$action	=	new AutoActionTable();

		if ( ! $action->load( $id ) ) {
			$this->logTask( Text::sprintf( 'PLG_TASK_CBAUTOACTIONSTASKBOT_TASK_ACTION_NA', $id ), 'warning' );

			return null;
		}

		if ( ! $action->getId() ) {
			$this->logTask( Text::sprintf( 'PLG_TASK_CBAUTOACTIONSTASKBOT_TASK_ACTION_NA', $id ), 'warning' );

			return null;
		}

		return $action;
	}

	/**
	 * @param int $id
	 * @return null|BatchTable
	 * @throws Exception
	 */
	private function getBatchAction( int $id ): ?BatchTable
	{
		if ( ! $this->loadAutoActions() ) {
			return null;
		}

		$batch	=	new BatchTable();

		if ( ! $batch->load( $id ) ) {
			$this->logTask( Text::sprintf( 'PLG_TASK_CBAUTOACTIONSTASKBOT_TASK_BATCH_NA', $id ), 'warning' );

			return null;
		}

		if ( ! $batch->getId() ) {
			$this->logTask( Text::sprintf( 'PLG_TASK_CBAUTOACTIONSTASKBOT_TASK_BATCH_NA', $id ), 'warning' );

			return null;
		}

		return $batch;
	}

	/**
	 * @param ExecuteTaskEvent $event
	 * @param string           $method
	 * @return int
	 * @throws Exception
	 */
	private function runAutoActionMethod( ExecuteTaskEvent $event, string $method ): int
	{
		$params						=	$event->getArgument( 'params' );
		$ids						=	( $params->action_ids ?? [] );

		if ( ! is_array( $ids ) ) {
			$ids					=	explode( ',', $ids );
		}

		if ( ! $ids ) {
			$this->logTask( Text::_( 'PLG_TASK_CBAUTOACTIONSTASKBOT_TASK_ACTIONS_NA' ), 'warning' );

			return Status::NO_RUN;
		}

		foreach ( $ids as $id ) {
			$action					=	$this->getAutoAction( (int) $id );

			if ( ! $action ) {
				continue;
			}

			switch ( $method ) {
				case 'run':
					$variables		=	[ 'trigger' => 'joomla_taskRunAction', 'var1' => $event ];

					$action->runAction( $variables );
					break;
				case 'execute':
					$action->executeAction( null, [ 'trigger' => 'joomla_taskExecuteAction', 'var1' => $event ] );
					break;
				case 'trigger':
				default:
					$action->triggerAction( null, [ 'trigger' => 'joomla_taskTriggerAction', 'var1' => $event ] );
					break;
			}

			if ( $action->getError() ) {
				$this->logTask( Text::sprintf( 'PLG_TASK_CBAUTOACTIONSTASKBOT_TASK_ACTION_ERROR', $action->getError() ), 'error' );

				return Status::NO_RUN;
			}
		}

		return Status::OK;
	}

	/**
	 * @param ExecuteTaskEvent $event
	 * @return int
	 * @throws Exception
	 */
	protected function runAutoAction( ExecuteTaskEvent $event ): int
	{
		return $this->runAutoActionMethod( $event, 'run' );
	}

	/**
	 * @param ExecuteTaskEvent $event
	 * @return int
	 * @throws Exception
	 */
	protected function triggerAutoAction( ExecuteTaskEvent $event ): int
	{
		return $this->runAutoActionMethod( $event, 'trigger' );
	}

	/**
	 * @param ExecuteTaskEvent $event
	 * @return int
	 * @throws Exception
	 */
	protected function executeAutoAction( ExecuteTaskEvent $event ): int
	{
		return $this->runAutoActionMethod( $event, 'execute' );
	}

	/**
	 * @param ExecuteTaskEvent $event
	 * @return int
	 * @throws Exception
	 */
	protected function executeBatch( ExecuteTaskEvent $event ): int
	{
		$params						=	$event->getArgument( 'params' );
		$ids						=	( $params->batch_ids ?? [] );

		if ( ! is_array( $ids ) ) {
			$ids					=	explode( ',', $ids );
		}

		if ( ! $ids ) {
			$this->logTask( Text::_( 'PLG_TASK_CBAUTOACTIONSTASKBOT_TASK_BATCHS_NA' ), 'warning' );

			return Status::NO_RUN;
		}

		foreach ( $ids as $id ) {
			$batch					=	$this->getBatchAction( (int) $id );

			if ( ! $batch ) {
				continue;
			}

			$batch->runBatch( null, [ 'trigger' => 'joomla_taskBatchAction', 'var1' => $event, 'var2' => $batch ] );

			if ( $batch->getError() ) {
				$this->logTask( Text::sprintf( 'PLG_TASK_CBAUTOACTIONSTASKBOT_TASK_BATCH_ERROR', $batch->getError() ), 'error' );

				return Status::NO_RUN;
			}
		}

		return Status::OK;
	}
}