<?php
/**
* Community Builder (TM)
* @version $Id: $
* @package CommunityBuilder
* @copyright (C) 2004-2022 www.joomlapolis.com / Lightning MultiCom SA - and its licensors, all rights reserved
* @license http://www.gnu.org/licenses/old-licenses/gpl-2.0.html GNU/GPL version 2
*/

use CBLib\Language\CBTxt;
use CB\Database\Table\UserTable;
use CB\Database\Table\TabTable;
use CBLib\Application\Application;
use CB\Plugin\AntiSpam\Captcha;
use CB\Plugin\AntiSpam\CBAntiSpam;
use CB\Plugin\AntiSpam\Table\BlockTable;
use CB\Plugin\AntiSpam\Table\WhitelistTable;
use CB\Plugin\AntiSpam\Table\AttemptTable;
use CB\Plugin\AntiSpam\Table\LogTable;

if ( ! ( defined( '_VALID_CB' ) || defined( '_JEXEC' ) || defined( '_VALID_MOS' ) ) ) { die( 'Direct Access to this location is not allowed.' ); }

/** @noinspection AutoloadingIssuesInspection */
class CBplug_cbantispam extends cbPluginHandler
{

	/**
	 * @param null|TabTable  $tab
	 * @param null|UserTable $user
	 * @param null|int       $ui
	 * @param null|array     $postdata
	 * @throws Exception
	 */
	public function getCBpluginComponent( ?TabTable $tab, ?UserTable $user, ?int $ui, ?array $postdata ): void
	{
		$format							=	$this->getInput()->getString( 'format', '' );

		if ( $format !== 'raw' ) {
			outputCbJs();
			outputCbTemplate();
		}

		$action							=	$this->getInput()->getString( 'action', '' );
		$function						=	$this->getInput()->getString( 'func', '' );

		if ( $action === 'captcha' ) {
			$id							=	$this->getInput()->getString( 'id', '' );
		} else {
			$id							=	$this->getInput()->getInt( 'id', 0 );
		}

		if ( $format !== 'raw' ) {
			ob_start();
		}

		switch ( $action ) {
			case 'prune':
				switch ( $function ) {
					case 'block':
					case 'log':
					case 'attempts':
						$this->pruneItems( $function, false );
						break;
					case 'all':
					default:
						$this->pruneAll();
						break;
				}
				break;
			case 'captcha':
				switch ( $function ) {
					case 'image':
						$this->captchaImage( $id );
						break;
					case 'audio':
						$this->captchaAudio( $id );
						break;
					case 'load':
						$this->captchaLoad( $id );
						break;
				}
				break;
			case 'block':
				if ( ( ! $this->params->getBool( 'general_block', true ) ) || ( ! Application::MyUser()->isGlobalModerator() ) ) {
					CBAntiSpam::returnRedirect( 'index.php', CBTxt::T( 'Not authorized.' ), 'error' );
				}

				switch ( $function ) {
					case 'account':
						$this->showBlock( $id, 'account' );
						break;
					case 'user':
						$this->showBlock( $id, 'user' );
						break;
					case 'reset':
						$this->showBlock( $id, 'reset' );
						break;
					case 'ip':
						$this->showBlock( $id, 'ip' );
						break;
					case 'ip_range':
						$this->showBlock( $id, 'ip_range' );
						break;
					case 'email':
						$this->showBlock( $id, 'email' );
						break;
					case 'domain':
						$this->showBlock( $id, 'domain' );
						break;
					case 'edit':
						$this->showBlock( $id );
						break;
					case 'save':
						cbSpoofCheck( 'plugin' );
						$this->saveBlock( $id );
						break;
					case 'delete':
						$this->deleteBlock( $id );
						break;
					case 'new':
					default:
						$this->showBlock();
						break;
				}
				break;
			case 'whitelist':
				if ( ( ! $this->params->getBool( 'general_whitelist', true ) ) || ( ! Application::MyUser()->isGlobalModerator() ) ) {
					CBAntiSpam::returnRedirect( 'index.php', CBTxt::T( 'Not authorized.' ), 'error' );
				}

				switch ( $function ) {
					case 'user':
						$this->showWhitelist( $id, 'user' );
						break;
					case 'ip':
						$this->showWhitelist( $id, 'ip' );
						break;
					case 'email':
						$this->showWhitelist( $id, 'email' );
						break;
					case 'domain':
						$this->showWhitelist( $id, 'domain' );
						break;
					case 'edit':
						$this->showWhitelist( $id );
						break;
					case 'save':
						cbSpoofCheck( 'plugin' );
						$this->saveWhitelist( $id );
						break;
					case 'delete':
						$this->deleteWhitelist( $id );
						break;
					case 'new':
					default:
						$this->showWhitelist();
						break;
				}
				break;
			case 'attempt':
				if ( ( ! $this->params->getBool( 'general_attempts', true ) ) || ( ! Application::MyUser()->isGlobalModerator() ) ) {
					CBAntiSpam::returnRedirect( 'index.php', CBTxt::T( 'Not authorized.' ), 'error' );
				}

				switch ( $function ) {
					case 'delete':
						$this->deleteAttempt( $id );
						break;
				}
				break;
			case 'log':
				if ( ( ! $this->params->getBool( 'general_log', true ) ) || ( ! Application::MyUser()->isGlobalModerator() ) ) {
					CBAntiSpam::returnRedirect( 'index.php', CBTxt::T( 'Not authorized.' ), 'error' );
				}

				switch ( $function ) {
					case 'delete':
						$this->deleteLog( $id );
						break;
				}
				break;
		}

		if ( $format !== 'raw' ) {
			$html						=	ob_get_clean();

			$class						=	$this->params->getString( 'general_class', '' );

			$return						=	'<div class="cbAntiSpam' . ( $class ? ' ' . htmlspecialchars( $class ) : null ) . '">'
										.		$html
										.	'</div>';

			echo $return;
		}
	}

	/**
	 * Prunes old items of $type
	 *
	 * @param string $type
	 * @param bool   $force
	 */
	private function pruneItems( string $type, bool $force ): void
	{
		global $_CB_framework, $_CB_database;

		$clean					=	false;

		if ( $force ) {
			$clean				=	true;
		} else {
			$token				=	$this->getInput()->getString( 'token', '' );

			if ( $token === md5( $_CB_framework->getCfg( 'secret' ) ) ) {
				$clean			=	true;
			}
		}

		if ( $clean ) {
			switch ( $type ) {
				case 'block':
					$query		=	'SELECT *'
								.	"\n FROM " . $_CB_database->NameQuote( '#__comprofiler_plugin_antispam_block' );
					$_CB_database->setQuery( $query );
					$blocks		=	$_CB_database->loadObjectList( 'id', BlockTable::class, [ $_CB_database ] );

					/** @var BlockTable[] $blocks */
					foreach ( $blocks as $block ) {
						if ( $block->isExpired() ) {
							$block->delete();
						}
					}
					break;
				case 'log':
					$query		=	'SELECT *'
								.	"\n FROM " . $_CB_database->NameQuote( '#__comprofiler_plugin_antispam_log' )
								.	"\n WHERE " . $_CB_database->NameQuote( 'date' ) . " <= " . $_CB_database->Quote( Application::Date( 'now', 'UTC' )->modify( strtoupper( $this->params->getString( 'cleanup_log_dur', '-1 YEAR' ) ) )->format( 'Y-m-d H:i:s' ) );
					$_CB_database->setQuery( $query );
					$logs		=	$_CB_database->loadObjectList( 'id', LogTable::class, [ $_CB_database ] );

					/** @var LogTable[] $logs */
					foreach ( $logs as $log ) {
						$log->delete();
					}
					break;
				case 'attempts':
					$query		=	'SELECT *'
								.	"\n FROM " . $_CB_database->NameQuote( '#__comprofiler_plugin_antispam_attempts' )
								.	"\n WHERE " . $_CB_database->NameQuote( 'date' ) . " <= " . $_CB_database->Quote( Application::Date( 'now', 'UTC' )->modify( strtoupper( $this->params->getString( 'cleanup_attempts_dur', '-1 YEAR' ) ) )->format( 'Y-m-d H:i:s' ) );
					$_CB_database->setQuery( $query );
					$attempts	=	$_CB_database->loadObjectList( 'id', AttemptTable::class, [ $_CB_database ] );

					/** @var AttemptTable[] $attempts */
					foreach ( $attempts as $attempt ) {
						$attempt->delete();
					}
					break;
			}

			if ( ! $force ) {
				header( 'HTTP/1.0 200 OK' );
				exit();
			}
		} elseif ( ! $force ) {
			header( 'HTTP/1.0 403 Forbidden' );
			exit();
		}
	}

	/**
	 * Prunes all old items
	 */
	private function pruneAll(): void
	{
		global $_CB_framework;

		if ( $this->getInput()->getString( 'token', '' ) !== md5( $_CB_framework->getCfg( 'secret' ) ) ) {
			header( 'HTTP/1.0 403 Forbidden' );
			exit();
		}

		$this->pruneItems( 'block', true );
		$this->pruneItems( 'log', true );
		$this->pruneItems( 'attempts', true );

		header( 'HTTP/1.0 200 OK' );
		exit();
	}

	/**
	 * Generates a captcha image
	 *
	 * @param string $id
	 * @throws Exception
	 */
	private function captchaImage( string $id ): void
	{
		global $_CB_framework, $_PLUGINS;

		if ( ! $id ) {
			header( 'HTTP/1.0 404 Not Found' );
			exit();
		}

		/** @noinspection PhpStatementHasEmptyBodyInspection */
		/** @noinspection LoopWhichDoesNotLoopInspection */
		/** @noinspection MissingOrEmptyGroupStatementInspection */
		while ( @ob_end_clean() ) {}

		$absPath						=	$_PLUGINS->getPluginPath( $this->getPluginId() );
		$captcha						=	new Captcha();

		$captcha->load( $id );

		if ( $this->getInput()->getString( 'ver', '' ) ) {
			$captcha->reset();
			$captcha->generateCode();
		}

		$code							=	$captcha->getCode();

		if ( ! $code ) {
			exit( CBTxt::T( 'failed to generate captcha code' ) );
		}

		$params							=	CBAntiSpam::getGlobalParams();
		$height							=	$params->getInt( 'captcha_internal_height', 60 );

		if ( ! $height ) {
			$height						=	50;
		}

		$bgColor						=	$params->getString( 'captcha_internal_bg_color', '255,255,255' );

		if ( ! $bgColor ) {
			$bgColor					=	'255,255,255';
		}

		$bgColors						=	explode( ',', $bgColor );
		$imgBgColors					=	[];

		if ( $bgColors ) {
			for ( $i = 0; $i < 3; $i++ ) {
				if ( ! isset( $bgColors[$i] ) ) {
					continue;
				}

				$color					=	(int) trim( $bgColors[$i] );

				$imgBgColors[]			=	( $color > 255 ? 255 : max( $color, 0 ) );
			}
		}

		if ( ! $imgBgColors ) {
			$imgBgColors				=	[ 255, 255, 255 ];
		} elseif ( count( $imgBgColors ) < 3 ) {
			if ( count( $imgBgColors ) < 2 ) {
				$imgBgColors[]			=	255;
			}

			$imgBgColors[]				=	255;
		}

		$txtColor						=	$params->getString( 'captcha_internal_txt_color', '20,40,100' );

		if ( ! $txtColor ) {
			$txtColor					=	'20,40,100';
		}

		$txtColors						=	explode( ',', $txtColor );
		$imgTxtColors					=	[];

		if ( $txtColors ) {
			for ( $i = 0; $i < 3; $i++ ) {
				if ( ! isset( $txtColors[$i] ) ) {
					continue;
				}

				$color					=	(int) trim( $txtColors[$i] );

				$imgTxtColors[]			=	( $color > 255 ? 255 : max( $color, 0 ) );
			}
		}

		if ( ! $imgTxtColors ) {
			$imgTxtColors				=	[ 20, 40, 100 ];
		} elseif ( count( $imgTxtColors ) < 3 ) {
			if ( count( $imgTxtColors ) < 2 ) {
				$imgTxtColors[]			=	40;
			}

			$imgTxtColors[]				=	100;
		}

		$nBgColor						=	$params->getString( 'captcha_internal_bg_noise_color', '100,120,180' );

		if ( ! $nBgColor ) {
			$nBgColor					=	'100,120,180';
		}

		$nBgColors						=	explode( ',', $nBgColor );
		$imgNoiseBgColors				=	[];

		if ( $nBgColors ) {
			for ( $i = 0; $i < 3; $i++ ) {
				if ( ! isset( $nBgColors[$i] ) ) {
					continue;
				}

				$color					=	(int) trim( $nBgColors[$i] );

				$imgNoiseBgColors[]		=	( $color > 255 ? 255 : max( $color, 0 ) );
			}
		}

		if ( ! $imgNoiseBgColors ) {
			$imgNoiseBgColors			=	[ 100, 120, 180 ];
		} elseif ( count( $imgNoiseBgColors ) < 3 ) {
			if ( count( $imgNoiseBgColors ) < 2 ) {
				$imgNoiseBgColors[]		=	120;
			}

			$imgNoiseBgColors[]			=	180;
		}

		$nFgColor						=	$params->getString( 'captcha_internal_fg_noise_color', '100,120,120' );

		if ( ! $nFgColor ) {
			$nFgColor					=	'100,120,120';
		}

		$nFgColors						=	explode( ',', $nFgColor );
		$imgNoiseFgColors				=	[];

		if ( $nFgColors ) {
			for ( $i = 0; $i < 3; $i++ ) {
				if ( ! isset( $nFgColors[$i] ) ) {
					continue;
				}

				$color					=	(int) trim( $nFgColors[$i] );

				$imgNoiseFgColors[]		=	( $color > 255 ? 255 : max( $color, 0 ) );
			}
		}

		if ( ! $imgNoiseFgColors ) {
			$imgNoiseFgColors			=	[ 100, 120, 120 ];
		} elseif ( count( $imgNoiseFgColors ) < 3 ) {
			if ( count( $imgNoiseFgColors ) < 2 ) {
				$imgNoiseFgColors[]		=	120;
			}

			$imgNoiseFgColors[]			=	120;
		}

		$fonts							=	explode( '|*|', $params->getString( 'captcha_internal_font', 'EARWIGFA.ttf|*|monofont.ttf' ) );

		if ( ! $fonts ) {
			$fonts						=	[ 'EARWIGFA.ttf', 'monofont.ttf' ];
		}

		$imgFonts						=	[];

		foreach ( $fonts as $font ) {
			$imgFont					=	$absPath . '/fonts/' . $font;

			if ( ! file_exists( $imgFont ) ) {
				continue;
			}

			$imgFonts[]					=	$imgFont;
		}

		if ( ! $imgFonts ) {
			exit( CBTxt::T( 'failed to locate font file' ) );
		}

		$charRotation					=	$params->getInt( 'captcha_internal_rotation', 13 );

		if ( $charRotation < 0 ) {
			$charRotation				=	0;
		}

		$charOffset						=	$params->getInt( 'captcha_internal_offset', 3 );

		if ( $charOffset < 0 ) {
			$charOffset					=	0;
		}

		$charRange						=	$params->getInt( 'captcha_internal_range', 5 );

		if ( $charRange < 0 ) {
			$charRange					=	0;
		}

		$colorRange						=	$params->getInt( 'captcha_internal_color_range', 0 );

		if ( $colorRange < 0 ) {
			$colorRange					=	0;
		}

		$fontSize						=	( $height * 0.75 );
		$textBox						=	imagettfbbox( ( $charRange ? ( $fontSize + $charRange ) : $fontSize ), 0, $imgFonts[0], $code );

		if ( $textBox === false ) {
			exit( CBTxt::T( 'imagettfbbox failed to establish image size' ) );
		}

		$width							=	( $textBox[4] + 20 );
		$image							=	imagecreatetruecolor( $width, $height );

		if ( $image === false ) {
			exit( CBTxt::T( 'imagecreate failed to create new image' ) );
		}

		if ( $colorRange ) {
			$bgColorR					=	random_int( ( ( $imgBgColors[0] - $colorRange ) < 0 ? 0 : ( $imgBgColors[0] - $colorRange ) ), ( ( $imgBgColors[0] + $colorRange ) > 255 ? 255 : ( $imgBgColors[0] + $colorRange ) ) );
			$bgColorG					=	random_int( ( ( $imgBgColors[1] - $colorRange ) < 0 ? 0 : ( $imgBgColors[1] - $colorRange ) ), ( ( $imgBgColors[1] + $colorRange ) > 255 ? 255 : ( $imgBgColors[1] + $colorRange ) ) );
			$bgColorB					=	random_int( ( ( $imgBgColors[2] - $colorRange ) < 0 ? 0 : ( $imgBgColors[2] - $colorRange ) ), ( ( $imgBgColors[2] + $colorRange ) > 255 ? 255 : ( $imgBgColors[2] + $colorRange ) ) );

			$backgroundColor			=	imagecolorallocate( $image, $bgColorR, $bgColorG, $bgColorB );

			if ( $backgroundColor === false ) {
				exit( CBTxt::T( 'CAPTCHA_IMAGE_BG_COLOR_FAILED', 'imagecolorallocate failed to set BG colors [r], [g], [b]', [ '[r]' => $bgColorR, '[g]' => $bgColorG, '[b]' => $bgColorB ] ) );
			}

			$textColorR					=	random_int( ( ( $imgTxtColors[0] - $colorRange ) < 0 ? 0 : ( $imgTxtColors[0] - $colorRange ) ), ( ( $imgTxtColors[0] + $colorRange ) > 255 ? 255 : ( $imgTxtColors[0] + $colorRange ) ) );
			$textColorG					=	random_int( ( ( $imgTxtColors[1] - $colorRange ) < 0 ? 0 : ( $imgTxtColors[1] - $colorRange ) ), ( ( $imgTxtColors[1] + $colorRange ) > 255 ? 255 : ( $imgTxtColors[1] + $colorRange ) ) );
			$textColorB					=	random_int( ( ( $imgTxtColors[2] - $colorRange ) < 0 ? 0 : ( $imgTxtColors[2] - $colorRange ) ), ( ( $imgTxtColors[2] + $colorRange ) > 255 ? 255 : ( $imgTxtColors[2] + $colorRange ) ) );

			$textColor					=	imagecolorallocate( $image, $textColorR, $textColorG, $textColorB );

			if ( $textColor === false ) {
				exit( CBTxt::T( 'CAPTCHA_IMAGE_TEXT_COLOR_FAILED', 'imagecolorallocate failed to set text colors [r], [g], [b]', [ '[r]' => $textColorR, '[g]' => $textColorG, '[b]' => $textColorB ] ) );
			}

			$noiseBgColorR				=	random_int( ( ( $imgNoiseBgColors[0] - $colorRange ) < 0 ? 0 : ( $imgNoiseBgColors[0] - $colorRange ) ), ( ( $imgNoiseBgColors[0] + $colorRange ) > 255 ? 255 : ( $imgNoiseBgColors[0] + $colorRange ) ) );
			$noiseBgColorG				=	random_int( ( ( $imgNoiseBgColors[1] - $colorRange ) < 0 ? 0 : ( $imgNoiseBgColors[1] - $colorRange ) ), ( ( $imgNoiseBgColors[1] + $colorRange ) > 255 ? 255 : ( $imgNoiseBgColors[1] + $colorRange ) ) );
			$noiseBgColorB				=	random_int( ( ( $imgNoiseBgColors[2] - $colorRange ) < 0 ? 0 : ( $imgNoiseBgColors[2] - $colorRange ) ), ( ( $imgNoiseBgColors[2] + $colorRange ) > 255 ? 255 : ( $imgNoiseBgColors[2] + $colorRange ) ) );

			$noiseBgColor				=	imagecolorallocate( $image, $noiseBgColorR, $noiseBgColorG, $noiseBgColorB );

			if ( $noiseBgColor === false ) {
				exit( CBTxt::T( 'CAPTCHA_IMAGE_BG_NOISE_COLOR_FAILED', 'imagecolorallocate failed to set noise BG colors [r], [g], [b]', [ '[r]' => $noiseBgColorR, '[g]' => $noiseBgColorG, '[b]' => $noiseBgColorB ] ) );
			}

			$noiseFgColorR				=	random_int( ( ( $imgNoiseFgColors[0] - $colorRange ) < 0 ? 0 : ( $imgNoiseFgColors[0] - $colorRange ) ), ( ( $imgNoiseFgColors[0] + $colorRange ) > 255 ? 255 : ( $imgNoiseFgColors[0] + $colorRange ) ) );
			$noiseFgColorG				=	random_int( ( ( $imgNoiseFgColors[1] - $colorRange ) < 0 ? 0 : ( $imgNoiseFgColors[1] - $colorRange ) ), ( ( $imgNoiseFgColors[1] + $colorRange ) > 255 ? 255 : ( $imgNoiseFgColors[1] + $colorRange ) ) );
			$noiseFgColorB				=	random_int( ( ( $imgNoiseFgColors[2] - $colorRange ) < 0 ? 0 : ( $imgNoiseFgColors[2] - $colorRange ) ), ( ( $imgNoiseFgColors[2] + $colorRange ) > 255 ? 255 : ( $imgNoiseFgColors[2] + $colorRange ) ) );

			$noiseFgColor				=	imagecolorallocate( $image, $noiseFgColorR, $noiseFgColorG, $noiseFgColorB );

			if ( $noiseFgColor === false ) {
				exit( CBTxt::T( 'CAPTCHA_IMAGE_FG_NOISE_COLOR_FAILED', 'imagecolorallocate failed to set noise FG colors [r], [g], [b]', [ '[r]' => $noiseFgColorR, '[g]' => $noiseFgColorG, '[b]' => $noiseFgColorB ] ) );
			}
		} else {
			$backgroundColor			=	imagecolorallocate( $image, $imgBgColors[0], $imgBgColors[1], $imgBgColors[2] );

			if ( $backgroundColor === false ) {
				exit( CBTxt::T( 'CAPTCHA_IMAGE_BG_COLOR_FAILED', 'imagecolorallocate failed to set BG colors [r], [g], [b]', [ '[r]' => $imgBgColors[0], '[g]' => $imgBgColors[1], '[b]' => $imgBgColors[2] ] ) );
			}

			$textColor					=	imagecolorallocate( $image, $imgTxtColors[0], $imgTxtColors[1], $imgTxtColors[2] );

			if ( $textColor === false ) {
				exit( CBTxt::T( 'CAPTCHA_IMAGE_TEXT_COLOR_FAILED', 'imagecolorallocate failed to set text colors [r], [g], [b]', [ '[r]' => $imgTxtColors[0], '[g]' => $imgTxtColors[1], '[b]' => $imgTxtColors[2] ] ) );
			}

			$noiseBgColor				=	imagecolorallocate( $image, $imgNoiseBgColors[0], $imgNoiseBgColors[1], $imgNoiseBgColors[2] );

			if ( $noiseBgColor === false ) {
				exit( CBTxt::T( 'CAPTCHA_IMAGE_BG_NOISE_COLOR_FAILED', 'imagecolorallocate failed to set noise BG colors [r], [g], [b]', [ '[r]' => $imgNoiseBgColors[0], '[g]' => $imgNoiseBgColors[1], '[b]' => $imgNoiseBgColors[2] ] ) );
			}

			$noiseFgColor				=	imagecolorallocate( $image, $imgNoiseFgColors[0], $imgNoiseFgColors[1], $imgNoiseFgColors[2] );

			if ( $noiseFgColor === false ) {
				exit( CBTxt::T( 'CAPTCHA_IMAGE_FG_NOISE_COLOR_FAILED', 'imagecolorallocate failed to set noise FG colors [r], [g], [b]', [ '[r]' => $imgNoiseFgColors[0], '[g]' => $imgNoiseFgColors[1], '[b]' => $imgNoiseFgColors[2] ] ) );
			}
		}

		if ( imagefill( $image, 0, 0, $backgroundColor ) === false ) {
			exit( CBTxt::T( 'imagefill failed to add BG color to the image' ) );
		}

		if ( $params->getBool( 'captcha_internal_bg_noise', true ) ) {
			for ( $i = 0; $i < ( ( $width * $height ) / 10 ); $i++ ) {
				if ( imagefilledellipse( $image, random_int( 0, $width ), random_int( 0, $height ), 1, 1, $noiseBgColor ) === false ) {
					exit( CBTxt::T( 'imagefilledellipse failed to add BG noise to the image' ) );
				}
			}

			for( $i = 1; ( $i < ( $width / 20 ) ); $i++ ) {
				if ( imageline( $image, ( ( $i * 10 ) - random_int( -20, 20 ) ), 0, ( ( $i * 10 ) + random_int( -20, 20 ) ), $height, $noiseBgColor ) === false ) {
					exit( CBTxt::T( 'imageline failed to add first pass BG noise to the image' ) );
				}
			}

			for( $i = 1; ( $i < ( $height / 20 ) ); $i++ ) {
				if ( imageline( $image, 0, ( ( $i * 10 ) - random_int( -20, 20 ) ), $width, ( ( $i * 10 ) + random_int( -20, 20 ) ), $noiseBgColor ) === false ) {
					exit( CBTxt::T( 'imageline failed to add second pass BG noise to the image' ) );
				}
			}
		}

		if ( $_CB_framework->outputCharset() === 'UTF-8' ) {
			$codeSplit					=	preg_split( '//u', $code, -1, PREG_SPLIT_NO_EMPTY );
		} else {
			$codeSplit					=	str_split( $code );
		}

		if ( ! $codeSplit ) {
			exit( CBTxt::T( 'CAPTCHA_IMAGE_TEXT_FAILED', 'imagettftext failed to add "[text]" to the image', [ '[text]' => $code ] ) );
		}

		$x								=	( ( $width - $textBox[4] ) / 2 );
		$y								=	( ( $height - $textBox[5] ) / 2 );
		$i								=	0;

		foreach ( $codeSplit as $c ) {
			$charFont					=	( count( $imgFonts ) > 1 ? array_rand( $imgFonts, 1 ) : 0 );

			if ( $charRange ) {
				$charSize				=	random_int( ( $fontSize - $charRange ), ( $fontSize + $charRange ) );

				if ( ( $charSize < 0 ) || ( $charSize > $height ) ) {
					$charSize			=	$fontSize;
				}
			} else {
				$charSize				=	$fontSize;
			}

			$result						=	imagettftext( $image, (float) $charSize, (float) ( $charRotation ? random_int( -$charRotation, $charRotation ) : 0 ), (int) ( $x + $i ), (int) ( $charOffset ? ( $y + random_int( -$charOffset, $charOffset ) ) : $y ), -$textColor, $imgFonts[$charFont], $c );

			if ( ! $result ) {
				exit( CBTxt::T( 'CAPTCHA_IMAGE_TEXT_FAILED', 'imagettftext failed to add "[text]" to the image', [ '[text]' => $c ] ) );
			}

			$i							+=	( $textBox[4] / count( $codeSplit ) );
		}

		if ( $params->getBool( 'captcha_internal_fg_noise', true ) ) {
			for ( $i = 0; $i < 3; $i++ ) {
				if ( imageline( $image, random_int( 0, $width ), random_int( 0, $height ), random_int( 0, $width ), random_int( 0, $height ), $noiseFgColor ) === false ) {
					exit( CBTxt::T( 'imageline failed to add FG noise to the image' ) );
				}
			}
		}

		header( 'Expires: Mon, 26 Jul 1997 05:00:00 GMT' );
		header( 'Last-Modified: ' . gmdate( 'D, d M Y H:i:s' ) . ' GMT' );
		header( 'Cache-Control: no-store, no-cache, must-revalidate' );
		header( 'Cache-Control: post-check=0, pre-check=0', false );
		header( 'Pragma: no-cache' );

		switch ( $params->getString( 'captcha_internal_format', 'png' ) ) {
			case 'gif':
				header( 'Content-Type: image/gif; charset=utf-8' );

				if ( imagegif( $image ) === false ) {
					exit( CBTxt::T( 'imagegif failed to generate image' ) );
				}
				break;
			case 'png':
				header( 'Content-Type: image/png; charset=utf-8' );

				if ( imagepng( $image, null, 0 ) === false ) {
					exit( CBTxt::T( 'imagepng failed to generate image' ) );
				}
				break;
			case 'jpeg':
			default:
				header( 'Content-Type: image/jpeg; charset=utf-8' );

				if ( imagejpeg( $image, null, 100 ) === false ) {
					exit( CBTxt::T( 'imagejpeg failed to generate image' ) );
				}
				break;
		}

		if ( imagedestroy( $image ) === false ) {
			exit( CBTxt::T( 'imagedestroy failed to destroy the image' ) );
		}

		@ob_flush();
		flush();

		exit();
	}

	/**
	 * Generates and returns captcha input, image, and audio HTML
	 *
	 * @param string $id
	 */
	private function captchaLoad( string $id ): void
	{
		$captcha	=	new Captcha();

		if ( ! $captcha->load( $id ) ) {
			header( 'HTTP/1.0 404 Not Found' );
			exit();
		}

		if ( $captcha->getCode() || ( ! in_array( $captcha->getMode(), [ 'internal', 'question', 'math' ], true ) ) ) {
			header( 'HTTP/1.0 403 Forbidden' );
			exit();
		}

		$captcha->generateCode();

		header( 'Expires: Mon, 26 Jul 1997 05:00:00 GMT' );
		header( 'Content-Type: application/json' );
		header( 'Cache-Control: no-store, no-cache, must-revalidate' );
		header( 'Cache-Control: post-check=0, pre-check=0', false );
		header( 'Pragma: no-cache' );
		header( 'HTTP/1.0 200 OK' );

		/** @noinspection PhpStatementHasEmptyBodyInspection */
		/** @noinspection LoopWhichDoesNotLoopInspection */
		/** @noinspection MissingOrEmptyGroupStatementInspection */
		while ( @ob_end_clean() ) {}

		echo json_encode( [ 'input' => $captcha->getInput(), 'image' => $captcha->getImage(), 'audio' => $captcha->getAudio(), 'captcha' => $captcha->getCaptcha() ] );

		exit();
	}

	/**
	 * Generates a captcha audio file
	 *
	 * @param string $id
	 */
	private function captchaAudio( string $id ): void
	{
		global $_PLUGINS;

		if ( ( ! $id ) || ( $id === 'preview' ) ) {
			header( 'HTTP/1.0 404 Not Found' );
			exit();
		}

		/** @noinspection PhpStatementHasEmptyBodyInspection */
		/** @noinspection LoopWhichDoesNotLoopInspection */
		/** @noinspection MissingOrEmptyGroupStatementInspection */
		while ( @ob_end_clean() ) {}

		$absPath			=	$_PLUGINS->getPluginPath( $this->getPluginId() );
		$captcha			=	new Captcha();

		$captcha->load( $id );

		$code				=	$captcha->getCode();

		if ( ! $code ) {
			exit( CBTxt::T( 'failed to generate captcha code' ) );
		}

		$sounds				=	[];

		for ( $i = 0; $i < cbIsoUtf_strlen( $code ); $i++ ) {
			$file			=	$absPath . '/audio/' . $code[$i] . '.mp3';

			if ( ! file_exists( $file ) ) {
				exit( CBTxt::T( 'CAPTCHA_AUDIO_FILE_FAILED', 'failed to locate "[file]" audio file', [ '[file]' => $file ] ) );
			}

			$sounds[]		=	$file;
		}

		header( 'Expires: Mon, 26 Jul 1997 05:00:00 GMT' );
		header( 'Last-Modified: ' . gmdate( 'D, d M Y H:i:s' ) . ' GMT' );
		header( 'Cache-Control: no-store, no-cache, must-revalidate' );
		header( 'Cache-Control: post-check=0, pre-check=0', false );
		header( 'Pragma: no-cache' );
		header( 'Content-Type: audio/mpeg' );
		header( 'Content-Disposition: inline; filename=cbcaptcha.mp3;' );
		header( 'Content-Transfer-Encoding: binary' );

		$out				=	'';
		$count				=	count( $sounds );
		$i					=	0;

		foreach ( $sounds as $sound ) {
			$i++;

			if ( $i !== $count ) {
				$offset		=	128;
			} else {
				$offset		=	0;
			}

			$fh				=	fopen( $sound, 'rb' );
			$size			=	filesize( $sound );

			$out			.=	fread( $fh, ( $size - $offset ) );

			fclose( $fh );
		}

		header( 'Content-Length: ' . cbIsoUtf_strlen( $out ) );

		echo $out;

		@ob_flush();
		flush();

		exit();
	}

	/**
	 * Displays block user page
	 *
	 * @param int         $id
	 * @param null|string $type
	 */
	private function showBlock( int $id = 0, ?string $type = null ): void
	{
		global $_CB_framework;

		$blockUser				=	CBuser::getUserDataInstance( $this->getInput()->getInt( 'user', 0 ) );

		if ( Application::User( $blockUser->getInt( 'id', 0 ) )->isGlobalModerator() ) {
			CBAntiSpam::returnRedirect( 'index.php', CBTxt::T( 'Not authorized.' ), 'error' );
		}

		$row					=	new BlockTable();

		if ( $id ) {
			$row->load( $id );
		} else {
			$row->setType( $type );

			switch ( $type ) {
				case 'account':
				case 'user':
				case 'ban':
				case 'reset':
					$row->setValue( $blockUser->getInt( 'id', 0 ) );
					break;
				case 'ip':
					$row->setValue( CBAntiSpam::getUserIP( $blockUser ) );
					break;
				case 'ip_range':
					$row->setValue( CBAntiSpam::getUserIP( $blockUser ) . ':' . CBAntiSpam::getUserIP( $blockUser ) );
					break;
				case 'email':
					$row->setValue( $blockUser->getString( 'email', '' ) );
					break;
				case 'domain':
					$row->setValue( CBAntiSpam::getEmailDomain( $blockUser ) );
					break;
			}

			$row->setDate( Application::Database()->getUtcDateTime() );
		}

		$js						=	"$( '#durations' ).on( 'change', function() {"
								.		"var value = $( this ).val();"
								.		"if ( value ) {"
								.			"$( '#duration' ).attr( 'value', value ).focus();"
								.			"$( this ).attr( 'value', '' );"
								.		"}"
								.	"});"
								.	"$( '#type' ).on( 'change', function() {"
								.		"if ( $( this ).val() == 'user' ) {"
								.			"$( '.blockEditDate,.blockEditDuration,.blockEditReason,.blockEditBan,.blockEditBlock' ).removeClass( 'hidden' );"
								.			"$( '#ban_user' ).trigger( 'change' );"
								.		"} else if ( ( $( this ).val() == 'account' ) || ( $( this ).val() == 'reset' ) ) {"
								.			"$( '.blockEditDate,.blockEditDuration,.blockEditReason,.blockEditBan,.blockEditBlock' ).addClass( 'hidden' );"
								.			"$( '#ban_user' ).val( 0 ).trigger( 'change' );"
								.		"} else {"
								.			"$( '.blockEditDate,.blockEditDuration,.blockEditReason' ).removeClass( 'hidden' );"
								.			"$( '.blockEditBan,.blockEditBlock,.blockEditBanReason' ).addClass( 'hidden' );"
								.		"}"
								.	"}).change();"
								.	"$( '#ban_user' ).on( 'change', function() {"
								.		"if ( $( this ).val() == 1 ) {"
								.			"$( '.blockEditBanReason' ).removeClass( 'hidden' );"
								.		"} else {"
								.			"$( '.blockEditBanReason' ).addClass( 'hidden' );"
								.		"}"
								.	"});";

		$_CB_framework->outputCbJQuery( $js );

		cbValidator::loadValidation();
		initToolTip();

		if ( $blockUser->getInt( 'id', 0 ) ) {
			$pageTitle			=	CBTxt::T( 'BLOCK_NAME', 'Block [name]', [ '[name]' => $blockUser->getFormattedName() ] );
		} else {
			$pageTitle			=	( $row->getId() ? CBTxt::T( 'Edit Block' ) : CBTxt::T( 'New Block' ) );
		}

		$_CB_framework->setPageTitle( $pageTitle );

		$returnUrl				=	CBAntiSpam::getReturn( true, true );

		if ( ! $returnUrl ) {
			switch ( ( $row->getId() ? $row->getType() : $type ) ) {
				case 'account':
				case 'user':
				case 'ban':
				case 'reset':
					$returnUrl	=	$_CB_framework->viewUrl( 'userprofile', true, [ 'user' => ( $row->getId() ? $row->getUser() : $blockUser->getInt( 'id', 0 ) ) ] );
					break;
				default:
					$returnUrl	=	$_CB_framework->cbSef( 'index.php' );
					break;
			}
		}

		$input					=	[];

		$listType				=	[];
		$listType[]				=	moscomprofilerHTML::makeOption( 'account', CBTxt::T( 'Account' ) );
		$listType[]				=	moscomprofilerHTML::makeOption( 'user', CBTxt::T( 'User' ) );
		$listType[]				=	moscomprofilerHTML::makeOption( 'reset', CBTxt::T( 'Reset Password' ) );
		$listType[]				=	moscomprofilerHTML::makeOption( 'ip', CBTxt::T( 'IP Address' ) );
		$listType[]				=	moscomprofilerHTML::makeOption( 'ip_range', CBTxt::T( 'IP Address Range' ) );
		$listType[]				=	moscomprofilerHTML::makeOption( 'email', CBTxt::T( 'Email Address' ) );
		$listType[]				=	moscomprofilerHTML::makeOption( 'domain', CBTxt::T( 'Email Domain' ) );

		$type					=	$this->getInput()->getString( 'post/type', $row->getType() );
		$typeTooltip			=	cbTooltip( null, CBTxt::T( 'Select the block type. Type determines what value should be supplied.' ), null, null, null, null, null, 'data-hascbtooltip="true"' );

		$input['type']			=	moscomprofilerHTML::selectList( $listType, 'type', 'class="form-control required"' . $typeTooltip, 'value', 'text', $type, 1, true, false, false );

		$valueTooltip			=	cbTooltip( null, CBTxt::T( 'Input block value in relation to the type. User type use the users user_id (e.g. 42). IP Address type use a full valid IP Address (e.g. 192.168.0.1). IP Address Range type use two full valid IP Addresses separated by a colon (e.g. 192.168.0.1:192.168.0.100). Email type use a fill valid email address (e.g. invalid@cb.invalid). Email Domain type use a full email address domain after @ (e.g. example.com). Additionally IP Address, Email Address, and Email Domain types support % wildcard.' ), null, null, null, null, null, 'data-hascbtooltip="true"' );

		$input['value']			=	'<input type="text" id="value" name="value" value="' . htmlspecialchars( $this->getInput()->getString( 'post/value', $row->getValue() ) ) . '" class="form-control required" size="25"' . $valueTooltip . ' />';

		$calendar				=	new cbCalendars( 1 );
		$dateTooltip			=	cbTooltip( null, CBTxt::T( 'Select the date and time the block should go in affect. Note date and time always functions in UTC.' ), null, null, null, null, null, 'data-hascbtooltip="true"' );

		$input['date']			=	$calendar->cbAddCalendar( 'date', null, true, $this->getInput()->getString( 'post/date', $row->getDate() ), false, true, null, null, $dateTooltip );

		$durationTooltip		=	cbTooltip( null, CBTxt::T( 'Input the strtotime relative date (e.g. +1 Day). This duration will be added to the datetime specified above. Leave blank for a forever duration.' ), null, null, null, null, null, 'data-hascbtooltip="true"' );

		$input['duration']		=	'<input type="text" id="duration" name="duration" value="' . htmlspecialchars( $this->getInput()->getString( 'post/duration', $row->getDuration() ) ) . '" class="form-control" size="25"' . $durationTooltip . ' />';

		$listDurations			=	[];
		$listDurations[]		=	moscomprofilerHTML::makeOption( '', CBTxt::T( '- Select Duration -' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'MIDNIGHT', CBTxt::T( 'Midnight' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'NOON', CBTxt::T( 'Noon' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'TOMORROW', CBTxt::T( 'Tomorrow' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'NEXT WEEK', CBTxt::T( 'Next Week' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'NEXT MONTH', CBTxt::T( 'Next Month' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'NEXT YEAR', CBTxt::T( 'Next Year' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'LAST DAY OF THIS MONTH', CBTxt::T( 'Last Day of This Month' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'FIRST DAY OF NEXT MONTH', CBTxt::T( 'First Day of Next Month' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'LAST DAY OF NEXT MONTH', CBTxt::T( 'Last Day of Next Month' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'LAST DAY OF THIS YEAR', CBTxt::T( 'Last Day of This Year' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'FIRST DAY OF NEXT YEAR', CBTxt::T( 'First Day of Next Year' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'LAST DAY OF NEXT YEAR', CBTxt::T( 'Last Day of Next Year' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'FIRST DAY OF JANUARY', CBTxt::T( 'First Day of January' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'LAST DAY OF JANUARY', CBTxt::T( 'Last Day of January' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'FIRST DAY OF FEBRUARY', CBTxt::T( 'First Day of February' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'LAST DAY OF FEBRUARY', CBTxt::T( 'Last Day of February' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'FIRST DAY OF MARCH', CBTxt::T( 'First Day of March' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'LAST DAY OF MARCH', CBTxt::T( 'Last Day of March' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'FIRST DAY OF APRIL', CBTxt::T( 'First Day of Apil' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'LAST DAY OF APRIL', CBTxt::T( 'Last Day of Apil' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'FIRST DAY OF MAY', CBTxt::T( 'First Day of May' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'LAST DAY OF MAY', CBTxt::T( 'Last Day of May' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'FIRST DAY OF JUNE', CBTxt::T( 'First Day of June' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'LAST DAY OF JUNE', CBTxt::T( 'Last Day of June' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'FIRST DAY OF JULY', CBTxt::T( 'First Day of July' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'LAST DAY OF JULY', CBTxt::T( 'Last Day of July' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'FIRST DAY OF AUGUST', CBTxt::T( 'First Day of August' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'LAST DAY OF AUGUST', CBTxt::T( 'Last Day of August' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'FIRST DAY OF SEPTEMBER', CBTxt::T( 'First Day of September' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'LAST DAY OF SEPTEMBER', CBTxt::T( 'Last Day of September' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'FIRST DAY OF OCTOBER', CBTxt::T( 'First Day of October' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'LAST DAY OF OCTOBER', CBTxt::T( 'Last Day of October' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'FIRST DAY OF NOVEMBER', CBTxt::T( 'First Day of November' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'LAST DAY OF NOVEMBER', CBTxt::T( 'Last Day of November' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'FIRST DAY OF DECEMBER', CBTxt::T( 'First Day of December' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( 'LAST DAY OF DECEMBER', CBTxt::T( 'Last Day of December' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( '+30 MINUTES', CBTxt::T( '30 Minutes' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( '+1 HOUR', CBTxt::T( '1 Hour' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( '+3 HOURS', CBTxt::T( '3 Hours' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( '+6 HOURS', CBTxt::T( '6 Hours' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( '+12 HOURS', CBTxt::T( '12 Hours' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( '+1 DAY', CBTxt::T( '1 Day' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( '+3 DAYS', CBTxt::T( '3 Days' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( '+6 DAYS', CBTxt::T( '6 Days' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( '+12 DAYS', CBTxt::T( '12 Days' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( '+1 WEEK', CBTxt::T( '1 Week' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( '+2 WEEKS', CBTxt::T( '2 Weeks' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( '+3 WEEKS', CBTxt::T( '3 Weeks' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( '+1 MONTH', CBTxt::T( '1 Month' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( '+3 MONTHS', CBTxt::T( '3 Months' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( '+6 MONTHS', CBTxt::T( '6 Months' ) );
		$listDurations[]		=	moscomprofilerHTML::makeOption( '+1 YEAR', CBTxt::T( '1 Year' ) );
		$input['durations']		=	moscomprofilerHTML::selectList( $listDurations, 'durations', 'class="form-control"', 'value', 'text', null, 0, true, false, false );

		$reasonTooltip			=	cbTooltip( null, CBTxt::T( 'Optionally input block reason. If left blank will default to spam.' ), null, null, null, null, null, 'data-hascbtooltip="true"' );

		$input['reason']		=	'<textarea id="reason" name="reason" class="form-control" cols="40" rows="5"' . $reasonTooltip . '>' . htmlspecialchars( $this->getInput()->getString( 'post/reason', $row->getReason() ) ) . '</textarea>';

		$banUserTooltip			=	cbTooltip( null, CBTxt::T( 'Ban the users profile using Community Builder moderator ban feature. Note normal ban notification will be sent with the ban.' ), null, null, null, null, null, 'data-hascbtooltip="true"' );

		$input['ban_user']		=	moscomprofilerHTML::yesnoSelectList( 'ban_user', 'class="form-control"' . $banUserTooltip, $this->getInput()->getInt( 'post/ban_user', 0 ) );

		$banReasonTooltip		=	cbTooltip( null, CBTxt::T( 'Optionally input reason for profile ban.' ), null, null, null, null, null, 'data-hascbtooltip="true"' );

		$input['ban_reason']	=	'<textarea id="ban_reason" name="ban_reason" class="form-control" cols="40" rows="5"' . $banReasonTooltip . '>' . htmlspecialchars( $this->getInput()->getString( 'post/ban_reason', '' ) ) . '</textarea>';

		$blockUserTooltip		=	cbTooltip( null, CBTxt::T( 'Block the users profile using Joomla block state.' ), null, null, null, null, null, 'data-hascbtooltip="true"' );

		$input['block_user']	=	moscomprofilerHTML::yesnoSelectList( 'block_user', 'class="form-control"' . $blockUserTooltip, $this->getInput()->getInt( 'post/block_user', 0 ) );

		$formUrl				=	$_CB_framework->pluginClassUrl( 'cbantispam', true, [ 'action' => 'block', 'func' => 'save', 'id' => $row->getId(), 'user' => $blockUser->getInt( 'id', 0 ), 'return' => CBAntiSpam::getReturn( true ) ] );

		require CBAntiSpam::getTemplate( 'block' );
	}

	/**
	 * Saves a user block
	 *
	 * @param int $id
	 */
	private function saveBlock( int $id = 0 ): void
	{
		global $_CB_framework;

		$row						=	new BlockTable();

		if ( $id ) {
			$row->load( $id );
		}

		$row->setType( $this->getInput()->getString( 'post/type', $row->getType() ) );
		$row->setValue( $this->getInput()->getString( 'post/value', $row->getValue() ) );
		$row->setReason( $this->getInput()->getString( 'post/reason', $row->getReason() ) );
		$row->setDate( $this->getInput()->getString( 'post/date', $row->getDate() ) );
		$row->setDuration( $this->getInput()->getString( 'post/duration', $row->getDuration() ) );

		$new						=	( ! $row->getId() );

		if ( $row->getError() || ( ! $row->check() ) ) {
			$_CB_framework->enqueueMessage( CBTxt::T( 'BLOCK_SAVE_FAILED', 'Block failed to save! Error: [error]', [ '[error]' => $row->getError() ] ), 'error' );

			$this->showBlock( $id, $row->getType() );
			return;
		}

		if ( $row->getError() || ( ! $row->store() ) ) {
			$_CB_framework->enqueueMessage( CBTxt::T( 'BLOCK_SAVE_FAILED', 'Block failed to save! Error: [error]', [ '[error]' => $row->getError() ] ), 'error' );

			$this->showBlock( $id, $row->getType() );
			return;
		}

		if ( $row->getType() === 'user' ) {
			if ( $this->getInput()->getInt( 'post/block_user', 0 ) ) {
				if ( ! $row->blockUser() ) {
					$_CB_framework->enqueueMessage( CBTxt::T( 'BLOCK_PROFILE_BLOCK_FAILED', 'Block saved successfully, but Profile Block failed to save! Error: [error]', [ '[error]' => $row->getError() ] ), 'error' );

					$this->showBlock( $row->getId(), $row->getType() );
					return;
				}
			} elseif ( $this->getInput()->getInt( 'post/ban_user', 0 ) ) {
				if ( ! $row->banUser( $this->getInput()->getString( 'post/ban_reason', '' ) ) ) {
					$_CB_framework->enqueueMessage( CBTxt::T( 'BLOCK_PROFILE_BAN_FAILED', 'Block saved successfully, but Profile Ban failed to save! Error: [error]', [ '[error]' => $row->getError() ] ), 'error' );

					$this->showBlock( $row->getId(), $row->getType() );
					return;
				}
			}
		}

		$returnUrl					=	( $row->getUserId() ? $_CB_framework->viewUrl( 'userprofile', true, [ 'user' => $row->getUserId() ] ) : $_CB_framework->cbSef( 'index.php' ) );

		if ( $new ) {
			CBAntiSpam::returnRedirect( $returnUrl, CBTxt::T( 'Block created successfully!' ) );
		} else {
			CBAntiSpam::returnRedirect( $returnUrl, CBTxt::T( 'Block saved successfully!' ) );
		}
	}

	/**
	 * Deletes a user block
	 *
	 * @param int $id
	 */
	private function deleteBlock( int $id ): void
	{
		global $_CB_framework;

		$row		=	new BlockTable();

		$row->load( $id );

		$returnUrl	=	( $row->getUserId() ? $_CB_framework->viewUrl( 'userprofile', true, [ 'user' => $row->getUserId() ] ) : $_CB_framework->cbSef( 'index.php' ) );

		if ( ! $row->getId() ) {
			CBAntiSpam::returnRedirect( $returnUrl, CBTxt::T( 'Not authorized.' ), 'error' );
		}

		if ( ! $row->delete() ) {
			CBAntiSpam::returnRedirect( $returnUrl, CBTxt::T( 'BLOCK_DELETE_FAILED', 'Block failed to delete! Error: [error]', [ '[error]' => $row->getError() ] ), 'error' );
		}

		CBAntiSpam::returnRedirect( $returnUrl, CBTxt::T( 'Block deleted successfully!' ) );
	}

	/**
	 * Displays whitelist user page
	 *
	 * @param int         $id
	 * @param null|string $type
	 */
	private function showWhitelist( int $id = 0, ?string $type = null ): void
	{
		global $_CB_framework;

		$whitelistUser			=	CBuser::getUserDataInstance( $this->getInput()->getInt( 'user', 0 ) );

		if ( Application::User( $whitelistUser->getInt( 'id', 0 ) )->isGlobalModerator() ) {
			CBAntiSpam::returnRedirect( 'index.php', CBTxt::T( 'Not authorized.' ), 'error' );
		}

		$row					=	new WhitelistTable();

		if ( $id ) {
			$row->load( $id );
		} else {
			$row->setType( $type );

			switch ( $type ) {
				case 'account':
				case 'user':
				case 'ban':
				case 'reset':
					$row->setValue( $whitelistUser->getInt( 'id', 0 ) );
					break;
				case 'ip':
					$row->setValue( CBAntiSpam::getUserIP( $whitelistUser ) );
					break;
				case 'ip_range':
					$row->setValue( CBAntiSpam::getUserIP( $whitelistUser ) . ':' . CBAntiSpam::getUserIP( $whitelistUser ) );
					break;
				case 'email':
					$row->setValue( $whitelistUser->getString( 'email', '' ) );
					break;
				case 'domain':
					$row->setValue( CBAntiSpam::getEmailDomain( $whitelistUser ) );
					break;
			}
		}

		cbValidator::loadValidation();
		initToolTip();

		if ( $whitelistUser->getInt( 'id', 0 ) ) {
			$pageTitle			=	CBTxt::T( 'WHITELIST_NAME', 'Whitelist [name]', [ '[name]' => $whitelistUser->getFormattedName() ] );
		} else {
			$pageTitle			=	( $row->getId() ? CBTxt::T( 'Edit Whitelist' ) : CBTxt::T( 'New Whitelist' ) );
		}

		$_CB_framework->setPageTitle( $pageTitle );

		$returnUrl				=	CBAntiSpam::getReturn( true, true );

		if ( ! $returnUrl ) {
			switch ( ( $row->getId() ? $row->getType() : $type ) ) {
				case 'account':
				case 'user':
				case 'ban':
				case 'reset':
					$returnUrl	=	$_CB_framework->viewUrl( 'userprofile', true, [ 'user' => ( $row->getId() ? $row->getUser() : $whitelistUser->getInt( 'id', 0 ) ) ] );
					break;
				default:
					$returnUrl	=	$_CB_framework->cbSef( 'index.php' );
					break;
			}
		}

		$input					=	[];

		$listType				=	[];
		$listType[]				=	moscomprofilerHTML::makeOption( 'user', CBTxt::T( 'User' ) );
		$listType[]				=	moscomprofilerHTML::makeOption( 'ip', CBTxt::T( 'IP Address' ) );
		$listType[]				=	moscomprofilerHTML::makeOption( 'ip_range', CBTxt::T( 'IP Address Range' ) );
		$listType[]				=	moscomprofilerHTML::makeOption( 'email', CBTxt::T( 'Email Address' ) );
		$listType[]				=	moscomprofilerHTML::makeOption( 'domain', CBTxt::T( 'Email Domain' ) );

		$type					=	$this->getInput()->getString( 'post/type', $row->getType() );
		$typeTooltip			=	cbTooltip( null, CBTxt::T( 'Select whitelist block type. Type determines what value should be supplied.' ), null, null, null, null, null, 'data-hascbtooltip="true"' );

		$input['type']			=	moscomprofilerHTML::selectList( $listType, 'type', 'class="form-control required"' . $typeTooltip, 'value', 'text', $type, 1, true, false, false );

		$valueTooltip			=	cbTooltip( null, CBTxt::T( 'Input whitelist value in relation to the type. User type use the users user_id (e.g. 42). IP Address type use a full valid IP Address (e.g. 192.168.0.1). IP Address Range type use two full valid IP Addresses separated by a colon (e.g. 192.168.0.1:192.168.0.100). Email type use a fill valid email address (e.g. invalid@cb.invalid). Email Domain type use a full email address domain after @ (e.g. example.com). Additionally IP Address, Email Address, and Email Domain types support % wildcard.' ), null, null, null, null, null, 'data-hascbtooltip="true"' );

		$input['value']			=	'<input type="text" id="value" name="value" value="' . htmlspecialchars( $this->getInput()->getString( 'post/value', $row->getValue() ) ) . '" class="form-control required" size="25"' . $valueTooltip . ' />';

		$reasonTooltip			=	cbTooltip( null, CBTxt::T( 'Optionally input whitelist reason. Note this is for administrative purposes only.' ), null, null, null, null, null, 'data-hascbtooltip="true"' );

		$input['reason']		=	'<textarea id="reason" name="reason" class="form-control" cols="40" rows="5"' . $reasonTooltip . '>' . htmlspecialchars( $this->getInput()->getString( 'post/reason', $row->getReason() ) ) . '</textarea>';

		$formUrl				=	$_CB_framework->pluginClassUrl( 'cbantispam', true, [ 'action' => 'whitelist', 'func' => 'save', 'id' => $row->getId(), 'user' => $whitelistUser->getInt( 'id', 0 ), 'return' => CBAntiSpam::getReturn( true ) ] );

		require CBAntiSpam::getTemplate( 'whitelist' );
	}

	/**
	 * Saves a user whitelist
	 *
	 * @param int $id
	 */
	private function saveWhitelist( int $id = 0 ): void
	{
		global $_CB_framework;

		$row		=	new WhitelistTable();

		if ( $id ) {
			$row->load( $id );
		}

		$row->setType( $this->getInput()->getString( 'post/type', $row->getType() ) );
		$row->setValue( $this->getInput()->getString( 'post/value', $row->getValue() ) );
		$row->setReason( $this->getInput()->getString( 'post/reason', $row->getReason() ) );

		$new		=	( ! $row->getId() );

		if ( $row->getError() || ( ! $row->check() ) ) {
			$_CB_framework->enqueueMessage( CBTxt::T( 'WHITELIST_SAVE_FAILED', 'Whitelist failed to save! Error: [error]', [ '[error]' => $row->getError() ] ), 'error' );

			$this->showWhitelist( $id, $row->getType() );
			return;
		}

		if ( $row->getError() || ( ! $row->store() ) ) {
			$_CB_framework->enqueueMessage( CBTxt::T( 'WHITELIST_SAVE_FAILED', 'Whitelist failed to save! Error: [error]', [ '[error]' => $row->getError() ] ), 'error' );

			$this->showWhitelist( $id, $row->getType() );
			return;
		}

		$returnUrl	=	( $row->getUserId() ? $_CB_framework->viewUrl( 'userprofile', true, [ 'user' => $row->getUserId() ] ) : $_CB_framework->cbSef( 'index.php' ) );

		if ( $new ) {
			CBAntiSpam::returnRedirect( $returnUrl, CBTxt::T( 'Whitelist created successfully!' ) );
		} else {
			CBAntiSpam::returnRedirect( $returnUrl, CBTxt::T( 'Whitelist saved successfully!' ) );
		}
	}

	/**
	 * Deletes a user whitelist
	 *
	 * @param int $id
	 */
	private function deleteWhitelist( int $id ): void
	{
		global $_CB_framework;

		$row		=	new WhitelistTable();

		$row->load( $id );

		$returnUrl	=	( $row->getUserId() ? $_CB_framework->viewUrl( 'userprofile', true, [ 'user' => $row->getUserId() ] ) : $_CB_framework->cbSef( 'index.php' ) );

		if ( ! $row->getId() ) {
			CBAntiSpam::returnRedirect( $returnUrl, CBTxt::T( 'Not authorized.' ), 'error' );
		}

		if ( ! $row->delete() ) {
			CBAntiSpam::returnRedirect( $returnUrl, CBTxt::T( 'WHITELIST_DELETE_FAILED', 'Whitelist failed to delete! Error: [error]', [ '[error]' => $row->getError() ] ), 'error' );
		}

		CBAntiSpam::returnRedirect( $returnUrl, CBTxt::T( 'Whitelist deleted successfully!' ) );
	}

	/**
	 * Deletes a user attempt
	 *
	 * @param int $id
	 */
	private function deleteAttempt( int $id ): void
	{
		$row	=	new AttemptTable();

		$row->load( $id );

		if ( ! $row->getId() ) {
			CBAntiSpam::returnRedirect( 'index.php', CBTxt::T( 'Not authorized.' ), 'error' );
		}

		if ( ! $row->delete() ) {
			CBAntiSpam::returnRedirect( 'index.php', CBTxt::T( 'ATTEMPT_DELETE_FAILED', 'Attempt failed to delete! Error: [error]', [ '[error]' => $row->getError() ] ), 'error' );
		}

		CBAntiSpam::returnRedirect( 'index.php', CBTxt::T( 'Attempt deleted successfully!' ) );
	}

	/**
	 * Deletes a user log
	 *
	 * @param int $id
	 */
	private function deleteLog( int $id ): void
	{
		$row	=	new LogTable();

		$row->load( $id );

		if ( ! $row->getId() ) {
			CBAntiSpam::returnRedirect( 'index.php', CBTxt::T( 'Not authorized.' ), 'error' );
		}

		if ( ! $row->delete() ) {
			CBAntiSpam::returnRedirect( 'index.php', CBTxt::T( 'LOG_DELETE_FAILED', 'Log failed to delete! Error: [error]', [ '[error]' => $row->getError() ] ), 'error' );
		}

		CBAntiSpam::returnRedirect( 'index.php', CBTxt::T( 'Log deleted successfully!' ) );
	}
}