<?php
/**
* Community Builder (TM)
* @version $Id: $
* @package CommunityBuilder
* @copyright (C) 2004-2022 www.joomlapolis.com / Lightning MultiCom SA - and its licensors, all rights reserved
* @license http://www.gnu.org/licenses/old-licenses/gpl-2.0.html GNU/GPL version 2
*/

use CB\Database\Table\TabTable;
use CB\Database\Table\UserTable;
use CB\Plugin\AutoActions\Table\AutoActionTable;
use CB\Plugin\AutoActions\Table\BatchTable;

\defined( 'CBLIB' ) or die();

class CBplug_cbautoactions extends cbPluginHandler
{

	/**
	 * @param  TabTable   $tab       Current tab
	 * @param  UserTable  $user      Current user
	 * @param  int        $ui        1 front, 2 admin UI
	 * @param  array      $postdata  Raw unfiltred POST data
	 */
	public function getCBpluginComponent( $tab, $user, $ui, $postdata )
	{
		global $_CB_framework, $_CB_database;

		$action										=	$this->getInput()->getString( 'action', '' );
		$token										=	$this->getInput()->getString( 'token', '' );
		$userIds									=	$this->getInput()->getString( 'users', '' );
		$actionIds									=	$this->getInput()->getString( 'actions', '' );
		$batchIds									=	$this->getInput()->getString( 'batches', '' );
		$output										=	( $this->getInput()->getString( 'format', '' ) !== 'raw' );

		if ( $user === null ) {
			$user									=	CBuser::getMyUserDataInstance();
		}

		$return										=	null;

		if ( ! is_array( $userIds ) ) {
			if ( $userIds ) {
				$userIds							=	explode( ',', $userIds );
			} else {
				$userIds							=	[];
			}
		}

		if ( ! is_array( $actionIds ) ) {
			if ( $actionIds ) {
				$actionIds							=	explode( ',', $actionIds );
			} else {
				$actionIds							=	[];
			}
		}

		if ( ! is_array( $batchIds ) ) {
			if ( $batchIds ) {
				$batchIds							=	explode( ',', $batchIds );
			} else {
				$batchIds							=	[];
			}
		}

		if ( ( $token === md5( $_CB_framework->getCfg( 'secret' ) ) ) || ( ( $action === 'action' ) && $actionIds ) ) {
			if ( ( ( $action === 'batch' ) && $batchIds ) || ( $action === 'general' ) || ( $action === 'users' ) ) {
				$query								=	"SELECT *"
													.	"\n FROM " . $_CB_database->NameQuote( '#__comprofiler_plugin_autoactions_batches' );
				if ( $action === 'general' ) {
					$query							.=	"\n WHERE ( " . $_CB_database->NameQuote( 'title' ) . " = " . $_CB_database->Quote( 'internalGeneral' )
													.	" OR " . $_CB_database->NameQuote( 'users' ) . " = 0 )";
				} elseif ( $action === 'users' ) {
					$query							.=	"\n WHERE ( " . $_CB_database->NameQuote( 'title' ) . " = " . $_CB_database->Quote( 'internalUsers' )
													.	" OR " . $_CB_database->NameQuote( 'users' ) . " = 1 )";
				} elseif ( count( $batchIds ) > 1 ) {
					$query							.=	"\n WHERE " . $_CB_database->NameQuote( 'id' ) . " IN " . $_CB_database->safeArrayOfIntegers( $batchIds );
				} else {
					$query							.=	"\n WHERE " . $_CB_database->NameQuote( 'id' ) . " = " . (int) $batchIds[0];
				}
				$query								.=	"\n AND " . $_CB_database->NameQuote( 'published' ) . " = 1";
				if ( in_array( $action, [ 'general', 'users' ], true ) ) {
					$_CB_database->setQuery( $query, 0, 1 );
				} else {
					$_CB_database->setQuery( $query );
				}
				$rows								=	$_CB_database->loadObjectList( 'id', BatchTable::class, [ $_CB_database ] );

				/** @var $rows BatchTable[] */
				foreach ( $rows as $row ) {
					if ( $action === 'general' ) {
						$trigger					=	'internalGeneral';
					} elseif ( $action === 'users' ) {
						$trigger					=	'internalUsers';
					} else {
						$trigger					=	'batch' . $row->getId();
					}

					$return							.=	$row->runBatch( $user, [ 'trigger' => $trigger, 'var1' => $user, 'var2' => $row ] );
				}
			} elseif ( ( $action === 'action' ) && $actionIds ) {
				$query								=	"SELECT *"
													.	"\n FROM " . $_CB_database->NameQuote( '#__comprofiler_plugin_autoactions' );
				if ( count( $actionIds ) > 1 ) {
					$query							.=	"\n WHERE " . $_CB_database->NameQuote( 'id' ) . " IN " . $_CB_database->safeArrayOfIntegers( $actionIds );
				} else {
					$query							.=	"\n WHERE " . $_CB_database->NameQuote( 'id' ) . " = " . (int) $actionIds[0];
				}
				$query								.=	"\n AND " . $_CB_database->NameQuote( 'trigger' ) . " = " . $_CB_database->Quote( '' )
													.	"\n AND " . $_CB_database->NameQuote( 'published' ) . " = 1"
													.	"\n ORDER BY " . $_CB_database->NameQuote( 'ordering' ) . " ASC";
				$_CB_database->setQuery( $query );
				$rows								=	$_CB_database->loadObjectList( 'id', AutoActionTable::class, [ $_CB_database ] );

				/** @var $rows AutoActionTable[] */
				if ( $rows ) {
					if ( $userIds ) {
						$query						=	"SELECT *"
													.	"\n FROM " . $_CB_database->NameQuote( '#__comprofiler' ) . " AS c"
													.	"\n INNER JOIN " . $_CB_database->NameQuote( '#__users' ) . ' AS u'
													.	' ON u.' . $_CB_database->NameQuote( 'id' ) . ' = c.' . $_CB_database->NameQuote( 'id' );
						if ( count( $userIds ) > 1 ) {
							$query					.=	"\n WHERE c." . $_CB_database->NameQuote( 'id' ) . " IN " . $_CB_database->safeArrayOfIntegers( $userIds );
						} else {
							$query					.=	"\n WHERE c." . $_CB_database->NameQuote( 'id' ) . " = " . (int) $userIds[0];
						}
						$_CB_database->setQuery( $query );
						$users						=	$_CB_database->loadObjectList( null, UserTable::class, [ $_CB_database ] );

						if ( $users ) {
							foreach ( $users as $u ) {
								foreach ( $rows as $row ) {
									if ( ! $row->getParams()->getBool( 'direct_access', true ) ) {
										continue;
									}

									$variables	=	[ 'trigger' => 'internalAction', 'var1' => $u ];

									$return		.=	$row->runAction( $variables );
								}
							}
						}
					} else {
						foreach ( $rows as $row ) {
							if ( ! $row->getParams()->getBool( 'direct_access', true ) ) {
								continue;
							}

							$variables			=	[ 'trigger' => 'internalAction', 'var1' => $user ];

							$return				.=	$row->runAction( $variables );
						}
					}
				}
			} else {
				if ( ! $output ) {
					header( 'HTTP/1.0 405 Method Not Allowed' );
					exit();
				}

				cbNotAuth();
			}
		} else {
			if ( ! $output ) {
				header( 'HTTP/1.0 403 Forbidden' );
				exit();
			}

			cbNotAuth();
		}

		echo $return;
	}
}