<?php
/**
* Community Builder (TM)
* @version $Id: $
* @package CommunityBuilder
* @copyright (C)2005-2015 www.joomlapolis.com / Lightning MultiCom SA - and its licensors, all rights reserved
* @license http://www.gnu.org/licenses/old-licenses/gpl-2.0.html GNU/GPL version 2
*/

use CB\Database\Table\TabTable;
use CB\Database\Table\UserTable;
use CBLib\Language\CBTxt;
use CB\Plugin\ReconfirmEmail\Helper;
use CB\Plugin\ReconfirmEmail\Table\ReconfirmEmailTable;

if ( ! ( defined( '_VALID_CB' ) || defined( '_JEXEC' ) || defined( '_VALID_MOS' ) ) ) { die( 'Direct Access to this location is not allowed.' ); }

global $_PLUGINS;

$_PLUGINS->loadPluginGroup( 'user' );

/** @noinspection AutoloadingIssuesInspection */
class CBplug_cbreconfirmemail extends cbPluginHandler
{
	/**
	 * @param null|TabTable  $tab
	 * @param null|UserTable $user
	 * @param null|int       $ui
	 * @param null|array     $postdata
	 */
	public function getCBpluginComponent( ?TabTable $tab, ?UserTable $user, ?int $ui, ?array $postdata ): void
	{
		$action			=	$this->getInput()->getString( 'action', '' );
		$confirmcode	=	$this->getInput()->getString( 'confirmcode', '' );

		if ( ! $confirmcode ) {
			cbRedirect( 'index.php', CBTxt::T( 'Confirm code missing.' ), 'error' );
		}

		$row			=	new ReconfirmEmailTable();

		$row->load( [ 'code' => $confirmcode ] );

		if ( ! $row->getInt( 'id', 0 ) ) {
			cbRedirect( 'index.php', CBTxt::T( 'Confirm code missing.' ), 'error' );
		}

		if ( ! $row->getInt( 'user_id', 0 ) ) {
			cbRedirect( 'index.php', CBTxt::T( 'User not associated with confirm code.' ), 'error' );
		}

		if ( $row->expired() ) {
			cbRedirect( 'index.php', CBTxt::T( 'Email address change request has expired.' ), 'error' );
		}

		if ( $row->getString( 'status', '' ) === 'X' ) {
			cbRedirect( 'index.php', CBTxt::T( 'Email address change request has already been cancelled.' ), 'notice' );
		} elseif ( $row->getString( 'status', '' ) !== 'P' ) {
			cbRedirect( 'index.php', CBTxt::T( 'Email address has already been confirmed' ), 'notice' );
		}

		switch ( $action ) {
			case 'cancel':
				$this->cancelEmailChange( $row );
				break;
			case 'confirm':
			default:
				$this->confirmEmailChange( $row );
				break;
		}
	}

	/**
	 * @param ReconfirmEmailTable $row
	 * @return void
	 */
	private function confirmEmailChange( ReconfirmEmailTable $row ): void
	{
		global $_PLUGINS;

		$cbUser		=	CBuser::getInstance( $row->getInt( 'user_id', 0 ), false );
		$user		=	$cbUser->getUserData();

		Helper::setEmailChanged( $row->getInt( 'user_id', 0 ), $row->getString( 'to', '' ), $user->getString( 'email', '' ) );

		$_PLUGINS->trigger( 'reconfirm_onBeforeConfirm', [ &$user, &$row, $row->getString( 'code', '' ) ] );

		if ( ! $row->confirm() ) {
			// Force a failed confirmation to cancelled as these are single-use URLs
			$row->set( 'status', 'X' );
			$row->store();

			cbRedirect( 'index.php', CBTxt::T( 'FAILED_RECONFIRM_EMAIL', 'Failed to confirm new email address! Error: [error]', [ '[error]' => $row->getError() ] ), 'error' );
		}

		$_PLUGINS->trigger( 'reconfirm_onAfterConfirm', [ $user, $row, $row->getString( 'code', '' ) ] );

		$redirect	=	$cbUser->replaceUserVars( $this->params->getString( 'reconfirm_redirect', 'index.php' ), false, false, null, false );
		$message	=	$cbUser->replaceUserVars( CBTxt::T( $this->params->getHtml( 'reconfirm_message', 'New email address confirmed successfully!' ) ), true, false, null, false );

		if ( $redirect ) {
			cbRedirect( cbSef( $redirect, false ), $message );
		} else {
			echo '<div class="cbReconfirmEmail cbReconfirmEmailConfirmed">' . $message . '</div>';
		}
	}

	/**
	 * @param ReconfirmEmailTable $row
	 * @return void
	 */
	private function cancelEmailChange( ReconfirmEmailTable $row ): void
	{
		global $_PLUGINS;

		$cbUser		=	CBuser::getInstance( $row->getInt( 'user_id', 0 ), false );
		$user		=	$cbUser->getUserData();

		$_PLUGINS->trigger( 'reconfirm_onBeforeCancel', [ &$user, &$row, $row->getString( 'code', '' ) ] );

		if ( ! $row->cancel() ) {
			cbRedirect( 'index.php', CBTxt::T( 'FAILED_CANCEL_RECONFIRM_EMAIL', 'Failed to cancel new email address! Error: [error]', [ '[error]' => $row->getError() ] ), 'error' );
		}

		$_PLUGINS->trigger( 'reconfirm_onAfterCancel', [ $user, $row, $row->getString( 'code', '' ) ] );

		$redirect	=	$cbUser->replaceUserVars( $this->params->getString( 'reconfirm_cancel_redirect', '' ), false, false, null, false );
		$message	=	$cbUser->replaceUserVars( CBTxt::T( $this->params->getHtml( 'reconfirm_cancel_message', 'Email address change cancelled successfully!' ) ), true, false, null, false );

		if ( $redirect ) {
			cbRedirect( cbSef( $redirect, false ), $message );
		} else {
			echo '<div class="cbReconfirmEmail cbReconfirmEmailCancelled">' . $message . '</div>';
		}
	}
}