(() => {
	'use strict';

	/**
	 * @property {object} cbantispam
	 * @property {string} cbantispam.url
	 * @property {object} cbantispam.recaptcha
	 * @property {string} cbantispam.recaptcha.theme
	 * @property {string} cbantispam.recaptcha.size
	 * @property {object} cbantispam.recaptcha.badge
	 * @property {string} cbantispam.recaptcha.badge.v3
	 * @property {string} cbantispam.recaptcha.badge.invisible
	 * @property {string} cbantispam.recaptcha.v2
	 * @property {string} cbantispam.recaptcha.v3
	 * @property {string} cbantispam.recaptcha.invisible
	 *
	 * @property {object} grecaptcha
	 * @property {function} grecaptcha.render
	 * @property {function} grecaptcha.reset
	 * @property {function} grecaptcha.getResponse
	 * @property {function} grecaptcha.ready
	 * @property {function} grecaptcha.execute
	 */

	if ( typeof cbantispam === 'undefined' ) {
		console.error( 'CB Antispam: Captcha Unable to Load: Missing Configuration!' );

		return;
	}

	function loadInternalCaptcha() {
		const id = this.dataset.cbantispamCaptcha;
		const url = new URL( cbantispam.url );

		url.searchParams.set( 'id', id );

		fetch( url ).then( response => {
			return response.json();
		}).then( data => {
			/**
			 * @param {object} data
			 * @param {string} data.input
			 * @param {string} data.image
			 * @param {string} data.audio
			 * @param {string} data.captcha
			 */
			if ( data.input ) {
				document.querySelectorAll( 'span.cbantispamCaptchaInput[data-cbantispam-captcha="' + id + '"]' ).forEach( input => {
					input.replaceWith( getHTMLResponse( data.input ) );
				});
			}

			if ( data.image ) {
				document.querySelectorAll( 'span.cbantispamCaptchaImage[data-cbantispam-captcha="' + id + '"]' ).forEach( img => {
					img.replaceWith( getHTMLResponse( data.image ) );
				});
			}

			if ( data.audio ) {
				document.querySelectorAll( 'span.cbantispamCaptchaAudioFile[data-cbantispam-captcha="' + id + '"]' ).forEach( audio => {
					audio.replaceWith( getHTMLResponse( data.audio ) );
				});
			}

			if ( data.captcha ) {
				document.querySelectorAll( 'span.cbantispamCaptcha[data-cbantispam-captcha="' + id + '"]' ).forEach( captcha => {
					const html = getHTMLResponse( data.captcha );

					captcha.replaceWith( html );

					html.querySelectorAll( '.cbantispamCaptchaRefresh' ).forEach( el => {
						el.addEventListener( 'click', refreshInternalCaptcha );
					});

					html.querySelectorAll( '.cbantispamCaptchaAudio' ).forEach( el => {
						el.addEventListener( 'click', listenInternalCaptcha );
					});
				});
			}
		}).catch( ( error ) => {
			console.error( 'CB Antispam: Captcha Unable to Load: ' + error );
		});
	}

	function getHTMLResponse( html ) {
		const container = document.createElement( 'div' );

		container.innerHTML = html;

		return container.firstElementChild;
	}

	function refreshInternalCaptcha() {
		const captcha = this.closest( '.cbantispamCaptcha' );
		const id = captcha.dataset.cbantispamCaptcha;
		const input = captcha.querySelector( 'input.cbantispamCaptchaInput' );
		const url = captcha.querySelector( 'img.cbantispamCaptchaImage' ).getAttribute( 'src' ).replace( /(&ver=[0-9]+)*/ig, '' );
		const ver = Math.floor( Math.random() * 10000 );

		const audio = document.querySelectorAll( 'audio.cbantispamCaptchaAudioFile' + id );
		const inputs = document.querySelectorAll( 'input.cbantispamCaptchaInput' + id );

		document.querySelectorAll( 'img.cbantispamCaptchaImage' + id ).forEach( img => {
			img.setAttribute( 'src', url + '&ver=' + ver );

			audio.forEach( audio => {
				if ( audio.paused ) {
					return;
				}

				audio.pause();
			});

			inputs.forEach( input => {
				input.value = '';
			});

			img.addEventListener( 'load', () => {
				audio.forEach( audio => {
					audio.setAttribute( 'src', audio.getAttribute( 'src' ).replace( /(&ver=[0-9]+)*/ig, '' ) + '&ver=' + ver );
				});

				if ( input ) {
					input.focus();
				}
			}, {
				once: true,
			});
		});

		const formRow = this.closest( '.form-group' );

		if ( formRow ) {
			formRow.classList.remove( 'cbValidationError' );

			const rowLabel = formRow.querySelector( '.col-form-label' );

			if ( rowLabel ) {
				rowLabel.classList.remove( 'cbValidationError' );
				rowLabel.classList.remove( 'text-danger' );
			}

			const rowInput = formRow.querySelector( '.form-control' );

			if ( rowInput ) {
				rowInput.classList.remove( 'cbValidationError' );
				rowInput.classList.remove( 'is-invalid' );
			}

			const rowMsg = formRow.querySelector( '.cbValidationMessage' );

			if ( rowMsg ) {
				rowMsg.remove();
			}
		}
	}

	function listenInternalCaptcha() {
		const captcha = this.closest( '.cbantispamCaptcha' );
		const audio = captcha.querySelector( 'audio.cbantispamCaptchaAudioFile' );

		if ( audio ) {
			if ( audio.paused ) {
				audio.currentTime = 0;

				audio.play();
			} else {
				audio.pause();
			}

			audio.addEventListener( 'play', () => {
				this.classList.remove( 'fa-volume-up' );
				this.classList.add( 'fa-pause' );
			}, {
				once: true,
			});

			audio.addEventListener( 'pause', () => {
				this.classList.add( 'fa-volume-up' );
				this.classList.remove( 'fa-pause' );
			}, {
				once: true,
			});
		}
	}

	function loadReCaptchaV2() {
		const id = this.dataset.cbantispamCaptcha;
		const widget = grecaptcha.render( this, {
			'sitekey': cbantispam.recaptcha.v2,
			'theme': cbantispam.recaptcha.theme,
			'size': cbantispam.recaptcha.size,
		});

		if ( this.classList.contains( 'preview' ) ) {
			return;
		}

		const form = this.closest( 'form' );

		if ( ! form ) {
			return;
		}

		if ( form.classList.contains( 'cbValidation' ) ) {
			const jQuery = getjQuery();

			if ( ! jQuery ) {
				return;
			}

			this.querySelector( '#g-recaptcha-response' ).setAttribute( 'data-rule-cbrecaptcha' + id, 'true' );

			jQuery.validator.addMethod( 'cbrecaptcha' + id, ( value, element, params ) => {
				return ( !! grecaptcha.getResponse( widget ) );
			}, jQuery.validator.messages['required'] );

			return;
		}

		form.addEventListener( 'submit', event => {
			if ( grecaptcha.getResponse( widget ) ) {
				return;
			}

			event.preventDefault();
		});
	}

	function getjQuery()
	{
		if ( typeof window.cbjQuery !== 'undefined' ) {
			return window.cbjQuery;
		}

		return null;
	}

	function loadReCaptchaV3() {
		const id = this.dataset.cbantispamCaptcha;
		const form = this.closest( 'form' );
		const jQuery = getjQuery();
		const widget = grecaptcha.render( this, {
			'sitekey': cbantispam.recaptcha.v3,
			'badge': cbantispam.recaptcha.badge.v3,
			'size': 'invisible',
			'expired-callback': () => {
				if ( this.classList.contains( 'preview' ) ) {
					return;
				}

				this.classList.remove( 'validated' );
				this.classList.remove( 'requested' );
			}
		});

		if ( this.classList.contains( 'preview' ) ) {
			return;
		}

		if ( ! form ) {
			grecaptcha.ready( () => {
				grecaptcha.execute( widget, { action: 'cbantispam' } ).then( token => {
					document.querySelectorAll( 'input.cbantispamCaptchaInput' + id ).forEach( input => {
						input.value = token;
					});
				});
			});

			return;
		}

		form.addEventListener( 'submit', event => {
			if ( form.classList.contains( 'cbValidation' ) && jQuery && ( ! jQuery( form ).cbvalidate( 'valid' ) ) ) {
				return;
			}

			if ( this.classList.contains( 'validated' ) ) {
				return
			}

			event.preventDefault();

			if ( this.classList.contains( 'requested' ) ) {
				return
			}

			this.classList.add( 'requested' );

			grecaptcha.ready( () => {
				grecaptcha.execute( widget, { action: 'cbantispam' } ).then( token => {
					this.classList.add( 'validated' );

					document.querySelectorAll( 'input.cbantispamCaptchaInput' + id ).forEach( input => {
						input.value = token;
					});

					if ( form ) {
						if ( form.classList.contains( 'cbValidation' ) && jQuery ) {
							jQuery( form ).submit();
						} else {
							form.submit();
						}
					}
				});
			});
		});
	}

	function loadReCaptchaInvisible() {
		const id = this.dataset.cbantispamCaptcha;
		const form = this.closest( 'form' );
		const jQuery = getjQuery();
		const widget = grecaptcha.render( this, {
			'sitekey': cbantispam.recaptcha.invisible,
			'badge': cbantispam.recaptcha.badge.invisible,
			'size': 'invisible',
			'callback': token => {
				if ( this.classList.contains( 'preview' ) ) {
					return;
				}

				this.classList.add( 'validated' );

				document.querySelectorAll( 'input.cbantispamCaptchaInput' + id ).forEach( input => {
					input.value = token;
				});

				if ( form ) {
					if ( form.classList.contains( 'cbValidation' ) && jQuery ) {
						jQuery( form ).submit();
					} else {
						form.submit();
					}
				}
			},
			'expired-callback': () => {
				if ( this.classList.contains( 'preview' ) ) {
					return;
				}

				this.classList.remove( 'validated' );
				this.classList.remove( 'requested' );
			}
		});

		if ( this.classList.contains( 'preview' ) ) {
			return;
		}

		if ( ! form ) {
			grecaptcha.execute( widget );

			return;
		}

		form.addEventListener( 'submit', event => {
			if ( form.classList.contains( 'cbValidation' ) && jQuery && ( ! jQuery( form ).cbvalidate( 'valid' ) ) ) {
				return;
			}

			if ( this.classList.contains( 'validated' ) ) {
				return
			}

			event.preventDefault();

			if ( this.classList.contains( 'requested' ) ) {
				return;
			}

			this.classList.add( 'requested' );

			grecaptcha.execute( widget );
		});
	}

	window.cbAntiSpamRecaptcha = () => {
		// v2
		document.querySelectorAll( '.cbantispamRecaptcha' ).forEach( el => {
			loadReCaptchaV2.call( el );
		});

		// v3
		document.querySelectorAll( '.cbantispamRecaptchaScore' ).forEach( el => {
			loadReCaptchaV3.call( el );
		});

		// Invisible
		document.querySelectorAll( '.cbantispamRecaptchaInvisible' ).forEach( el => {
			loadReCaptchaInvisible.call( el );
		});
	}

	document.addEventListener( 'DOMContentLoaded', () => {
		document.querySelectorAll( 'span.cbantispamCaptcha[data-cbantispam-captcha]' ).forEach( el => {
			loadInternalCaptcha.call( el );
		});

		document.querySelectorAll( '.cbantispamCaptchaRefresh' ).forEach( el => {
			el.addEventListener( 'click', refreshInternalCaptcha );
		});

		document.querySelectorAll( '.cbantispamCaptchaAudio' ).forEach( el => {
			el.addEventListener( 'click', listenInternalCaptcha );
		});
	});
})();