<?php
/**
* Community Builder (TM)
* @version $Id: $
* @package CommunityBuilder
* @copyright (C) 2004-2022 www.joomlapolis.com / Lightning MultiCom SA - and its licensors, all rights reserved
* @license http://www.gnu.org/licenses/old-licenses/gpl-2.0.html GNU/GPL version 2
*/

namespace CB\Plugin\AntiSpam;

use CB\Plugin\AntiSpam\Table\AttemptTable;
use CB\Plugin\AntiSpam\Table\LogTable;
use CBLib\Application\Application;
use CB\Database\Table\UserTable;
use CBLib\Language\CBTxt;
use CB\Plugin\AntiSpam\Table\BlockTable;
use CB\Plugin\AntiSpam\Table\WhitelistTable;
use CBLib\Registry\Registry;
use GuzzleHttp\Client;

\defined('CBLIB') or die();

class CBAntiSpam
{

	/**
	 * @return Registry
	 */
	public static function getGlobalParams(): Registry
	{
		global $_PLUGINS;

		static $params	=	null;

		if ( ! $params ) {
			$plugin		=	$_PLUGINS->getLoadedPlugin( 'user', 'cbantispam' );
			$params		=	new Registry();

			if ( $plugin ) {
				$params->load( $plugin->params );
			}
		}

		return $params;
	}

	/**
	 * @param null|string $file
	 * @param bool|array  $headers
	 * @return null|string
	 */
	public static function getTemplate( ?string $file = null, $headers = [ 'template', 'override' ] ): ?string
	{
		global $_CB_framework, $_PLUGINS;

		$plugin							=	$_PLUGINS->getLoadedPlugin( 'user', 'cbantispam' );

		if ( ! $plugin ) {
			return null;
		}

		$template						=	self::getGlobalParams()->getString( 'general_template', 'default' );

		if ( ! $template ) {
			$template					=	'default';
		}

		$livePath						=	$_PLUGINS->getPluginLivePath( $plugin );
		$absPath						=	$_PLUGINS->getPluginPath( $plugin );

		$file							=	preg_replace( '/[^-a-zA-Z0-9_]/', '', $file );
		$return							=	null;

		if ( $file ) {
			if ( $headers !== false ) {
				$headers[]				=	$file;
			}

			$php						=	$absPath . '/templates/' . $template . '/' . $file . '.php';

			if ( ! file_exists( $php ) ) {
				$php					=	$absPath . '/templates/default/' . $file . '.php';
			}

			if ( file_exists( $php ) ) {
				$return					=	$php;
			}
		}

		if ( $headers !== false ) {
			static $loaded				=	[];

			$loaded[$template]			=	[];

			// Global CSS File:
			if ( \in_array( 'template', $headers, true ) && ( ! \in_array( 'template', $loaded[$template], true ) ) ) {
				$global					=	'/templates/' . $template . '/template.css';

				if ( ! file_exists( $absPath . $global ) ) {
					$global				=	'/templates/default/template.css';
				}

				if ( file_exists( $absPath . $global ) ) {
					$_CB_framework->document->addHeadStyleSheet( $livePath . $global );
				}

				$loaded[$template][]	=	'template';
			}

			// File or Custom CSS/JS Headers:
			foreach ( $headers as $header ) {
				if ( \in_array( $header, $loaded[$template], true ) || \in_array( $header, [ 'template', 'override' ], true ) ) {
					continue;
				}

				$header					=	preg_replace( '/[^-a-zA-Z0-9_]/', '', $header );

				if ( ! $header ) {
					continue;
				}

				$css					=	'/templates/' . $template . '/' . $header . '.css';
				$js						=	'/templates/' . $template . '/' . $header . '.js';

				if ( ! file_exists( $absPath . $css ) ) {
					$css				=	'/templates/default/' . $header . '.css';
				}

				if ( file_exists( $absPath . $css ) ) {
					$_CB_framework->document->addHeadStyleSheet( $livePath . $css );
				}

				if ( ! file_exists( $absPath . $js ) ) {
					$js					=	'/templates/default/' . $header . '.js';
				}

				if ( file_exists( $absPath . $js ) ) {
					$_CB_framework->document->addHeadScriptUrl( $livePath . $js );
				}

				$loaded[$template][]	=	$header;
			}

			// Override CSS File:
			if ( \in_array( 'override', $headers, true ) && ( ! \in_array( 'override', $loaded[$template], true ) ) ) {
				$override				=	'/templates/' . $template . '/override.css';

				if ( file_exists( $absPath . $override ) ) {
					$_CB_framework->document->addHeadStyleSheet( $livePath . $override );
				}

				$loaded[$template][]	=	'override';
			}
		}

		return $return;
	}

	/**
	 * Returns the current return url or generates one from current page
	 *
	 * @param bool $current
	 * @param bool $raw
	 * @return null|string
	 */
	public static function getReturn( bool $current = false, bool $raw = false ): ?string
	{
		static $cache				=	[];

		if ( ! isset( $cache[$current] ) ) {
			$url					=	null;

			if ( $current ) {
				$returnUrl			=	Application::Input()->getBase64( 'get/return', '' );

				if ( $returnUrl ) {
					$returnUrl		=	base64_decode( $returnUrl );

					if ( \JUri::isInternal( $returnUrl ) || ( $returnUrl[0] === '/' ) ) {
						$url		=	$returnUrl;
					}
				}
			} else {
				$isHttps			=	( isset( $_SERVER['HTTPS'] ) && ( ! empty( $_SERVER['HTTPS'] ) ) && ( $_SERVER['HTTPS'] !== 'off' ) );
				$returnUrl			=	'http' . ( $isHttps ? 's' : '' ) . '://' . $_SERVER['HTTP_HOST'];

				if ( ( ! empty( $_SERVER['PHP_SELF'] ) ) && ( ! empty( $_SERVER['REQUEST_URI'] ) ) ) {
					$returnUrl		.=	$_SERVER['REQUEST_URI'];
				} else {
					$returnUrl		.=	$_SERVER['SCRIPT_NAME'];

					if ( isset( $_SERVER['QUERY_STRING'] ) && ( ! empty( $_SERVER['QUERY_STRING'] ) ) ) {
						$returnUrl	.=	'?' . $_SERVER['QUERY_STRING'];
					}
				}

				$url				=	cbUnHtmlspecialchars( preg_replace( '/[\\\"\\\'][\\s]*javascript:(.*)[\\\"\\\']/', '""', preg_replace( '/eval\((.*)\)/', '', htmlspecialchars( urldecode( $returnUrl ) ) ) ) );
			}

			$cache[$current]		=	$url;
		}

		$return						=	$cache[$current];

		if ( ( ! $raw ) && $return ) {
			$return					=	base64_encode( $return );
		}

		return $return;
	}

	/**
	 * Redirects to the return url if available otherwise to the url specified
	 *
	 * @param string      $url
	 * @param null|string $message
	 * @param string      $messageType
	 */
	public static function returnRedirect( string $url, ?string $message = null, string $messageType = 'message' ): void
	{
		cbRedirect( ( self::getReturn( true, true ) ?: $url ), $message, $messageType );
	}

	/**
	 * Returns the domain portion of an email address
	 *
	 * @param string|UserTable $email
	 * @return string
	 */
	public static function getEmailDomain( $email ): string
	{
		if ( $email instanceof UserTable ) {
			$email			=	$email->getString( 'email', '' );
		}

		if ( ! $email ) {
			return '';
		}

		$emailParts			=	explode( '@', $email );
		$emailDomain		=	null;

		if ( \count( $emailParts ) > 1 ) {
			$emailDomain	=	array_pop( $emailParts );
		}

		return $emailDomain;
	}

	/**
	 * Returns a users current ip address
	 *
	 * @param int|string|UserTable $user
	 * @return string
	 */
	public static function getUserIP( $user = null ): string
	{
		global $_CB_database;

		static $cache		=	[];

		if ( $user === null ) {
			$user			=	\CBuser::getMyUserDataInstance();
		} elseif ( ! ( $user instanceof UserTable ) ) {
			$user			=	\CBuser::getUserDataInstance( (int) $user );
		}

		$userId				=	$user->getInt( 'id', 0 );

		if ( ! $userId ) {
			if ( ! Application::MyUser()->getUserId() ) {
				return (string) Application::Input()->getRequestIP();
			}

			return '';
		}

		if ( \array_key_exists( $userId, $cache ) ) {
			return $cache[$userId];
		}

		$query				=	'SELECT *'
							.	"\n FROM " . $_CB_database->NameQuote( '#__comprofiler_plugin_antispam_log' )
							.	"\n WHERE " . $_CB_database->NameQuote( 'user_id' ) . " = " . $user->getInt( 'id', 0 )
							.	"\n ORDER BY " . $_CB_database->NameQuote( 'date' ) . " DESC";
		$_CB_database->setQuery( $query, 0, 1 );
		$log				=	new LogTable();
		$_CB_database->loadObject( $log );

		if ( ! $log->getInt( 'id', 0 ) ) {
			$ipAddress		=	$user->getString( 'registeripaddr', '' );
		} else {
			$ipAddress		=	$log->getString( 'ip_address', '' );
		}

		if ( ( ! $ipAddress ) && ( $userId === Application::MyUser()->getUserId() ) ) {
			$ipAddress		=	(string) Application::Input()->getRequestIP();
		}

		$cache[$userId]		=	$ipAddress;

		return $ipAddress;
	}

	/**
	 * Returns the users sessions count
	 *
	 * @param int $userId
	 * @return int
	 */
	public static function countUserSessions( int $userId ): int
	{
		global $_CB_database;

		if ( ! $userId ) {
			return 0;
		}

		static $cache		=	[];

		if ( \array_key_exists( $userId, $cache ) ) {
			return $cache[$userId];
		}

		$query				=	'SELECT COUNT(*)'
							.	"\n FROM " . $_CB_database->NameQuote( '#__session' )
							.	"\n WHERE " . $_CB_database->NameQuote( 'userid' ) . " = " . $userId;
		$_CB_database->setQuery( $query );
		$count				=	(int) $_CB_database->loadResult();

		$cache[$userId]		=	$count;

		return $count;
	}

	/**
	 * Checks if user or ip address can be blocked
	 *
	 * @param int|string|UserTable $user
	 * @param null|string          $ipAddress
	 * @return bool
	 */
	public static function isBlockable( $user = null, ?string $ipAddress = null ): bool
	{
		$params				=	self::getGlobalParams();

		if ( ! $params->getBool( 'general_block', true ) ) {
			return false;
		}

		if ( $user === null ) {
			$user			=	\CBuser::getMyUserDataInstance();
		} elseif ( ! ( $user instanceof UserTable ) ) {
			$user			=	\CBuser::getUserDataInstance( (int) $user );
		}

		if ( Application::User( $user->getInt( 'id', 0 ) )->isGlobalModerator() ) {
			return false;
		}

		if ( ! $params->getBool( 'general_whitelist', true ) ) {
			return true;
		}

		return ( ! self::getWhitelist( $user, $ipAddress ) );
	}

	/**
	 * Returns the user or ip addresses most recent block
	 *
	 * @param null|int|UserTable $user
	 * @param null|string        $ipAddress
	 * @return null|BlockTable
	 */
	public static function getBlock( $user = null, ?string $ipAddress = null ): ?BlockTable
	{
		global $_CB_database;

		if ( ! self::getGlobalParams()->getBool( 'general_block', true ) ) {
			return null;
		}

		static $cache			=	[];

		if ( $user === null ) {
			$user				=	\CBuser::getMyUserDataInstance();
		} elseif ( ! ( $user instanceof UserTable ) ) {
			$user				=	\CBuser::getUserDataInstance( (int) $user );
		}

		$userId					=	$user->getInt( 'id', 0 );
		$email					=	$user->getString( 'email', '' );

		if ( $ipAddress === null ) {
			$ipAddress			=	self::getUserIP( $user );
		}

		if ( ( ! $userId ) && ( ! $email ) && ( ! $ipAddress ) ) {
			return null;
		}

		$cacheId				=	$userId . ':' . $ipAddress;

		if ( \array_key_exists( $cacheId, $cache ) ) {
			return $cache[$cacheId];
		}

		if ( ! self::isBlockable( $user, $ipAddress ) ) {
			$cache[$cacheId]	=	null;

			return null;
		}

		$where					=	[];

		if ( $user->getInt( 'id', 0 ) ) {
			$where[]			.=	"( " . $_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'user' )
								.	" AND " . $_CB_database->NameQuote( 'value' ) . " = " . $user->getInt( 'id', 0 ) . " )";
		}

		$emailDomain			=	'';

		if ( $email ) {
			$emailDomain		=	self::getEmailDomain( $user );

			$where[]			.=	"( " . $_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'email' )
								.	" AND " . $_CB_database->NameQuote( 'value' ) . " = " . $_CB_database->Quote( $email ) . " )";

			$where[]			.=	"( " . $_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'email_partial' )
								.	" AND " . $_CB_database->Quote( $email ) . " LIKE " . $_CB_database->NameQuote( 'value' ) . " )";

			$where[]			.=	$_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'email_list' );

			if ( $emailDomain ) {
				$where[]		.=	"( " . $_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'domain' )
								.	" AND " . $_CB_database->NameQuote( 'value' ) . " = " . $_CB_database->Quote( $emailDomain ) . " )";

				$where[]		.=	"( " . $_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'domain_partial' )
								.	" AND " . $_CB_database->Quote( $emailDomain ) . " LIKE " . $_CB_database->NameQuote( 'value' ) . " )";

				$where[]		.=	$_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'domain_list' );
			}
		}

		if ( $ipAddress ) {
			$where[]			.=	"( " . $_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'ip' )
								.	" AND " . $_CB_database->NameQuote( 'value' ) . " = " . $_CB_database->Quote( $ipAddress ) . " )";

			$where[]			.=	"( " . $_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'ip_partial' )
								.	" AND " . $_CB_database->Quote( $ipAddress ) . " LIKE " . $_CB_database->NameQuote( 'value' ) . " )";

			$where[]			.=	"( " . $_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'ip_range' )
								.	" AND ( INET_ATON( " . $_CB_database->Quote( $ipAddress ) . " ) BETWEEN INET_ATON( SUBSTRING_INDEX( " . $_CB_database->NameQuote( 'value' ) . ", ':', 1 ) ) AND INET_ATON( SUBSTRING_INDEX( " . $_CB_database->NameQuote( 'value' ) . ", ':', -1 ) ) ) )";

			$where[]			.=	$_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'ip_list' );
		}

		$blocked				=	null;

		if ( $where ) {
			$query				=	"SELECT *"
								.	"\n FROM " . $_CB_database->NameQuote( '#__comprofiler_plugin_antispam_block' )
								.	"\n WHERE " . implode( " OR ", $where );
			$_CB_database->setQuery( $query );
			$blocks				=	$_CB_database->loadObjectList( 'id', BlockTable::class, [ $_CB_database ] );

			/** @var BlockTable[] $blocks */
			foreach ( $blocks as $row ) {
				if ( $row->isExpired() ) {
					continue;
				}

				if ( \in_array( $row->getType(), [ 'ip_list', 'email_list', 'domain_list' ], true ) ) {
					switch ( $row->getType() ) {
						case 'ip_list':
							if ( ! \in_array( $ipAddress, $row->getList(), true ) ) {
								continue 2;
							}
							break;
						case 'email_list':
							if ( ! \in_array( $email, $row->getList(), true ) ) {
								continue 2;
							}
							break;
						case 'domain_list':
							if ( ! \in_array( $emailDomain, $row->getList(), true ) ) {
								continue 2;
							}
							break;
					}
				}

				$blocked		=	$row;
				break;
			}
		}

		$cache[$cacheId]		=	$blocked;

		return $blocked;
	}

	/**
	 * Returns the user or ip addresses most recent whitelist
	 *
	 * @param null|int|UserTable $user
	 * @param null|string        $ipAddress
	 * @return null|WhitelistTable
	 */
	public static function getWhitelist( $user = null, ?string $ipAddress = null ): ?WhitelistTable
	{
		global $_CB_database;

		if ( ! self::getGlobalParams()->getBool( 'general_whitelist', true ) ) {
			return null;
		}

		static $cache		=	[];

		if ( $user === null ) {
			$user			=	\CBuser::getMyUserDataInstance();
		} elseif ( ! ( $user instanceof UserTable ) ) {
			$user			=	\CBuser::getUserDataInstance( (int) $user );
		}

		$userId				=	$user->getInt( 'id', 0 );
		$email				=	$user->getString( 'email', '' );

		if ( $ipAddress === null ) {
			$ipAddress		=	self::getUserIP( $user );
		}

		if ( ( ! $userId ) && ( ! $email ) && ( ! $ipAddress ) ) {
			return null;
		}

		$cacheId			=	$userId . ':' . $ipAddress;

		if ( \array_key_exists( $cacheId, $cache ) ) {
			return $cache[$cacheId];
		}

		$where				=	[];

		if ( $user->getInt( 'id', 0 ) ) {
			$where[]		.=	"( " . $_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'user' )
							.	" AND " . $_CB_database->NameQuote( 'value' ) . " = " . $user->getInt( 'id', 0 ) . " )";
		}

		$emailDomain		=	'';

		if ( $email ) {
			$emailDomain	=	self::getEmailDomain( $user );

			$where[]		.=	"( " . $_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'email' )
							.	" AND " . $_CB_database->NameQuote( 'value' ) . " = " . $_CB_database->Quote( $email ) . " )";

			$where[]		.=	"( " . $_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'email_partial' )
							.	" AND " . $_CB_database->Quote( $email ) . " LIKE " . $_CB_database->NameQuote( 'value' ) . " )";

			$where[]		.=	$_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'email_list' );

			if ( $emailDomain ) {
				$where[]	.=	"( " . $_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'domain' )
							.	" AND " . $_CB_database->NameQuote( 'value' ) . " = " . $_CB_database->Quote( $emailDomain ) . " )";

				$where[]	.=	"( " . $_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'domain_partial' )
							.	" AND " . $_CB_database->Quote( $emailDomain ) . " LIKE " . $_CB_database->NameQuote( 'value' ) . " )";

				$where[]	.=	$_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'domain_list' );
			}
		}

		if ( $ipAddress ) {
			$where[]		.=	"( " . $_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'ip' )
							.	" AND " . $_CB_database->NameQuote( 'value' ) . " = " . $_CB_database->Quote( $ipAddress ) . " )";

			$where[]		.=	"( " . $_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'ip_partial' )
							.	" AND " . $_CB_database->Quote( $ipAddress ) . " LIKE " . $_CB_database->NameQuote( 'value' ) . " )";

			$where[]		.=	"( " . $_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'ip_range' )
							.	" AND ( INET_ATON( " . $_CB_database->Quote( $ipAddress ) . " ) BETWEEN INET_ATON( SUBSTRING_INDEX( " . $_CB_database->NameQuote( 'value' ) . ", ':', 1 ) ) AND INET_ATON( SUBSTRING_INDEX( " . $_CB_database->NameQuote( 'value' ) . ", ':', -1 ) ) ) )";

			$where[]		.=	$_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'ip_list' );
		}

		$whitelist			=	null;

		if ( $where ) {
			$query			=	"SELECT *"
							.	"\n FROM " . $_CB_database->NameQuote( '#__comprofiler_plugin_antispam_whitelist' )
							.	"\n WHERE " . implode( " OR ", $where );
			$_CB_database->setQuery( $query );
			$whitelists		=	$_CB_database->loadObjectList( 'id', WhitelistTable::class, [ $_CB_database ] );

			/** @var WhitelistTable[] $whitelists */
			/** @noinspection LoopWhichDoesNotLoopInspection */
			foreach ( $whitelists as $row ) {
				if ( \in_array( $row->getType(), [ 'ip_list', 'email_list', 'domain_list' ], true ) ) {
					switch ( $row->getType() ) {
						case 'ip_list':
							if ( ! \in_array( $ipAddress, $row->getList(), true ) ) {
								continue 2;
							}
							break;
						case 'email_list':
							if ( ! \in_array( $email, $row->getList(), true ) ) {
								continue 2;
							}
							break;
						case 'domain_list':
							if ( ! \in_array( $emailDomain, $row->getList(), true ) ) {
								continue 2;
							}
							break;
					}
				}

				$whitelist	=	$row;
				break;
			}
		}

		$cache[$cacheId]	=	$whitelist;

		return $whitelist;
	}

	/**
	 * Logs an ip address use
	 *
	 * @param null|int|UserTable $user
	 * @param string|string[]    $type
	 */
	public static function logIPAddress( $user = null, $type = 'login' ): void
	{
		global $_CB_database;

		$params					=	self::getGlobalParams();

		if ( ( ! $params->getBool( 'general_attempts', true ) ) && ( ! $params->getBool( 'general_log', true ) ) ) {
			return;
		}

		$ipAddress				=	Application::Input()->getRequestIP();

		if ( ! $ipAddress ) {
			return;
		}

		if ( $params->getBool( 'general_attempts', true ) ) {
			$query				=	'SELECT *'
								.	"\n FROM " . $_CB_database->NameQuote( '#__comprofiler_plugin_antispam_attempts' )
								.	"\n WHERE " . $_CB_database->NameQuote( 'ip_address' ) . " = " . $_CB_database->Quote( $ipAddress );
			if ( $type ) {
				$query			.=	"\n AND " . $_CB_database->NameQuote( 'type' ) . ( \is_array( $type ) ? " IN " . $_CB_database->safeArrayOfStrings( $type ) : " = " . $_CB_database->Quote( $type ) );
			}
			$query				.=	"\n ORDER BY " . $_CB_database->NameQuote( 'date' ) . " DESC";
			$_CB_database->setQuery( $query );
			$attempts			=	$_CB_database->loadObjectList( 'id', AttemptTable::class, [ $_CB_database ] );

			/** @var AttemptTable[] $attempts */
			foreach ( $attempts as $attempt ) {
				$attempt->delete();
			}
		}

		if ( ! $params->getBool( 'general_log', true ) ) {
			return;
		}

		if ( $user === null ) {
			$user				=	\CBuser::getMyUserDataInstance();
		} elseif ( ! ( $user instanceof UserTable ) ) {
			$user				=	\CBuser::getUserDataInstance( (int) $user );
		}

		if ( ! $user->getInt( 'id', 0 ) ) {
			return;
		}

		$query					=	'SELECT *'
								.	"\n FROM " . $_CB_database->NameQuote( '#__comprofiler_plugin_antispam_log' )
								.	"\n WHERE " . $_CB_database->NameQuote( 'user_id' ) . " = " . $user->getInt( 'id', 0 )
								.	"\n AND " . $_CB_database->NameQuote( 'ip_address' ) . " = " . $_CB_database->Quote( $ipAddress )
								.	"\n ORDER BY " . $_CB_database->NameQuote( 'date' ) . " DESC";
		$_CB_database->setQuery( $query, 0, 1 );
		$row					=	new LogTable();
		$_CB_database->loadObject( $row );

		if ( ! $row->getId() ) {
			if ( ( $type === 'login' ) || ( \is_array( $type ) && \in_array( 'login', $type, true ) ) ) {
				if ( $params->getBool( 'login_log_notify_user', false ) ) {
					$notify		=	new \cbNotification();

					$extras		=	[	'device'		=>	self::getDevice(),
										'browser'		=>	self::getBrowser(),
										'ip_address'	=>	$ipAddress,
									];

					$subject	=	CBTxt::T( 'New sign-in to your account' );
					$message	=	CBTxt::T( 'We noticed a new sign-in with your account: [username]<br /><br />Device: [device]<br />Browser: [browser]<br />IP Address: <a href="https://ipinfo.io/[ip_address]" target="_blank">[ip_address]</a><br /><br />If this was not you please contact administration and change your password.' );

					$notify->sendFromSystem( $user->getInt( 'id', 0 ), $subject, $message, true, 1, null, null, null, $extras );
				}

				if ( $params->getBool( 'login_log_notify_moderator', false ) ) {
					$notify		=	new \cbNotification();

					$extras		=	[	'device'		=>	self::getDevice(),
										'browser'		=>	self::getBrowser(),
										'ip_address'	=>	$ipAddress,
									];

					$subject	=	CBTxt::T( 'New account sign-in for [username]' );
					$message	=	CBTxt::T( 'There was a new sign-in with the account: [username]<br /><br />Device: [device]<br />Browser: [browser]<br />IP Address: <a href="https://ipinfo.io/[ip_address]" target="_blank">[ip_address]</a>' );

					$notify->sendToModerators( $subject, $message, true, 1, null, null, null, $extras );
				}
			}

			$row->setUserId( $user->getInt( 'id', 0 ) );
			$row->setIpAddress( $ipAddress );
			$row->setCount( 1 );
		} else {
			$row->incrementCount( 1 );
		}

		$row->store();
	}

	/**
	 * Logs an attempt
	 *
	 * @param string $type
	 * @param string $timeframe
	 * @return null|AttemptTable
	 */
	public static function logAttempt( string $type = '', string $timeframe = '' ): ?AttemptTable
	{
		global $_CB_database;

		if ( ! $type ) {
			return null;
		}

		$params					=	self::getGlobalParams();

		if ( ! $params->getBool( 'general_attempts', true ) ) {
			return null;
		}

		$ipAddress				=	Application::Input()->getRequestIP();

		if ( ! $ipAddress ) {
			return null;
		}

		if ( ! $timeframe ) {
			switch ( $type ) {
				case 'login':
					$timeframe	=	$params->getString( 'login_autoblock_timeframe', '-1 MONTH' );
					break;
				case 'forgot':
					$timeframe	=	$params->getString( 'forgot_autoblock_timeframe', '-1 WEEK' );
					break;
				case 'reg':
					$timeframe	=	$params->getString( 'reg_autoblock_timeframe', '-1 MONTH' );
					break;
				case 'captcha':
					$timeframe	=	$params->getString( 'captcha_autoblock_timeframe', '-1 DAY' );
					break;
			}
		}

		$query					=	'SELECT *'
								.	"\n FROM " . $_CB_database->NameQuote( '#__comprofiler_plugin_antispam_attempts' )
								.	"\n WHERE " . $_CB_database->NameQuote( 'ip_address' ) . " = " . $_CB_database->Quote( $ipAddress )
								.	"\n AND " . $_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( $type )
								.	"\n ORDER BY " . $_CB_database->NameQuote( 'date' ) . " DESC";
		$_CB_database->setQuery( $query, 0, 1 );
		$attempt				=	new AttemptTable();
		$_CB_database->loadObject( $attempt );

		if ( ! $attempt->getInt( 'id', 0 ) ) {
			$attempt->setIpAddress( $ipAddress );
			$attempt->setType( $type );
			$attempt->setCount( 1 );
		} elseif ( ( ! $timeframe ) || ( Application::Date( $attempt->getDate(), 'UTC' )->getTimestamp() >= Application::Date( 'now', 'UTC' )->modify( strtoupper( $timeframe ) )->getTimestamp() ) ) {
			// We are counting all failed attempts over a lifetime or the failed attempt is within the specified timeframe (e.g. new failed attempt is within 1 month of the previous):
			$attempt->incrementCount( 1 );
		} else {
			// We're outside the timeframe limit so reset the attempt count:
			$attempt->setCount( 1 );
		}

		$attempt->store();

		return $attempt;
	}

	/**
	 * Attempts to block based off attempt
	 *
	 * @param AttemptTable $attempt
	 * @return null|BlockTable
	 */
	public static function blockAttempt( AttemptTable $attempt ): ?BlockTable
	{
		if ( ! self::isBlockable( null, $attempt->getIpAddress() ) ) {
			return null;
		}

		$params					=	self::getGlobalParams();
		$block					=	false;
		$duration				=	'+1 HOUR';
		$reason					=	null;
		$method					=	0;

		switch ( $attempt->getType() ) {
			case 'login':
				$duration		=	$params->getString( 'login_autoblock_dur', '+1 HOUR' );
				$reason			=	$params->getString( 'login_autoblock_reason', 'Too many failed login attempts.' );
				$method			=	$params->getInt( 'login_autoblock_method', 0 );

				if ( $params->getBool( 'login_autoblock', false ) ) {
					$count		=	$params->getInt( 'login_autoblock_count', 5 );

					if ( ! $count ) {
						$count	=	5;
					}

					if ( $attempt->getCount() >= $count ) {
						$block	=	true;
					}
				}
				break;
			case 'forgot':
				$duration		=	$params->getString( 'forgot_autoblock_dur', '+1 HOUR' );
				$reason			=	$params->getString( 'forgot_autoblock_reason', 'Too many forgot login attempts.' );
				$method			=	$params->getInt( 'forgot_autoblock_method', 0 );

				if ( $params->getBool( 'forgot_autoblock', false ) ) {
					$count		=	$params->getInt( 'forgot_autoblock_count', 5 );

					if ( ! $count ) {
						$count	=	5;
					}

					if ( $attempt->getCount() >= $count ) {
						$block	=	true;
					}
				}
				break;
			case 'reg':
				$duration		=	$params->getString( 'reg_autoblock_dur', '+1 HOUR' );
				$reason			=	$params->getString( 'reg_autoblock_reason', 'Too many failed registration attempts.' );
				$method			=	$params->getInt( 'reg_autoblock_method', 0 );

				if ( $params->getBool( 'reg_autoblock', false ) ) {
					$count		=	$params->getInt( 'reg_autoblock_count', 5 );

					if ( ! $count ) {
						$count	=	5;
					}

					if ( $attempt->getCount() >= $count ) {
						$block	=	true;
					}
				}
				break;
			case 'captcha':
				$duration		=	$params->getString( 'captcha_autoblock_dur', '+1 HOUR' );
				$reason			=	$params->getString( 'captcha_autoblock_reason', 'Too many failed captcha attempts.' );
				$method			=	$params->getInt( 'captcha_autoblock_method', 0 );

				if ( $params->getBool( 'captcha_autoblock', false ) ) {
					$count		=	$params->getInt( 'captcha_autoblock_count', 20 );

					if ( ! $count ) {
						$count	=	20;
					}

					if ( $attempt->getCount() >= $count ) {
						$block	=	true;
					}
				}
				break;
		}

		if ( ! $block ) {
			return null;
		}

		$row					=	new BlockTable();

		$row->setType( 'ip' );
		$row->setValue( $attempt->getIpAddress() );
		$row->setDuration( $duration );
		$row->setReason( $reason );

		if ( $method ) {
			$row->store();
		}

		return $row;
	}

	/**
	 * Returns a preview of captcha output
	 *
	 * @return string
	 */
	public function previewCaptcha(): string
	{
		return ( new Captcha( 'preview' ) )->getCaptcha( false );
	}

	/**
	 * Returns internal clean up urls
	 *
	 * @param string $name
	 * @return string
	 */
	public function loadCleanUpURL( string $name ): string
	{
		global $_CB_framework;

		switch( $name ) {
			case 'cleanup_attempts':
				$function		=	'attempts';
				break;
			case 'cleanup_log':
				$function		=	'log';
				break;
			case 'cleanup_block':
				$function		=	'block';
				break;
			case 'cleanup_all':
			default:
				$function		=	'all';
				break;
		}

		return '<a href="' . $_CB_framework->pluginClassUrl( 'cbantispam', true, [ 'action' => 'prune', 'func' => $function, 'token' => md5( $_CB_framework->getCfg( 'secret' ) ) ], 'raw', 0, true ) . '" target="_blank">' . CBTxt::T( 'Click to Process' ) . '</a>';
	}

	/**
	 * Generate and output a simple math equation
	 *
	 * @param int $value
	 * @param int $leftMin
	 * @param int $leftMax
	 * @param int $rightMin
	 * @param int $rightMax
	 * @return string
	 */
	public static function getMathEquation( int &$value = 0, int $leftMin = 1, int $leftMax = 10, int $rightMin = 1, int $rightMax = 10 ): string
	{
		if ( $leftMin < 1 ) {
			$leftMin			=	1;
		}

		if ( $leftMax <= $leftMin ) {
			$leftMax			=	( $leftMin + 1 );
		}

		if ( $rightMin < 1 ) {
			$rightMin			=	1;
		}

		if ( $rightMax <= $rightMin ) {
			$rightMax			=	( $rightMin + 1 );
		}

		$operators				=	[ '+', 'add', 'plus', '-', 'subtract', 'minus', 'x', 'times', 'multiply', '/', 'divide' ];
		$left					=	random_int( $leftMin, $leftMax );
		$right					=	random_int( $rightMin, $rightMax );
		$operator				=	$operators[array_rand( $operators, 1 )];

		switch ( $operator ) {
			case '+':
			case 'add':
			case 'plus':
				$value			=	( $left + $right );
				break;
			case '-':
			case 'subtract':
			case 'minus':
				// Prevent a negative value:
				if ( $right > $left ) {
					$rightOld	=	$right;
					$leftOld	=	$left;

					$right		=	$leftOld;
					$left		=	$rightOld;
				}

				$value			=	( $left - $right );
				break;
			case 'x':
			case 'multiply':
				$value			=	( $left * $right );
				break;
			case '/':
			case 'divide':
				// Prevent a negative value:
				if ( $right > $left ) {
					$rightOld	=	$right;
					$leftOld	=	$left;

					$right		=	$leftOld;
					$left		=	$rightOld;
				}

				$value			=	( $left / $right );

				if ( round( $value ) !== $value ) {
					$value		=	0;
				}
				break;
		}

		if ( $value <= 0 ) {
			return self::getMathEquation( $value, $leftMin, $leftMax, $rightMin, $rightMax );
		}

		if ( \strlen( $operator ) === 1 ) {
			$textValues			=	[	1	=>	'one',		2	=>	'two',		3	=>	'three',	4	=>	'four',			5	=>	'five',			6	=>	'six',
										7	=>	'seven',	8	=>	'eight',	9	=>	'nine',		10	=>	'ten',			11	=>	'eleven',		12	=>	'twelve',
										13	=>	'thirteen',	14	=>	'fourteen',	15	=>	'fifteen',	16	=>	'sixteen',		17	=>	'seventeen',	18	=>	'eighteen',
										19	=>	'nineteen',	20	=>	'twenty',	30	=>	'thirty',	40	=>	'fourty',		50	=>	'fifty',		60	=>	'sixty',
										70	=>	'seventy',	80	=>	'eighty',	90	=>	'ninety',
									];

			if ( isset( $textValues[$left] ) && ( random_int( 1, 2 ) === 2 ) ) {
				$left			=	$textValues[$left];
			}

			if ( \is_int( $left ) && isset( $textValues[$right] ) && ( random_int( 1, 2 ) === 2 ) ) {
				$right			=	$textValues[$right];
			}
		}

		return CBTxt::T( 'CAPTCHA_MATH_EQUATION', 'What is [left] [operator] [right]?', [ '[left]' => ( ! \is_int( $left ) ? CBTxt::T( $left ) : $left ), '[operator]' => ( \strlen( $operator ) > 1 ? CBTxt::T( $operator ) : $operator ), '[right]' => ( ! \is_int( $right ) ? CBTxt::T( $right ) : $right ) ] );
	}

	/**
	 * Get the users device parsed from user agent
	 * 
	 * @return string
	 */
	public static function getDevice(): string
	{
		global $_SERVER;

		$userAgent	=	$_SERVER['HTTP_USER_AGENT'];
		$devices	=	[	'/win/i'			=>	'Windows',
							'/mac/i'			=>	'Mac',
							'/linux/i'			=>	'Linux',
							'/ubuntu/i'			=>	'Ubuntu',
							'/android/i'		=>	'Android',
							'/iphone/i'			=>	'iPhone',
							'/ipod/i'			=>	'iPod',
							'/ipad/i'			=>	'iPad',
							'/webos|mobile/i'	=>	'Mobile',
						];

		foreach ( $devices as $regex => $device ) {
			if ( preg_match( $regex, $userAgent ) ) {
				return $device;
			}
		}

		return 'Unknown';
	}

	/**
	 * Get the users browser parsed from user agent
	 *
	 * @return string
	 */
	public static function getBrowser(): string
	{
		global $_SERVER;

		$userAgent	=	$_SERVER['HTTP_USER_AGENT'];
		$browsers	=	[	'/edge/i'		=>	'Edge',
							'/msie/i'		=>	'Internet Explorer',
							'/chrome/i'		=>	'Chrome',
							'/firefox/i'	=>	'Firefox',
							'/opera/i'		=>	'Opera',
							'/safari/i'		=>	'Safari',
							'/netscape/i'	=>	'Netscape',
							'/outlook/i'	=>	'Outlook',
							'/mobile/i'		=>	'Mobile',
						];

		foreach ( $browsers as $regex => $browser ) {
			if ( preg_match( $regex, $userAgent ) ) {
				return $browser;
			}
		}

		return 'Unknown';
	}

	/**
	 * @param string $type
	 * @return string
	 */
	public static function getType( string $type ): string
	{
		if ( ! $type ) {
			return CBTxt::T( 'Unknown' );
		}

		switch ( $type ) {
			// Official Types
			case 'user':
				return CBTxt::T( 'ANTISPAM_TYPE_USER', 'User' );
			case 'ip':
				return CBTxt::T( 'ANTISPAM_TYPE_IP', 'IP Address' );
			case 'ip_partial':
				return CBTxt::T( 'ANTISPAM_TYPE_IP_PARTIAL', 'IP Address (Partial)' );
			case 'ip_list':
				return CBTxt::T( 'ANTISPAM_TYPE_IP_LIST', 'IP Address (List)' );
			case 'ip_range':
				return CBTxt::T( 'ANTISPAM_TYPE_IP_RANGE', 'IP Address Range' );
			case 'email':
				return CBTxt::T( 'ANTISPAM_TYPE_EMAIL', 'Email Address' );
			case 'email_partial':
				return CBTxt::T( 'ANTISPAM_TYPE_EMAIL_PARTIAL', 'Email Address (Partial)' );
			case 'email_list':
				return CBTxt::T( 'ANTISPAM_TYPE_EMAIL_LIST', 'Email Address (List)' );
			case 'domain':
				return CBTxt::T( 'ANTISPAM_TYPE_DOMAIN', 'Email Domain' );
			case 'domain_partial':
				return CBTxt::T( 'ANTISPAM_TYPE_DOMAIN_PARTIAL', 'Email Domain (Partial)' );
			case 'domain_list':
				return CBTxt::T( 'ANTISPAM_TYPE_DOMAIN_LIST', 'Email Domain (List)' );
			// Attempt/Log Types
			case 'login':
				return CBTxt::T( 'ANTISPAM_TYPE_LOGIN', 'Login' );
			case 'forgot':
				return CBTxt::T( 'ANTISPAM_TYPE_FORGOT', 'Forgot Login' );
			case 'registration':
			case 'reg':
				return CBTxt::T( 'ANTISPAM_TYPE_REG', 'Registration' );
			case 'captcha':
				return CBTxt::T( 'ANTISPAM_TYPE_CAPTCHA', 'Captcha' );
			case 'kunena':
				return CBTxt::T( 'ANTISPAM_TYPE_KUNENA', 'Forums' );
			default:
				return CBTxt::T( $type );
		}
	}

	/**
	 * @param string $url
	 * @return string[]
	 */
	public static function getList( string $url ): array
	{
		global $_CB_framework;

		static $cache				=	[];

		if ( \array_key_exists( $url, $cache ) ) {
			return $cache[$url];
		}

		$list						=	[];

		if ( $url[0] === '/' ) {
			$contents				=	file_get_contents( $_CB_framework->getCfg( 'absolute_path' ) . $url );

			if ( $contents !== false ) {
				$list				=	explode( "\n", $contents );
			}
		} else {
			$cachePath				=	$_CB_framework->getCfg( 'absolute_path' ) . '/cache/antispam';
			$cacheFile				=	$cachePath . '/' . md5( $url ) . '.json';

			if ( file_exists( $cacheFile ) && ( ( ( Application::Date( 'now', 'UTC' )->getTimestamp() - filemtime( $cacheFile ) ) / 3600 ) < 48 ) ) {
				$list				=	json_decode( file_get_contents( $cacheFile ) );
			}

			if ( ! $list ) {
				try {
					$result			=	( new Client() )->get( $url );

					if ( (int) $result->getStatusCode() === 200 ) {
						$response	=	(string) $result->getBody();

						if ( $response ) {
							$list	=	explode( "\n", $response );
						}

						if ( ! is_dir( $cachePath ) ) {
							mkdir( $cachePath, 0755, true );
						}

						file_put_contents( $cacheFile, json_encode( $list ) );
					}
				} catch ( \Exception $e ) {}
			}
		}

		$cache[$url]				=	$list;

		return $list;
	}
}
