<?php
/**
* Community Builder (TM)
* @version $Id: $
* @package CommunityBuilder
* @copyright (C) 2004-2023 www.joomlapolis.com / Lightning MultiCom SA - and its licensors, all rights reserved
* @license http://www.gnu.org/licenses/old-licenses/gpl-2.0.html GNU/GPL version 2
*/

namespace CB\Plugin\ReconfirmEmail;

use CB\Database\Table\UserTable;
use CBLib\Registry\Registry;

\defined( 'CBLIB' ) or die();

class Helper
{
	/**
	 * @var array The array of users who have changed their email
	 */
	private static array $changed	=	[];
	/**
	 * @var array The array of users who are changing their email
	 */
	private static array $changing	=	[];
	/**
	 * @var array The array of emails that already exist or not
	 */
	private static array $existing	=	[];

	/**
	 * @return Registry
	 */
	public static function getGlobalParams(): Registry
	{
		global $_PLUGINS;

		static $params	=	null;

		if ( ! $params ) {
			$plugin		=	$_PLUGINS->getLoadedPlugin( 'user', 'cbreconfirmemail' );
			$params		=	new Registry();

			if ( $plugin ) {
				$params->load( $plugin->params );
			}
		}

		return $params;
	}

	/**
	 * Checks if user email address was changed
	 *
	 * @param int $userId
	 * @return bool
	 */
	public static function getEmailChanged( int $userId ): bool
	{
		return \array_key_exists( $userId, self::$changed );
	}

	/**
	 * Sets if the user email address was changed and to what
	 *
	 * @param int         $userId
	 * @param null|string $newEmail
	 * @param null|string $oldEmail
	 */
	public static function setEmailChanged( int $userId, ?string $newEmail, ?string $oldEmail = null ): void
	{
		if ( $newEmail === null ) {
			if ( self::getEmailChanged( $userId ) ) {
				unset( self::$changed[$userId] );
			}

			return;
		}

		self::$changed[$userId]	=	[ 'new' => $newEmail, 'old' => $oldEmail ];
	}

	/**
	 * Checks if user email address is currently changing
	 *
	 * @param int $userId
	 * @return bool
	 */
	public static function getEmailChanging( int $userId ): bool
	{
		return \array_key_exists( $userId, self::$changing );
	}

	/**
	 * Sets if the user email address is currently changing
	 *
	 * @param int         $userId
	 * @param null|string $newEmail
	 * @param null|string $oldEmail
	 * @return void
	 */
	public static function setEmailChanging( int $userId, ?string $newEmail, ?string $oldEmail = null ): void
	{
		if ( $newEmail === null ) {
			if ( self::getEmailChanging( $userId ) ) {
				unset( self::$changing[$userId] );
			}

			return;
		}

		self::$changing[$userId]	=	[ 'new' => $newEmail, 'old' => $oldEmail ];
	}

	/**
	 * Returns the old email address that is being changed
	 *
	 * @param int $userId
	 * @return null|string
	 */
	public static function getOldEmail( int $userId ): ?string
	{
		if ( \array_key_exists( $userId, self::$changing ) ) {
			return self::$changing[$userId]['old'];
		}

		if ( \array_key_exists( $userId, self::$changed ) ) {
			return self::$changed[$userId]['old'];
		}

		return null;
	}

	/**
	 * Returns the new email address being changed to
	 *
	 * @param int $userId
	 * @return null|string
	 */
	public static function getNewEmail( int $userId ): ?string
	{
		if ( \array_key_exists( $userId, self::$changing ) ) {
			return self::$changing[$userId]['new'];
		}

		if ( \array_key_exists( $userId, self::$changed ) ) {
			return self::$changed[$userId]['new'];
		}

		return null;
	}

	/**
	 * Checks if an email already exists or not
	 *
	 * @param string $email
	 * @return bool
	 */
	public static function emailExists( string $email ): bool
	{
		if ( \array_key_exists( $email, self::$existing ) ) {
			return self::$existing[$email];
		}

		$user					=	new UserTable();

		$user->loadByEmail( $email );

		$exists					=	(bool) $user->getInt( 'id', 0 );

		self::$existing[$email]	=	$exists;

		return $exists;
	}
}