<?php
/**
* Community Builder (TM)
* @version $Id: $
* @package CommunityBuilder
* @copyright (C) 2004-2022 www.joomlapolis.com / Lightning MultiCom SA - and its licensors, all rights reserved
* @license http://www.gnu.org/licenses/old-licenses/gpl-2.0.html GNU/GPL version 2
*/

namespace CB\Plugin\AntiSpam\Tab;

use CB\Database\Table\TabTable;
use CB\Database\Table\UserTable;
use CB\Plugin\AntiSpam\CBAntiSpam;
use CB\Plugin\AntiSpam\Table\AttemptTable;
use CB\Plugin\AntiSpam\Table\BlockTable;
use CB\Plugin\AntiSpam\Table\LogTable;
use CB\Plugin\AntiSpam\Table\WhitelistTable;
use CBLib\Application\Application;
use CBLib\Language\CBTxt;
use CBLib\Registry\ParamsInterface;
use CBLib\Registry\Registry;
use cbPageNav;
use cbTabHandler;
use cbTabs;
use CBuser;

\defined('CBLIB') or die();

class AntispamTab extends cbTabHandler
{

	/**
	 * @param TabTable  $tab
	 * @param UserTable $user
	 * @param int       $ui
	 * @return null|string
	 */
	public function getDisplayTab( $tab, $user, $ui ): ?string
	{
		global $_CB_framework, $_CB_database;

		if ( ( ! Application::MyUser()->isGlobalModerator() ) || Application::User( $user->getInt( 'id', 0 ) )->isGlobalModerator() ) {
			return null;
		}

		if ( ! ( $tab->params instanceof ParamsInterface ) ) {
			$tab->params			=	new Registry( $tab->params );
		}

		$blocksEnabled				=	( $this->params->getBool( 'general_block', true ) && $tab->params->getBool( 'tab_block', ( $tab->getString( 'pluginclass', '' ) === 'cbantispamTabBlocks' ) ) );
		$whitelistsEnabled			=	( $this->params->getBool( 'general_whitelist', true ) && $tab->params->getBool( 'tab_whitelist', ( $tab->getString( 'pluginclass', '' ) === 'cbantispamTabWhitelists' ) ) );
		$attemptsEnabled			=	( $this->params->getBool( 'general_attempts', true ) && $tab->params->getBool( 'tab_attempts', ( $tab->getString( 'pluginclass', '' ) === 'cbantispamTabAttempts' ) ) );
		$logsEnabled				=	( $this->params->getBool( 'general_log', true ) && $tab->params->getBool( 'tab_logs', ( $tab->getString( 'pluginclass', '' ) === 'cbantispamTabLog' ) ) );

		if ( ( ! $blocksEnabled ) && ( ! $whitelistsEnabled ) && ( ! $attemptsEnabled ) && ( ! $logsEnabled ) ) {
			return null;
		}

		$tabPrefix				=	'tab_' . $tab->getInt( 'tabid', 0 ) . '_';

		outputCbJs();
		outputCbTemplate();
		initToolTip();

		$ipAddress				=	CBAntiSpam::getUserIP( $user );
		$emailDomain			=	CBAntiSpam::getEmailDomain( $user );

		$layoutData				=	[	'user'		=>	$user,
										'isOwner'	=>	( CBuser::getMyUserDataInstance()->getInt( 'id', 0 ) === $user->getInt( 'id', 0 ) ),
										'tabUrl'	=>	$_CB_framework->userProfileUrl( $user->getInt( 'id', 0 ), true, $tab->getInt( 'tabid', 0 ) ),
										'returnUrl'	=>	base64_encode( $_CB_framework->userProfileUrl( $user->getInt( 'id', 0 ), false, $tab->getInt( 'tabid', 0 ) ) ),
									];

		$where					=	[];

		if ( $blocksEnabled || $whitelistsEnabled ) {
			$where[]			.=	"( " . $_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'user' )
								.	" AND " . $_CB_database->NameQuote( 'value' ) . " = " . $user->getInt( 'id', 0 ) . " )";

			$where[]			.=	"( " . $_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'email' )
								.	" AND " . $_CB_database->NameQuote( 'value' ) . " = " . $_CB_database->Quote( $user->getString( 'email', '' ) ) . " )";

			$where[]			.=	"( " . $_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'email_partial' )
								.	" AND " . $_CB_database->Quote( $user->getString( 'email', '' ) ) . " LIKE " . $_CB_database->NameQuote( 'value' ) . " )";

			if ( $emailDomain ) {
				$where[]		.=	"( " . $_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'domain' )
								.	" AND " . $_CB_database->NameQuote( 'value' ) . " = " . $_CB_database->Quote( $emailDomain ) . " )";

				$where[]		.=	"( " . $_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'domain_partial' )
								.	" AND " . $_CB_database->Quote( $emailDomain ) . " LIKE " . $_CB_database->NameQuote( 'value' ) . " )";
			}

			if ( $ipAddress ) {
				$where[]		.=	"( " . $_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'ip' )
								.	" AND " . $_CB_database->NameQuote( 'value' ) . " = " . $_CB_database->Quote( $ipAddress ) . " )";

				$where[]		.=	"( " . $_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'ip_partial' )
								.	" AND " . $_CB_database->Quote( $ipAddress ) . " LIKE " . $_CB_database->NameQuote( 'value' ) . " )";

				$where[]		.=	"( " . $_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'ip_range' )
								.	" AND ( INET_ATON( " . $_CB_database->Quote( $ipAddress ) . " ) BETWEEN INET_ATON( SUBSTRING_INDEX( " . $_CB_database->NameQuote( 'value' ) . ", ':', 1 ) ) AND INET_ATON( SUBSTRING_INDEX( " . $_CB_database->NameQuote( 'value' ) . ", ':', -1 ) ) ) )";
			}
		}

		$subTabs				=	[];

		if ( $blocksEnabled ) {
			$subTabs[]			=	[	'id'		=>	'blocksTabBlocks',
										'title'		=>	CBTxt::T( 'TAB_BLOCKS', 'Blocks' ),
										'content'	=>	$this->getBlocksTab( $tab, $tabPrefix, $where, $layoutData ),
									];
		}

		if ( $whitelistsEnabled ) {
			$subTabs[]			=	[	'id'		=>	'blocksTabWhitelists',
										'title'		=>	CBTxt::T( 'TAB_WHITELISTS', 'Whitelists' ),
										'content'	=>	$this->getWhitelistsTab( $tab, $tabPrefix, $where, $layoutData ),
									];
		}

		if ( $attemptsEnabled ) {
			$subTabs[]			=	[	'id'		=>	'blocksTabAttempts',
										'title'		=>	CBTxt::T( 'TAB_ATTEMPTS', 'Attempts' ),
										'content'	=>	$this->getAttemptsTab( $tab, $tabPrefix, $ipAddress, $layoutData ),
									];
		}

		if ( $logsEnabled ) {
			$subTabs[]			=	[	'id'		=>	'blocksTabLogs',
										'title'		=>	CBTxt::T( 'TAB_LOGS', 'Logs' ),
										'content'	=>	$this->getLogsTab( $tab, $tabPrefix, $user, $layoutData ),
									];
		}

		if ( ! \count( $subTabs ) ) {
			return null;
		}

		$class					=	$this->params->getString( 'general_class', '' );
		$return					=	'<div class="cbAntiSpam' . ( $class ? ' ' . htmlspecialchars( $class ) : null ) . '">';

		if ( \count( $subTabs ) > 1 ) {
			$tabs				=	new cbTabs( 1, 1 );

			$return				.=		$tabs->startPane( 'blocksTabs' );

			foreach ( $subTabs as $subTab ) {
				$return			.=			$tabs->startTab( null, $subTab['title'], $subTab['id'] )
								.				$subTab['content']
								.			$tabs->endTab();
			}

			$return				.=		$tabs->endPane();
		} else {
			$return				.=		( $subTabs[0]['content'] ?? '' );
		}

		$return					.=	'</div>';

		return $return;
	}

	/**
	 * @param TabTable $tab
	 * @param string   $tabPrefix
	 * @param array    $where
	 * @param array    $layoutData
	 * @return string
	 */
	private function getBlocksTab( TabTable $tab, string $tabPrefix, array $where, array $layoutData ): string
	{
		global $_CB_framework, $_CB_database;

		$blocksPrefix		=	$tabPrefix . 'blocks_';
		$limit				=	$tab->params->getInt( 'tab_limit', 15 );
		$limitstart			=	$_CB_framework->getUserStateFromRequest( $blocksPrefix . 'limitstart{com_comprofiler}', $blocksPrefix . 'limitstart' );

		$query				=	"SELECT COUNT(*)"
							.	"\n FROM " . $_CB_database->NameQuote( '#__comprofiler_plugin_antispam_block' )
							.	"\n WHERE " . implode( " OR ", $where );
		$_CB_database->setQuery( $query );
		$total				=	$_CB_database->loadResult();

		if ( $total <= $limitstart ) {
			$limitstart		=	0;
		}

		$pageNav			=	new cbPageNav( $total, $limitstart, $limit );

		$pageNav->setInputNamePrefix( $blocksPrefix );
		$pageNav->setStaticLimit( true );

		$rows				=	[];

		if ( $total ) {
			$query			=	"SELECT *"
							.	"\n FROM " . $_CB_database->NameQuote( '#__comprofiler_plugin_antispam_block' )
							.	"\n WHERE " . implode( " OR ", $where )
							.	"\n ORDER BY " . $_CB_database->NameQuote( 'id' ) . " ASC";
			if ( $tab->params->getBool( 'tab_paging', true ) ) {
				$_CB_database->setQuery( $query, $pageNav->limitstart, $pageNav->limit );
			} else {
				$_CB_database->setQuery( $query );
			}
			$rows			=	$_CB_database->loadObjectList( 'id', BlockTable::class, [ $_CB_database ] );
		}

		$hasPaging			=	( $tab->params->getBool( 'tab_paging', true ) && ( $pageNav->total > $pageNav->limit ) );

		extract( $layoutData, EXTR_REFS );

		$blockAccountUrl	=	( $tab->params->getBool( 'tab_block_account', true ) ? $_CB_framework->pluginClassUrl( 'cbantispam', true, [ 'action' => 'block', 'func' => 'account', 'user' => $user->getInt( 'id', 0 ), 'return' => $returnUrl ] ) : '' );
		$blockUserUrl		=	( $tab->params->getBool( 'tab_block_user', true ) ? $_CB_framework->pluginClassUrl( 'cbantispam', true, [ 'action' => 'block', 'func' => 'user', 'user' => $user->getInt( 'id', 0 ), 'return' => $returnUrl ] ) : '' );
		$blockIpUrl			=	( $tab->params->getBool( 'tab_block_ip', true ) ? $_CB_framework->pluginClassUrl( 'cbantispam', true, [ 'action' => 'block', 'func' => 'ip', 'user' => $user->getInt( 'id', 0 ), 'return' => $returnUrl ] ) : '' );
		$blockEmailUrl		=	( $tab->params->getBool( 'tab_block_email', false ) ? $_CB_framework->pluginClassUrl( 'cbantispam', true, [ 'action' => 'block', 'func' => 'email', 'user' => $user->getInt( 'id', 0 ), 'return' => $returnUrl ] ) : '' );
		$blockDomainUrl		=	( $tab->params->getBool( 'tab_block_domain', false ) ? $_CB_framework->pluginClassUrl( 'cbantispam', true, [ 'action' => 'block', 'func' => 'domain', 'user' => $user->getInt( 'id', 0 ), 'return' => $returnUrl ] ) : '' );

		ob_start();
		require CBAntiSpam::getTemplate( 'blocks' );
		return ob_get_clean();
	}

	/**
	 * @param TabTable $tab
	 * @param string   $tabPrefix
	 * @param array    $where
	 * @param array    $layoutData
	 * @return string
	 */
	private function getWhitelistsTab( TabTable $tab, string $tabPrefix, array $where, array $layoutData ): string
	{
		global $_CB_framework, $_CB_database;

		$whitelistsPrefix	=	$tabPrefix . 'whitelists_';
		$limit				=	$tab->params->getInt( 'tab_limit', 15 );
		$limitstart			=	$_CB_framework->getUserStateFromRequest( $whitelistsPrefix . 'limitstart{com_comprofiler}', $whitelistsPrefix . 'limitstart' );

		$query				=	"SELECT COUNT(*)"
							.	"\n FROM " . $_CB_database->NameQuote( '#__comprofiler_plugin_antispam_whitelist' )
							.	"\n WHERE " . implode( " OR ", $where );
		$_CB_database->setQuery( $query );
		$total				=	$_CB_database->loadResult();

		if ( $total <= $limitstart ) {
			$limitstart		=	0;
		}

		$pageNav			=	new cbPageNav( $total, $limitstart, $limit );

		$pageNav->setInputNamePrefix( $whitelistsPrefix );
		$pageNav->setStaticLimit( true );

		$rows				=	[];

		if ( $total ) {
			$query			=	"SELECT *"
							.	"\n FROM " . $_CB_database->NameQuote( '#__comprofiler_plugin_antispam_whitelist' )
							.	"\n WHERE " . implode( " OR ", $where )
							.	"\n ORDER BY " . $_CB_database->NameQuote( 'id' ) . " ASC";
			if ( $tab->params->getBool( 'tab_paging', true ) ) {
				$_CB_database->setQuery( $query, $pageNav->limitstart, $pageNav->limit );
			} else {
				$_CB_database->setQuery( $query );
			}
			$rows			=	$_CB_database->loadObjectList( 'id', WhitelistTable::class, [ $_CB_database ] );
		}

		$hasPaging			=	( $tab->params->getBool( 'tab_paging', true ) && ( $pageNav->total > $pageNav->limit ) );

		extract( $layoutData, EXTR_REFS );

		$whitelistUserUrl	=	( $tab->params->getBool( 'tab_whitelist_user', true ) ? $_CB_framework->pluginClassUrl( 'cbantispam', true, [ 'action' => 'whitelist', 'func' => 'user', 'user' => $user->getInt( 'id', 0 ), 'return' => $returnUrl ] ) : '' );
		$whitelistIpUrl		=	( $tab->params->getBool( 'tab_whitelist_ip', true ) ? $_CB_framework->pluginClassUrl( 'cbantispam', true, [ 'action' => 'whitelist', 'func' => 'ip', 'user' => $user->getInt( 'id', 0 ), 'return' => $returnUrl ] ) : '' );
		$whitelistEmailUrl	=	( $tab->params->getBool( 'tab_whitelist_email', false ) ? $_CB_framework->pluginClassUrl( 'cbantispam', true, [ 'action' => 'whitelist', 'func' => 'email', 'user' => $user->getInt( 'id', 0 ), 'return' => $returnUrl ] ) : '' );
		$whitelistDomainUrl	=	( $tab->params->getBool( 'tab_whitelist_domain', false ) ? $_CB_framework->pluginClassUrl( 'cbantispam', true, [ 'action' => 'whitelist', 'func' => 'domain', 'user' => $user->getInt( 'id', 0 ), 'return' => $returnUrl ] ) : '' );

		ob_start();
		require CBAntiSpam::getTemplate( 'whitelists' );
		return ob_get_clean();
	}

	/**
	 * @param TabTable    $tab
	 * @param string      $tabPrefix
	 * @param null|string $ipAddress
	 * @param array       $layoutData
	 * @return string
	 */
	private function getAttemptsTab( TabTable $tab, string $tabPrefix, ?string $ipAddress, array $layoutData ): string
	{
		global $_CB_framework, $_CB_database;

		$attemptsPrefix		=	$tabPrefix . 'attempts_';
		$limit				=	$tab->params->getInt( 'tab_limit', 15 );
		$limitstart			=	$_CB_framework->getUserStateFromRequest( $attemptsPrefix . 'limitstart{com_comprofiler}', $attemptsPrefix . 'limitstart' );
		$total				=	0;

		if ( $ipAddress ) {
			$query			=	"SELECT COUNT(*)"
							.	"\n FROM " . $_CB_database->NameQuote( '#__comprofiler_plugin_antispam_attempts' )
							.	"\n WHERE " . $_CB_database->NameQuote( 'ip_address' ) . " = " . $_CB_database->Quote( $ipAddress );
			$_CB_database->setQuery( $query );
			$total			=	$_CB_database->loadResult();
		}

		if ( $total <= $limitstart ) {
			$limitstart		=	0;
		}

		$pageNav			=	new cbPageNav( $total, $limitstart, $limit );

		$pageNav->setInputNamePrefix( $attemptsPrefix );
		$pageNav->setStaticLimit( true );

		$rows				=	[];

		if ( $ipAddress && $total ) {
			$query			=	"SELECT *"
							.	"\n FROM " . $_CB_database->NameQuote( '#__comprofiler_plugin_antispam_attempts' )
							.	"\n WHERE " . $_CB_database->NameQuote( 'ip_address' ) . " = " . $_CB_database->Quote( $ipAddress )
							.	"\n ORDER BY " . $_CB_database->NameQuote( 'date' ) . " DESC";
			if ( $tab->params->getBool( 'tab_paging', true ) ) {
				$_CB_database->setQuery( $query, $pageNav->limitstart, $pageNav->limit );
			} else {
				$_CB_database->setQuery( $query );
			}
			$rows			=	$_CB_database->loadObjectList( 'id', AttemptTable::class, [ $_CB_database ] );
		}

		$hasPaging			=	( $tab->params->getBool( 'tab_paging', true ) && ( $pageNav->total > $pageNav->limit ) );

		extract( $layoutData, EXTR_REFS );

		ob_start();
		require CBAntiSpam::getTemplate( 'attempts' );
		return ob_get_clean();
	}

	/**
	 * @param TabTable  $tab
	 * @param string    $tabPrefix
	 * @param UserTable $user
	 * @param array     $layoutData
	 * @return string
	 */
	private function getLogsTab( TabTable $tab, string $tabPrefix, UserTable $user, array $layoutData ): string
	{
		global $_CB_framework, $_CB_database;

		$logsPrefix			=	$tabPrefix . 'logs_';
		$limit				=	$tab->params->getInt( 'tab_limit', 15 );
		$limitstart			=	$_CB_framework->getUserStateFromRequest( $logsPrefix . 'limitstart{com_comprofiler}', $logsPrefix . 'limitstart' );

		$query				=	"SELECT COUNT(*)"
							.	"\n FROM " . $_CB_database->NameQuote( '#__comprofiler_plugin_antispam_log' )
							.	"\n WHERE " . $_CB_database->NameQuote( 'user_id' ) . " = " . $user->getInt( 'id', 0 );
		$_CB_database->setQuery( $query );
		$total				=	$_CB_database->loadResult();

		if ( $total <= $limitstart ) {
			$limitstart		=	0;
		}

		$pageNav			=	new cbPageNav( $total, $limitstart, $limit );

		$pageNav->setInputNamePrefix( $logsPrefix );
		$pageNav->setStaticLimit( true );

		$rows				=	[];

		if ( $total ) {
			$query			=	"SELECT *"
							.	"\n FROM " . $_CB_database->NameQuote( '#__comprofiler_plugin_antispam_log' )
							.	"\n WHERE " . $_CB_database->NameQuote( 'user_id' ) . " = " . $user->getInt( 'id', 0 )
							.	"\n ORDER BY " . $_CB_database->NameQuote( 'date' ) . " DESC";
			if ( $tab->params->getBool( 'tab_paging', true ) ) {
				$_CB_database->setQuery( $query, $pageNav->limitstart, $pageNav->limit );
			} else {
				$_CB_database->setQuery( $query );
			}
			$rows			=	$_CB_database->loadObjectList( 'id', LogTable::class, [ $_CB_database ] );
		}

		$hasPaging			=	( $tab->params->getBool( 'tab_paging', true ) && ( $pageNav->total > $pageNav->limit ) );

		extract( $layoutData, EXTR_REFS );

		ob_start();
		require CBAntiSpam::getTemplate( 'logs' );
		return ob_get_clean();
	}
}