<?php
/**
* Community Builder (TM)
* @version $Id: $
* @package CommunityBuilder
* @copyright (C) 2004-2022 www.joomlapolis.com / Lightning MultiCom SA - and its licensors, all rights reserved
* @license http://www.gnu.org/licenses/old-licenses/gpl-2.0.html GNU/GPL version 2
*/

namespace CB\Plugin\AntiSpam\Table;

use CBLib\Application\Application;
use CBLib\Database\Table\Table;
use CBLib\Language\CBTxt;
use CB\Plugin\AntiSpam\CBAntiSpam;

\defined('CBLIB') or die();

class AttemptTable extends Table
{
	/** @var null|int  */
	public $id;
	/** @var null|string  */
	public $ip_address;
	/** @var null|string  */
	public $type;
	/** @var null|int  */
	public $count;
	/** @var null|string  */
	public $date;

	/**
	 * Table name in database
	 *
	 * @var string
	 */
	protected $_tbl			=	'#__comprofiler_plugin_antispam_attempts';

	/**
	 * Primary key(s) of table
	 *
	 * @var string
	 */
	protected $_tbl_key		=	'id';

	/**
	 * @return bool
	 */
	public function check(): bool
	{
		if ( $this->getIpAddress() === '' ) {
			$this->setError( CBTxt::T( 'IP Address not specified!' ) );

			return false;
		}

		return true;
	}

	/**
	 * @param bool $updateNulls
	 * @return bool
	 */
	public function store( $updateNulls = false ): bool
	{
		global $_PLUGINS;

		$new	=	( ! $this->getId() );
		$old	=	new self();

		$this->setDate( $this->getString( 'date', Application::Database()->getUtcDateTime() ) );

		if ( ! $new ) {
			$old->load( $this->getId() );

			$_PLUGINS->trigger( 'antispam_onBeforeUpdateAttempt', [ &$this, $old ] );
		} else {
			$_PLUGINS->trigger( 'antispam_onBeforeCreateAttempt', [ &$this ] );
		}

		if ( ! parent::store( $updateNulls ) ) {
			return false;
		}

		if ( ! $new ) {
			$_PLUGINS->trigger( 'antispam_onAfterUpdateAttempt', [ $this, $old ] );
		} else {
			$_PLUGINS->trigger( 'antispam_onAfterCreateAttempt', [ $this ] );
		}

		return true;
	}

	/**
	 * @param null|int $id
	 * @return bool
	 */
	public function delete( $id = null ): bool
	{
		global $_PLUGINS;

		$_PLUGINS->trigger( 'antispam_onBeforeDeleteAttempt', [ &$this ] );

		if ( ! parent::delete( $id ) ) {
			return false;
		}

		$_PLUGINS->trigger( 'antispam_onAfterDeleteAttempt', [ $this ] );

		return true;
	}

	/**
	 * @return int
	 */
	public function getId(): int
	{
		return $this->getInt( 'id', 0 );
	}

	/**
	 * @param null|int $id
	 */
	public function setId( ?int $id ): void
	{
		$this->id	=	$id;
	}

	/**
	 * @return string
	 */
	public function getIpAddress(): string
	{
		return $this->getString( 'ip_address', '' );
	}

	/**
	 * @param string $ip_address
	 */
	public function setIpAddress( string $ip_address ): void
	{
		$this->ip_address	=	$ip_address;
	}

	/**
	 * @return string
	 */
	public function getType(): string
	{
		return $this->getString( 'type', '' );
	}

	/**
	 * @param null|string $type
	 */
	public function setType( ?string $type ): void
	{
		$this->type	=	$type;
	}

	/**
	 * @return int
	 */
	public function getCount(): int
	{
		return $this->getInt( 'count', 0 );
	}

	/**
	 * @param int $count
	 */
	public function setCount( int $count ): void
	{
		$this->count	=	$count;
	}

	/**
	 * @param int $count
	 */
	public function incrementCount( int $count ): void
	{
		$this->setCount( ( $this->getCount() + $count ) );
	}

	/**
	 * @return string
	 */
	public function getDate(): string
	{
		return $this->getString( 'date', '' );
	}

	/**
	 * @param string $date
	 */
	public function setDate( string $date ): void
	{
		$this->date	=	$date;
	}

	/**
	 * @return bool
	 */
	public function isBlocked(): bool
	{
		return (bool) CBAntiSpam::getBlock( 0, $this->getIpAddress() );
	}

	/**
	 * @return bool
	 */
	public function isWhitelisted(): bool
	{
		return (bool) CBAntiSpam::getWhitelist( 0, $this->getIpAddress() );
	}

	/**
	 * @return string
	 */
	public function renderType(): string
	{
		return CBAntiSpam::getType( $this->getType() );
	}
}