<?php
/**
* Community Builder (TM)
* @version $Id: $
* @package CommunityBuilder
* @copyright (C) 2004-2022 www.joomlapolis.com / Lightning MultiCom SA - and its licensors, all rights reserved
* @license http://www.gnu.org/licenses/old-licenses/gpl-2.0.html GNU/GPL version 2
*/

namespace CB\Plugin\AutoActions\Table;

use CB\Database\Table\UserTable;
use CBLib\Application\Application;
use CBLib\Database\Table\Table;
use CBLib\Language\CBTxt;
use CBLib\Registry\ParamsInterface;
use CBLib\Registry\Registry;

\defined( 'CBLIB' ) or die();

class BatchTable extends Table
{
	/** @var null|int */
	public $id;
	/** @var null|string */
	public $title;
	/** @var null|int */
	public $users;
	/** @var null|string */
	public $access;
	/** @var null|string */
	public $actions;
	/** @var null|string */
	public $filter_basic;
	/** @var null|string */
	public $filter_join;
	/** @var null|string */
	public $filter_where;
	/** @var null|int */
	public $batch_limit;
	/** @var null|int */
	public $batch_progress;
	/** @var null|int */
	public $batch_redirect;
	/** @var null|int */
	public $use_limit;
	/** @var null|int */
	public $use_progress;
	/** @var null|int */
	public $published;

	/** @var null|Registry  */
	protected ?Registry $_filter_basic	=	null;

	/**
	 * Table name in database
	 *
	 * @var string
	 */
	protected $_tbl			=	'#__comprofiler_plugin_autoactions_batches';

	/**
	 * Primary key(s) of table
	 *
	 * @var string
	 */
	protected $_tbl_key		=	'id';

	/**
	 * @return bool
	 */
	public function check(): bool
	{
		if ( ! $this->getTitle() ) {
			$this->setError( CBTxt::T( 'Title not specified!' ) );

			return false;
		}

		if ( ! $this->getActions() ) {
			$this->setError( CBTxt::T( 'Auto Actions not specified!' ) );

			return false;
		}

		return true;
	}

	/**
	 * @return AutoActionTable[]
	 */
	public function getAutoActions(): array
	{
		global $_CB_database;

		static $cache			=	[];

		$actions				=	$this->getActions();

		if ( ! $actions ) {
			return [];
		}

		$id						=	$this->getId();

		if ( ! isset( $cache[$id] ) ) {
			$cache[$id]			=	[];

			$actions			=	new Registry( $actions );
			$actionIds			=	[];

			foreach ( $actions as $action ) {
				/** @var ParamsInterface $action */
				$actionId		=	$action->getInt( 'action', 0 );

				if ( ! $actionId ) {
					continue;
				}

				$actionIds[]	=	$actionId;
			}

			if ( ! $actionIds ) {
				return [];
			}

			$query				=	'SELECT *'
								.	"\n FROM " . $_CB_database->NameQuote( '#__comprofiler_plugin_autoactions' )
								.	"\n WHERE " . $_CB_database->NameQuote( 'id' ) . " IN " . $_CB_database->safeArrayOfIntegers( $actionIds )
								.	"\n AND " . $_CB_database->NameQuote( 'published' ) . " = 1";
			$_CB_database->setQuery( $query );
			$autoActions		=	$_CB_database->loadObjectList( 'id', AutoActionTable::class, [ $_CB_database ] );

			foreach ( $actionIds as $actionId ) {
				if ( ! isset( $autoActions[$actionId] ) ) {
					continue;
				}

				$cache[$id][]	=	$autoActions[$actionId];
			}
		}

		return $cache[$id];
	}

	/**
	 * @return int
	 */
	public function getId(): int
	{
		return $this->getInt( 'id', 0 );
	}

	/**
	 * @param null|int $id
	 */
	public function setId( ?int $id ): void
	{
		$this->id	=	$id;
	}

	/**
	 * @return string
	 */
	public function getTitle(): string
	{
		return $this->getString( 'title', '' );
	}

	/**
	 * @param string $title
	 */
	public function setTitle( string $title ): void
	{
		$this->title	=	$title;
	}

	/**
	 * @return int
	 */
	public function getUsers(): int
	{
		return $this->getInt( 'users', 0 );
	}

	/**
	 * @param int $users
	 */
	public function setUsers( int $users ): void
	{
		$this->users	=	$users;
	}

	/**
	 * @return string
	 */
	public function getAccess(): string
	{
		return $this->getString( 'access', '-1' );
	}

	/**
	 * @param null|string $access
	 */
	public function setAccess( ?string $access ): void
	{
		$this->access	=	$access;
	}

	/**
	 * @return string
	 */
	public function getActions(): string
	{
		return $this->getRaw( 'actions', '' );
	}

	/**
	 * @param string $actions
	 */
	public function setActions( string $actions ): void
	{
		$this->actions	=	$actions;
	}

	/**
	 * @return Registry
	 */
	public function getFilterBasic(): Registry
	{
		if ( $this->_filter_basic === null ) {
			$this->_filter_basic	=	new Registry( $this->filter_basic );
		}

		return $this->_filter_basic;
	}

	/**
	 * @param string|array $filter_basic
	 */
	public function setFilterBasic( $filter_basic ): void
	{
		$this->getFilterBasic()->load( $filter_basic );
	}

	/**
	 * @return string
	 */
	public function getFilterJoin(): string
	{
		return $this->getRaw( 'filter_join', '' );
	}

	/**
	 * @param null|string $filter_join
	 */
	public function setFilterJoin( ?string $filter_join ): void
	{
		$this->filter_join	=	$filter_join;
	}

	/**
	 * @return string
	 */
	public function getFilterWhere(): string
	{
		return $this->getRaw( 'filter_where', '' );
	}

	/**
	 * @param null|string $filter_where
	 */
	public function setFilterWhere( ?string $filter_where ): void
	{
		$this->filter_where	=	$filter_where;
	}

	/**
	 * @return int
	 */
	public function getBatchLimit(): int
	{
		return $this->getInt( 'batch_limit', 0 );
	}

	/**
	 * @param int $batch_limit
	 */
	public function setBatchLimit( int $batch_limit ): void
	{
		$this->batch_limit	=	$batch_limit;
	}

	/**
	 * @return int
	 */
	public function getBatchProgress(): int
	{
		return $this->getInt( 'batch_progress', 0 );
	}

	/**
	 * Sets batch progress to the specified amount
	 * Returns true if successful and false if failed
	 *
	 * @param int $progress
	 * @return bool
	 */
	public function setBatchProgress( int $progress ): bool
	{
		if ( ! $this->getBatchLimit() ) {
			return true;
		}

		$this->set( 'batch_progress', $progress );

		return ( ( ! $this->getError() ) && $this->check() && $this->store() );
	}

	/**
	 * Increments the batch progress by the specified amount
	 * Returns true if successful and false if failed
	 *
	 * @param int $progress
	 * @return bool
	 */
	public function incrementBatchProgress( int $progress ): bool
	{
		if ( ! $this->getBatchLimit() ) {
			return true;
		}

		$this->set( 'batch_progress', ( $this->getBatchProgress() + $progress ) );

		return ( ( ! $this->getError() ) && $this->check() && $this->store() );
	}

	/**
	 * @return bool
	 */
	public function getBatchRedirect(): bool
	{
		return $this->getBool( 'batch_redirect', true );
	}

	/**
	 * @param bool $batch_redirect
	 */
	public function setBatchRedirect( bool $batch_redirect ): void
	{
		$this->batch_redirect	=	(int) $batch_redirect;
	}

	/**
	 * @return int
	 */
	public function getUseLimit(): int
	{
		return $this->getInt( 'use_limit', 0 );
	}

	/**
	 * @param int $use_limit
	 */
	public function setUseLimit( int $use_limit ): void
	{
		$this->use_limit	=	$use_limit;
	}

	/**
	 * @return int
	 */
	public function getUseProgress(): int
	{
		return $this->getInt( 'use_progress', 0 );
	}

	/**
	 * Sets batch use to the specified amount
	 * Returns true if successful and false if failed
	 *
	 * @param int $progress
	 * @return bool
	 */
	public function setUseProgress( int $progress ): bool
	{
		if ( ! $this->getUseLimit() ) {
			return true;
		}

		$this->set( 'use_progress', $progress );

		return ( ( ! $this->getError() ) && $this->check() && $this->store() );
	}

	/**
	 * Increments the use progress by the specified amount
	 * Returns true if successful and false if failed
	 *
	 * @param int $progress
	 * @return bool
	 */
	public function incrementUseProgress( int $progress ): bool
	{
		if ( ! $this->getUseLimit() ) {
			// Still reset the batch progress since we've completed a use:
			$this->setBatchProgress( 0 );

			return true;
		}

		if ( $this->getBatchLimit() ) {
			// Reset batch progress since we've completed a full batch process:
			$this->set( 'batch_progress', 0 );
		}

		$this->set( 'use_progress', ( $this->getUsers() + $progress ) );

		return ( ( ! $this->getError() ) && $this->check() && $this->store() );
	}

	/**
	 * @return int
	 */
	public function getPublished(): int
	{
		return $this->getInt( 'published', 0 );
	}

	/**
	 * @param int $published
	 */
	public function setPublished( int $published ): void
	{
		$this->published	=	$published;
	}

	/**
	 * @param null|UserTable $user
	 * @param array          $variables
	 * @param string         $method
	 * @return string
	 */
	public function runBatch( ?UserTable $user = null, array $variables = [], string $method = 'run' ): string
	{
		global $_CB_framework, $_CB_database;

		if ( ! $user ) {
			$cbUser						=	\CBuser::getMyInstance();
			$user						=	$cbUser->getUserData();
		} else {
			$cbUser						=	\CBuser::getInstance( $user->getInt( 'id', 0 ), false );
		}

		$return							=	'';
		$useLimit						=	$this->getUseLimit();

		if ( $useLimit && ( $useLimit === $this->getUseProgress() ) ) {
			return $return;
		}

		$autoActions					=	$this->getAutoActions();

		if ( ! $autoActions ) {
			return $return;
		}

		$users							=	$this->getUsers();

		if ( ! $users ) {
			foreach ( $autoActions as $autoAction ) {
				switch ( $method ) {
					case 'run':
						$return			.=	$autoAction->runAction( $variables );
						break;
					case 'execute':
						$return			.=	$autoAction->executeAction( $user, $variables );
						break;
					case 'trigger':
					default:
						$return			.=	$autoAction->triggerAction( $user, $variables );
						break;
				}

				if ( $autoAction->getError() ) {
					$this->setError( $autoAction->getError() );
				}
			}

			return $return;
		}

		$batchProgress					=	$this->getBatchProgress();
		$batchLimit						=	$this->getBatchLimit();

		$filterJOIN						=	$cbUser->replaceUserVars( $this->getFilterJoin(), [ $_CB_database, 'getEscaped' ], false );
		$filterWHERE					=	$cbUser->replaceUserVars( $this->getFilterWhere(), [ $_CB_database, 'getEscaped' ], false );

		$joinACL						=	false;
		$where							=	[];

		if ( $users >= 2 ) {
			$moderators					=	Application::CmsPermissions()->getGroupsOfViewAccessLevel( Application::Config()->getInt( 'moderator_viewaccesslevel', 3 ), true );

			if ( $moderators ) {
				$joinACL				=	true;

				switch ( $users ) {
					case 2: // All Moderators
						if ( \count( $moderators ) > 1 ) {
							$where[]	=	"g." . $_CB_database->NameQuote( 'group_id' ) . " IN " . $_CB_database->safeArrayOfIntegers( $moderators );
						} else {
							$where[]	=	"g." . $_CB_database->NameQuote( 'group_id' ) . " = " . (int) $moderators[0];
						}
						break;
					case 3: // All Non-Moderators
						if ( \count( $moderators ) > 1 ) {
							$where[]	=	"g." . $_CB_database->NameQuote( 'group_id' ) . " NOT IN " . $_CB_database->safeArrayOfIntegers( $moderators );
						} else {
							$where[]	=	"g." . $_CB_database->NameQuote( 'group_id' ) . " != " . (int) $moderators[0];
						}
						break;
				}
			}
		}

		$access							=	$this->getAccess();

		if ( ( $access !== '' ) && ( $access !== null ) && ( $access !== '-1' ) ) {
			$joinACL					=	true;
			$access						=	explode( '|*|', $access );

			if ( \count( $access ) > 1 ) {
				$where[]				=	"g." . $_CB_database->NameQuote( 'group_id' ) . " IN " . $_CB_database->safeArrayOfIntegers( $access );
			} else {
				$where[]				=	"g." . $_CB_database->NameQuote( 'group_id' ) . " = " . (int) $access[0];
			}
		}

		if ( $this->getFilterBasic()->count() ) {
			$orFilters					=	[];

			foreach ( $this->getFilterBasic() as $orFilter ) {
				/** @var ParamsInterface $orFilter */
				$andFilters				=	[];

				foreach ( $orFilter->subTree( 'filter' ) as $andFilter ) {
					/** @var ParamsInterface $andFilter */
					$column				=	$andFilter->getString( 'field', '' );

					if ( ! $column ) {
						continue;
					}

					if ( \in_array( $column, [ 'name', 'username', 'email', 'password', 'block', 'registerDate', 'lastvisitDate', 'params' ], true ) ) {
						$table			=	"u.";
					} else {
						$table			=	"c.";
					}

					$operator			=	$andFilter->getInt( 'operator', 0 );

					if ( $operator === 6 ) { // Empty
						$andFilters[]	=	"( " . $table . $_CB_database->NameQuote( $column ) . " IS NULL OR " . $_CB_database->NameQuote( $column ) . " = '' )";

						continue;
					}

					if ( $operator === 7 ) { // Not Empty
						$andFilters[]	=	"( " . $table . $_CB_database->NameQuote( $column ) . " IS NOT NULL AND " . $_CB_database->NameQuote( $column ) . " != '' )";

						continue;
					}

					if ( \in_array( $operator, [ 12, 13 ], true ) ) {
						$delimiter		=	$andFilter->getString( 'delimiter', '' );
					} else {
						$delimiter		=	null;
					}

					$value				=	$cbUser->replaceUserVars( $andFilter->getRaw( 'value', '' ), [ $_CB_database, 'getEscaped' ], false );

					switch ( $operator ) {
						case 1: // Not Equal To
							$operator	=	"!=";
							break;
						case 2: // Greater Than
							$operator	=	">";
							break;
						case 3: // Less Than
							$operator	=	"<";
							break;
						case 4: // Greater Than or Equal To
							$operator	=	">=";
							break;
						case 5: // Less Than or Equal To
							$operator	=	"<=";
							break;
						case 8: // Does Contain
							$operator	=	"LIKE";
							$value		=	'%' . addcslashes( $value, '%_' ) . '%';
							break;
						case 9: // Does Not Contain
							$operator	=	"NOT LIKE";
							$value		=	'%' . addcslashes( $value, '%_' ) . '%';
							break;
						case 10: // Is REGEX
							$operator	=	"REGEXP";
							break;
						case 11: // Is Not REGEX
							$operator	=	"NOT REGEXP";
							break;
						case 12: // Is In
							$operator	=	"IN";
							$value		=	explode( ( $delimiter ?: '|*|' ), $value );
							break;
						case 13: // Is Not In
							$operator	=	"NOT IN";
							$value		=	explode( ( $delimiter ?: '|*|' ), $value );
							break;
						case 0: // Equal To
						default:
							$operator	=	"=";
							break;
					}

					if ( \is_array( $value ) ) {
						$value			=	$_CB_database->safeArrayOfStrings( $value );
					} else {
						$value			=	$_CB_database->Quote( $value );
					}

					$filter				=	$table . $_CB_database->NameQuote( $column ) . " " . $operator . " " . $value;

					if ( ( $operator === '!=' ) && ( $value === '' ) ) {
						$filter			=	"( " . $table . $_CB_database->NameQuote( $column ) . " IS NOT NULL AND " . $filter . " )";
					} elseif ( \in_array( $operator, [ '!=', 'NOT LIKE', 'NOT REGEXP', 'NOT IN' ], true ) || ( ( $operator === '=' ) && ( $value === '' ) ) ) {
						$filter			=	"( " . $table . $_CB_database->NameQuote( $column ) . " IS NULL OR " . $filter . " )";
					}

					$andFilters[]		=	$filter;
				}

				if ( $andFilters ) {
					$orFilters[]		=	( \count( $andFilters ) > 1 ? "( " . implode( " AND ", $andFilters ) . " )" : $andFilters[0] );
				}
			}

			if ( $orFilters ) {
				$where[]				=	( \count( $orFilters ) > 1 ? "( " . implode( " OR ", $orFilters ) . " )" : $orFilters[0] );
			}
		}

		if ( $filterWHERE ) {
			if ( strpos( 'g.', $filterWHERE ) !== false ) {
				$joinACL				=	true;
			}

			$where[]					=	"( " . $filterWHERE . " )";
		}

		$query							=	"SELECT c.*, u.*"
										.	"\n FROM " . $_CB_database->NameQuote( '#__comprofiler' ) . " AS c"
										.	"\n INNER JOIN " . $_CB_database->NameQuote( '#__users' ) . " AS u"
										.	' ON u.' . $_CB_database->NameQuote( 'id' ) . ' = c.' . $_CB_database->NameQuote( 'id' );

		if ( $joinACL ) {
			$query						.=	"\n INNER JOIN " . $_CB_database->NameQuote( '#__user_usergroup_map' ) . " AS g"
										.	' ON g.' . $_CB_database->NameQuote( 'user_id' ) . ' = c.' . $_CB_database->NameQuote( 'id' );
		}

		$query							.=	( $filterJOIN ? "\n " . $filterJOIN : null )
										.	( $where ? "\n WHERE " . implode( "\n AND ", $where ) : null );

		if ( $joinACL ) {
			$query						.=	"\n GROUP BY c." . $_CB_database->NameQuote( 'id' );
		}

		if ( $batchLimit ) {
			$_CB_database->setQuery( $query, $batchProgress, $batchLimit );
		} else {
			$_CB_database->setQuery( $query );
		}

		$batchUsers						=	$_CB_database->loadObjectList( null, UserTable::class, [ $_CB_database ] );

		if ( $batchLimit && ( ! $batchUsers ) ) {
			// We're batch processing and have reached the end so lets increment uses:
			$this->incrementUseProgress( 1 );

			return $return;
		}

		foreach ( $batchUsers as $batchUser ) {
			foreach ( $autoActions as $autoAction ) {
				if ( ( $variables['var1'] ?? null ) instanceof UserTable ) {
					$variables['var1']			=	$batchUser;
				} else {
					for ( $i = 1, $n = 10; $i <= $n; $i++ ) {
						if ( ( $variables['var' . $i] ?? null ) ) {
							continue;
						}

						$variables['var' . $i]	=	$batchUser;
						break;
					}
				}

				switch ( $method ) {
					case 'run':
						$return					.=	$autoAction->runAction( $variables );
						break;
					case 'execute':
						$return					.=	$autoAction->executeAction( $batchUser, $variables );
						break;
					case 'trigger':
					default:
						$return					.=	$autoAction->triggerAction( $batchUser, $variables );
						break;
				}

				if ( $autoAction->getError() ) {
					$this->setError( $autoAction->getError() );
				}
			}

			if ( ! $this->incrementBatchProgress( 1 ) ) {
				// If incrementing fails we'll get stuck in a loop so abort processing this batch:
				return $return;
			}
		}

		if ( ! $batchLimit ) {
			// There is no batch so whatever amount of users was found is a completed use:
			$this->incrementUseProgress( 1 );
		} elseif ( $this->getBatchRedirect() ) {
			// Batching is in use and we're configured to redirect automatically to the next batch:
			if ( \in_array( Application::Input()->getString( 'format', '' ), [ 'raw', 'json' ], true ) ) {
				cbRedirect( $_CB_framework->pluginClassUrl( 'cbautoactions', false, [ 'action' => 'batch', 'batches' => $this->getId(), 'token' => md5( $_CB_framework->getCfg( 'secret' ) ) ], 'raw', 0, true ) );
			} else {
				cbRedirect( $_CB_framework->pluginClassUrl( 'cbautoactions', false, [ 'action' => 'batch', 'batches' => $this->getId(), 'token' => md5( $_CB_framework->getCfg( 'secret' ) ) ] ) );
			}
		}

		return $return;
	}
}