<?php
/**
* Community Builder (TM)
* @version $Id: $
* @package CommunityBuilder
* @copyright (C) 2004-2023 www.joomlapolis.com / Lightning MultiCom SA - and its licensors, all rights reserved
* @license http://www.gnu.org/licenses/old-licenses/gpl-2.0.html GNU/GPL version 2
*/

namespace CB\Plugin\Activity\Table;

use CBLib\Database\Table\OrderedTable;
use CBLib\Language\CBTxt;
use CBLib\Registry\Registry;
use CBLib\Application\Application;
use CB\Plugin\Activity\CBActivity;

defined('CBLIB') or die();

class EmoteTable extends OrderedTable
{
	/** @var int  */
	public $id				=	null;
	/** @var string  */
	public $value			=	null;
	/** @var string  */
	public $icon			=	null;
	/** @var string  */
	public $class			=	null;
	/** @var int  */
	public $access			=	null;
	/** @var int  */
	public $published		=	null;
	/** @var int  */
	public $ordering		=	null;
	/** @var string  */
	public $params			=	null;

	/** @var Registry  */
	protected $_params		=	null;

	/**
	 * Table name in database
	 *
	 * @var string
	 */
	protected $_tbl			=	'#__comprofiler_plugin_activity_emotes';

	/**
	 * Primary key(s) of table
	 *
	 * @var string
	 */
	protected $_tbl_key		=	'id';

	/**
	 * Ordering keys and for each their ordering groups.
	 * E.g.; array( 'ordering' => array( 'tab' ), 'ordering_registration' => array() )
	 * @var array
	 */
	protected $_orderings	=	array( 'ordering' => array() );

	/**
	 * @return bool
	 */
	public function check()
	{
		$emote		=	cbutf8_strtolower( preg_replace( '/[^-a-zA-Z0-9_.]/', '', $this->getString( 'value' ) ) );

		if ( ! $emote ) {
			$this->setError( CBTxt::T( 'Emote not specified!' ) );

			return false;
		}

		if ( ( ! $this->getString( 'icon' ) ) && ( ! $this->getString( 'class' ) ) ) {
			$this->setError( CBTxt::T( 'Icon not specified!' ) );

			return false;
		}

		$row	=	new EmoteTable();

		$row->load( array( 'value' => $emote ) );

		if ( $row->getInt( 'id', 0 ) && ( $this->getInt( 'id', 0 ) !== $row->getInt( 'id', 0 ) ) ) {
			$this->setError( CBTxt::T( 'Emote already exists!' ) );

			return false;
		}

		return true;
	}

	/**
	 * @param bool $updateNulls
	 * @return bool
	 */
	public function store( $updateNulls = false )
	{
		global $_PLUGINS;

		$new	=	( ! $this->getInt( 'id', 0 ) );
		$old	=	new self();

		$this->set( 'value', cbutf8_strtolower( preg_replace( '/[^-a-zA-Z0-9_.]/', '', $this->getString( 'value' ) ) ) );

		if ( ! $new ) {
			$old->load( $this->getInt( 'id', 0 ) );

			if ( in_array( false, $_PLUGINS->trigger( 'activity_onBeforeUpdateEmote', array( &$this, $old ) ), true ) ) {
				return false;
			}
		} elseif ( in_array( false, $_PLUGINS->trigger( 'activity_onBeforeCreateEmote', array( &$this ) ), true ) ) {
			return false;
		}

		if ( ! parent::store( $updateNulls ) ) {
			return false;
		}

		if ( ! $new ) {
			$_PLUGINS->trigger( 'activity_onAfterUpdateEmote', array( $this, $old ) );
		} else {
			$_PLUGINS->trigger( 'activity_onAfterCreateEmote', array( $this ) );
		}

		return true;
	}

	/**
	 * @param null|int $id
	 * @return bool
	 */
	public function delete( $id = null )
	{
		global $_PLUGINS;

		if ( in_array( false, $_PLUGINS->trigger( 'activity_onBeforeDeleteEmote', array( &$this ) ), true ) ) {
			return false;
		}

		if ( ! parent::delete( $id ) ) {
			return false;
		}

		$_PLUGINS->trigger( 'activity_onAfterDeleteEmote', array( $this ) );

		return true;
	}

	/**
	 * @return Registry
	 */
	public function params()
	{
		if ( ! ( $this->getRaw( '_params' ) instanceof Registry ) ) {
			$this->set( '_params', new Registry( $this->getRaw( 'params' ) ) );
		}

		return $this->getRaw( '_params' );
	}

	/**
	 * @return string
	 */
	public function icon()
	{
		global $_CB_framework;

		if ( Application::Application()->isClient( 'administrator' ) ) {
			CBActivity::getTemplate();
		}

		$emoteClass				=	'streamIconEmote streamIconEmote' . ucfirst( strtolower( preg_replace( '/[^-a-zA-Z0-9_]/', '', $this->getString( 'value' ) ) ) );

		if ( $this->getString( 'class' ) ) {
			$emoteClass			.=	' ' . $this->getString( 'class' );
		}

		if ( $this->getString( 'icon' ) ) {
			$template			=	CBActivity::getGlobalParams()->getString( 'general_template', 'default' );

			if ( ! $template ) {
				$template		=	'default';
			}

			$icon				=	'/components/com_comprofiler/plugin/user/plug_cbactivity/templates/' . $template . '/images/' . $this->getString( 'icon' );

			if ( ( $template !== 'default' ) && ( ! file_exists( $_CB_framework->getCfg( 'absolute_path' ) . $icon ) ) ) {
				$icon			=	'/components/com_comprofiler/plugin/user/plug_cbactivity/templates/default/images/' . $this->getString( 'icon' );
			}

			if ( ! file_exists( $_CB_framework->getCfg( 'absolute_path' ) . $icon ) ) {
				// Joomla images root fallback for B/C:
				$icon			=	'/images/' . $this->getString( 'icon' );
			}

			return '<img src="' . htmlspecialchars( $_CB_framework->getCfg( 'live_site' ) . $icon ) . '" loading="lazy" alt="' . htmlspecialchars( $this->getString( 'value' ) ) . '" class="img-fluid-inline align-text-bottom ' . htmlspecialchars( $emoteClass ) . '" />';
		}

		return '<span class="' . htmlspecialchars( $emoteClass ) . '"></span>';
	}
}