<?php
/**
* Community Builder (TM)
* @version $Id: $
* @package CommunityBuilder
* @copyright (C) 2004-2023 www.joomlapolis.com / Lightning MultiCom SA - and its licensors, all rights reserved
* @license http://www.gnu.org/licenses/old-licenses/gpl-2.0.html GNU/GPL version 2
*/

namespace CB\Plugin\GroupJivePhoto\Trigger;

use CB\Plugin\Activity\Notifications;
use CB\Plugin\Activity\NotificationsInterface;
use CB\Plugin\Activity\Table\NotificationTable;
use CB\Plugin\GroupJive\CBGroupJive;
use CB\Plugin\GroupJivePhoto\CBGroupJivePhoto;
use CB\Plugin\Activity\Table\ActivityTable;
use CB\Plugin\Activity\Activity;

defined('CBLIB') or die();

class ActivityTrigger extends \cbPluginHandler
{

	/**
	 * @param ActivityTable[]|NotificationTable[] $rows
	 * @param Activity|Notifications              $stream
	 */
	public function activityPrefetch( &$rows, $stream )
	{
		global $_CB_database;

		$notification			=	( $stream instanceof NotificationsInterface );
		$photoIds				=	array();

		foreach ( $rows as $k => $row ) {
			if ( ! preg_match( '/^groupjive\.group\.(\d+)\.photo\.(\d+)/', $row->getString( 'asset' ), $matches ) ) {
				continue;
			}

			$photoId			=	(int) $matches[2];

			if ( $photoId ) {
				$photoIds[$k]	=	$photoId;

				if ( ! $notification ) {
					$row->params()->set( 'overrides.tags_asset', 'asset' );
					$row->params()->set( 'overrides.likes_asset', 'asset' );
					$row->params()->set( 'overrides.comments_asset', 'asset' );
				}
			}
		}

		if ( ! $photoIds ) {
			return;
		}

		$query					=	'SELECT p.*'
								.	"\n FROM " . $_CB_database->NameQuote( '#__groupjive_plugin_photo' ) . " AS p"
								.	"\n WHERE p." . $_CB_database->NameQuote( 'id' ) . " IN " . $_CB_database->safeArrayOfIntegers( array_unique( $photoIds ) );
		$_CB_database->setQuery( $query );
		$photos					=	$_CB_database->loadObjectList( null, '\CB\Plugin\GroupJivePhoto\Table\PhotoTable', array( $_CB_database ) );

		if ( ! $photos ) {
			foreach ( $photoIds as $k => $photoId ) {
				unset( $rows[$k] );
			}

			return;
		}

		CBGroupJivePhoto::prefetchPhotos( $photos );
		CBGroupJive::prefetchUsers( $photos );

		$user					=	\CBuser::getMyUserDataInstance();

		foreach ( $photoIds as $k => $photoId ) {
			$photo				=	CBGroupJivePhoto::getPhoto( (int) $photoId );

			if ( ! $photo->getInt( 'id', 0 ) ) {
				unset( $rows[$k] );
			} elseif ( ( $photo->getInt( 'published', 1 ) !== 1 ) && ( ! CBGroupJive::isModerator( $user->getInt( 'id', 0 ) ) ) && ( CBGroupJive::getGroupStatus( $user, $photo->group() ) < 2 ) ) {
				unset( $rows[$k] );
			}
		}
	}

	/**
	 * @param ActivityTable|NotificationTable $row
	 * @param null|string                     $title
	 * @param null|string                     $date
	 * @param null|string                     $message
	 * @param null|string                     $insert
	 * @param null|string                     $footer
	 * @param array                           $menu
	 * @param Activity|Notifications          $stream
	 * @param string                          $output
	 */
	public function activityDisplay( &$row, &$title, &$date, &$message, &$insert, &$footer, &$menu, $stream, $output )
	{
		if ( ! preg_match( '/^groupjive\.group\.(\d+)\.photo\.(\d+)(?:\.([a-zA-Z_]+))?/', $row->getString( 'asset' ), $matches ) ) {
			return;
		}

		$photo		=	CBGroupJivePhoto::getPhoto( (int) $matches[2] );

		if ( ! $photo->getInt( 'id', 0 ) ) {
			return;
		}

		CBGroupJive::getTemplate( 'activity', true, true, $this->element );

		if ( ! $stream instanceof NotificationsInterface ) {
			$row->params()->set( 'overrides.tags_asset', 'asset' );
			$row->params()->set( 'overrides.likes_asset', 'asset' );
			$row->params()->set( 'overrides.comments_asset', 'asset' );
		}

		\HTML_groupjivePhotoActivity::showPhotoActivity( $row, $title, $date, $message, $insert, $footer, $menu, $stream, $matches, $photo, $this, $output );
	}

	/**
	 * @param string $asset
	 * @param mixed  $source
	 */
	public function assetSource( $asset, &$source )
	{
		if ( ! preg_match( '/^groupjive\.group\.(\d+)\.photo\.(\d+)/', $asset, $matches ) ) {
			return;
		}

		$photo		=	CBGroupJivePhoto::getPhoto( (int) $matches[2] );

		if ( ! $photo->getInt( 'id', 0 ) ) {
			return;
		}

		$source		=	$photo;
	}
}