<?php
/**
* Community Builder (TM)
* @version $Id: $
* @package CommunityBuilder
* @copyright (C) 2004-2022 www.joomlapolis.com / Lightning MultiCom SA - and its licensors, all rights reserved
* @license http://www.gnu.org/licenses/old-licenses/gpl-2.0.html GNU/GPL version 2
*/

namespace CB\Plugin\AntiSpam\Trigger;

use CB\Plugin\AntiSpam\Table\AttemptTable;
use CB\Plugin\AntiSpam\Table\LogTable;
use CB\Plugin\AntiSpam\Table\WhitelistTable;
use CBLib\Application\Application;
use CB\Plugin\AntiSpam\CBAntiSpam;
use CB\Database\Table\UserTable;
use CB\Plugin\AntiSpam\Table\BlockTable;
use CBLib\Language\CBTxt;
use CB\Plugin\AntiSpam\Captcha;

\defined('CBLIB') or die();

class UserTrigger extends \cbPluginHandler
{

	/**
	 * Deletes user data stored for the specified user
	 *
	 * @param UserTable $user
	 * @param int       $status
	 */
	public function deleteUserData( $user, $status ): void
	{
		global $_CB_database;

		if ( CBAntiSpam::getGlobalParams()->getBool( 'general_delete', true ) ) {
			$ipAddress		=	CBAntiSpam::getUserIP( $user );

			if ( $ipAddress ) {
				$query		=	'SELECT *'
							.	"\n FROM " . $_CB_database->NameQuote( '#__comprofiler_plugin_antispam_attempts' )
							.	"\n WHERE " . $_CB_database->NameQuote( 'ip_address' ) . " = " . $_CB_database->Quote( $ipAddress );
				$_CB_database->setQuery( $query );
				$attempts	=	$_CB_database->loadObjectList( null, AttemptTable::class, [ $_CB_database ] );

				/** @var AttemptTable[] $attempts */
				foreach ( $attempts as $attempt ) {
					$attempt->delete();
				}
			}

			$query			=	'SELECT *'
							.	"\n FROM " . $_CB_database->NameQuote( '#__comprofiler_plugin_antispam_log' )
							.	"\n WHERE " . $_CB_database->NameQuote( 'user_id' ) . " = " . $user->getInt( 'id', 0 );
			if ( $ipAddress ) {
				$query		.=	" OR " . $_CB_database->NameQuote( 'ip_address' ) . " = " . $_CB_database->Quote( $ipAddress );
			}
			$_CB_database->setQuery( $query );
			$logs			=	$_CB_database->loadObjectList( null, LogTable::class, [ $_CB_database ] );

			/** @var LogTable[] $logs */
			foreach ( $logs as $log ) {
				$log->delete();
			}

			$query			=	"SELECT *"
							.	"\n FROM " . $_CB_database->NameQuote( '#__comprofiler_plugin_antispam_block' )
							.	"\n WHERE ( " . $_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'user' )
							.	" AND " . $_CB_database->NameQuote( 'value' ) . " = " . $user->getInt( 'id', 0 ) . " )"
							.	" OR ( " . $_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'email' )
							.	" AND " . $_CB_database->NameQuote( 'value' ) . " = " . $_CB_database->Quote( $user->getString( 'email', '' ) ) . " )";
			if ( $ipAddress ) {
				$query		.=	" OR ( " . $_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'ip' )
							.	" AND " . $_CB_database->NameQuote( 'value' ) . " = " . $_CB_database->Quote( $ipAddress ) . " )";
			}
			$_CB_database->setQuery( $query );
			$blocks			=	$_CB_database->loadObjectList( null, BlockTable::class, [ $_CB_database ] );

			/** @var BlockTable[] $blocks */
			foreach ( $blocks as $block ) {
				$block->delete();
			}

			$query			=	"SELECT *"
							.	"\n FROM " . $_CB_database->NameQuote( '#__comprofiler_plugin_antispam_whitelist' )
							.	"\n WHERE ( " . $_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'user' )
							.	" AND " . $_CB_database->NameQuote( 'value' ) . " = " . $user->getInt( 'id', 0 ) . " )"
							.	" OR ( " . $_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'email' )
							.	" AND " . $_CB_database->NameQuote( 'value' ) . " = " . $_CB_database->Quote( $user->getString( 'email', '' ) ) . " )";
			if ( $ipAddress ) {
				$query		.=	" OR ( " . $_CB_database->NameQuote( 'type' ) . " = " . $_CB_database->Quote( 'ip' )
							.	" AND " . $_CB_database->NameQuote( 'value' ) . " = " . $_CB_database->Quote( $ipAddress ) . " )";
			}
			$_CB_database->setQuery( $query );
			$whitelists		=	$_CB_database->loadObjectList( null, WhitelistTable::class, [ $_CB_database ] );

			/** @var WhitelistTable[] $whitelists */
			foreach ( $whitelists as $whitelist ) {
				$whitelist->delete();
			}
		}
	}

	/**
	 * Blocks a registration attempt
	 *
	 * @param string|BlockTable $block
	 */
	private function blockRegistration( $block ): void
	{
		global $_PLUGINS;

		if ( \is_object( $block ) ) {
			if ( $block instanceof BlockTable ) {
				if ( $block->isExpired() ) {
					return;
				}

				$reason		=	$block->getReason;
				$duration	=	$block->getDuration();
				$date		=	$block->getDate();
				$expire		=	$block->getExpirationDate();
			} else {
				return;
			}
		} else {
			$reason			=	$block;
			$duration		=	null;
			$date			=	null;
			$expire			=	null;
		}

		if ( ! $reason ) {
			$reason			=	'Spam.';
		}

		$extras				=	[	'[duration]'	=>	ucwords( strtolower( str_replace( [ '+', '-' ], '', $duration ) ) ),
									'[date]'		=>	$date . ' UTC',
									'[expire]'		=>	$expire . ( $duration ? ' UTC' : null ),
								];

		$extras				=	array_merge( $extras, [ '[reason]' => CBTxt::T( 'REG_BLOCK_REDIRECT_REASON', $reason, $extras ) ] );

		$redirect			=	$this->params->getString( 'reg_block_redirect', '' );
		$redirectMsg		=	CBTxt::T( 'REG_BLOCK_REDIRECT_MSG', $this->params->getString( 'reg_block_redirect_msg', 'Your registration attempt has been blocked. Reason: [reason]' ), $extras );
		$redirectType		=	$this->params->getString( 'reg_block_redirect_type', 'error' );

		if ( ! $redirect ) {
			$redirect		=	'index.php';
		}

		cbRedirect( $redirect, $redirectMsg, $redirectType );

		$_PLUGINS->_setErrorMSG( $redirectMsg );
		$_PLUGINS->raiseError();
	}

	/**
	 * Displays legacy user email form captcha
	 *
	 * @param UserTable $rowFrom
	 * @param UserTable $rowTo
	 * @param string    $warning
	 * @param int       $ui
	 * @param int       $allowPublic
	 * @param string    $name
	 * @param string    $email
	 * @param string    $subject
	 * @param string    $message
	 * @return null|string
	 */
	public function displayEmailCaptcha( &$rowFrom, &$rowTo, &$warning, $ui, &$allowPublic, &$name, &$email, &$subject, &$message ): ?string
	{
		if ( ! $this->params->getBool( 'captcha_legacy_email', false ) ) {
			return null;
		}

		$captcha	=	new Captcha( 'email', $this->params->getString( 'captcha_legacy_email_mode', '' ) );

		return	$captcha->getCaptcha();
	}

	/**
	 * Validates legacy user email form captcha
	 *
	 * @param UserTable $rowFrom
	 * @param UserTable $rowTo
	 * @param int $ui
	 * @param string $emailName
	 * @param string $emailAddress
	 * @param string $subject
	 * @param string $message
	 */
	public function validateEmailCaptcha( &$rowFrom, &$rowTo, $ui, &$emailName, &$emailAddress, &$subject, &$message ): void
	{
		global $_PLUGINS;

		if ( ! $this->params->getBool( 'captcha_legacy_email', false ) ) {
			return;
		}

		$captcha	=	new Captcha( 'email', $this->params->getString( 'captcha_legacy_email_mode', '' ) );

		if ( ( ! $captcha->load() ) || ( ! $captcha->validateValue() ) ) {
			$_PLUGINS->_setErrorMSG( ( $captcha->getError() ?: CBTxt::T( 'Invalid Captcha Code' ) ) );
			$_PLUGINS->raiseError();
		}
	}

	/**
	 * Handles registration blocking (only ip address) and attempts log
	 */
	public function beforeRegistration(): void
	{
		$blocked		=	CBAntiSpam::getBlock();

		if ( $blocked ) {
			$this->blockRegistration( $blocked );
			return;
		}

		$attempt		=	CBAntiSpam::logAttempt( 'reg' );

		if ( $attempt ) {
			$blocked	=	CBAntiSpam::blockAttempt( $attempt );
		}

		if ( $blocked ) {
			$this->blockRegistration( $blocked );
		}
	}

	/**
	 * Handles registration blocking (user and ip address) and duplicate registration check
	 *
	 * @param UserTable $user
	 * @param UserTable $userDuplicate
	 */
	public function duringRegistration( &$user, &$userDuplicate ): void
	{
		global $_CB_database;

		$ipAddress					=	Application::Input()->getRequestIP();

		if ( ! CBAntiSpam::isBlockable( $user, $ipAddress ) ) {
			return;
		}

		$blocked					=	CBAntiSpam::getBlock( $user, $ipAddress );

		if ( $blocked ) {
			$this->blockRegistration( $blocked );
			return;
		}

		if ( $this->params->getBool( 'reg_duplicate', false ) ) {
			$timeframe				=	$this->params->getString( 'reg_duplicate_timeframe', '-1 YEAR' );

			$query					=	'SELECT COUNT(*)'
									.	"\n FROM " . $_CB_database->NameQuote( '#__comprofiler_plugin_antispam_log' ) . " AS l"
									.	"\n INNER JOIN " . $_CB_database->NameQuote( '#__users' ) . " AS u"
									.	' ON u.' . $_CB_database->NameQuote( 'id' ) . ' = l.' . $_CB_database->NameQuote( 'user_id' )
									.	"\n WHERE l." . $_CB_database->NameQuote( 'ip_address' ) . " = " . $_CB_database->Quote( $ipAddress );
			if ( $timeframe ) {
				$query				.=	"\n AND l." . $_CB_database->NameQuote( 'date' ) . " >= " . $_CB_database->Quote( Application::Date( 'now', 'UTC' )->modify( strtoupper( $timeframe ) )->format( 'Y-m-d H:i:s' ) );
			}
			$_CB_database->setQuery( $query );
			$accounts				=	$_CB_database->loadResult();

			$count					=	$this->params->getInt( 'reg_duplicate_count', 1 );

			if ( ! $count ) {
				$count				=	1;
			}

			if ( $accounts >= $count ) {
				$method				=	$this->params->getInt( 'reg_duplicate_method', 0 );
				$reason				=	$this->params->getString( 'reg_duplicate_reason', 'Already registered.' );

				if ( $method === 1 ) {
					$row			=	new BlockTable();

					$row->setType( 'ip' );
					$row->setValue( $ipAddress );
					$row->setDuration( $this->params->getString( 'reg_duplicate_dur', '+1 HOUR' ) );
					$row->setReason( $reason );

					$row->store();

					$this->blockRegistration( $row );
				} else {
					$this->blockRegistration( $reason );
				}
			}
		}
	}

	/**
	 * Updates users ip address log and clears attempts
	 *
	 * @param UserTable $userComplete
	 * @param array     $messagesToUser
	 * @param int       $ui
	 */
	public function afterRegistration( &$userComplete, &$messagesToUser, $ui ): void
	{
		CBAntiSpam::logIPAddress( $userComplete, 'reg' );
	}
}