<?php
/**
* Community Builder (TM)
* @version $Id: $
* @package CommunityBuilder
* @copyright (C) 2004-2022 www.joomlapolis.com / Lightning MultiCom SA - and its licensors, all rights reserved
* @license http://www.gnu.org/licenses/old-licenses/gpl-2.0.html GNU/GPL version 2
*/

use CBLib\Language\CBTxt;
use CB\Database\Table\UserTable;
use CB\Plugin\AntiSpam\Table\BlockTable;

if ( ! ( defined( '_VALID_CB' ) || defined( '_JEXEC' ) || defined( '_VALID_MOS' ) ) ) { die( 'Direct Access to this location is not allowed.' ); }

/**
 * @var BlockTable[] $rows
 * @var cbPageNav    $pageNav
 * @var bool         $isOwner
 * @var UserTable    $user
 * @var string       $tabUrl
 * @var string       $returnUrl
 * @var string       $blockAccountUrl
 * @var string       $blockUserUrl
 * @var string       $blockIpUrl
 * @var string       $blockEmailUrl
 * @var string       $blockDomainUrl
 * @var bool         $hasPaging
 */

global $_CB_framework;

$buttons			=	[];

if ( $blockAccountUrl ) {
	$buttons[]		=	'<button type="button" onclick="window.location.href=\'' . $blockAccountUrl . '\';" class="btn btn-secondary btn-sm-block blocksButton blocksButtonBlockAccount"><span class="fa fa-user"></span> ' . CBTxt::T( 'Block Account' ) . '</button>';
}

if ( $blockUserUrl ) {
	$buttons[]		=	'<button type="button" onclick="window.location.href=\'' . $blockUserUrl . '\';" class="btn btn-secondary btn-sm-block blocksButton blocksButtonBlockUser"><span class="fa fa-user"></span> ' . CBTxt::T( 'Block User' ) . '</button>';
}

if ( $blockIpUrl ) {
	$buttons[]		=	'<button type="button" onclick="window.location.href=\'' . $blockIpUrl . '\';" class="btn btn-secondary btn-sm-block blocksButton blocksButtonBlockIP"><span class="fa fa-flag"></span> ' . CBTxt::T( 'Block IP Address' ) . '</button>';
}

if ( $blockEmailUrl ) {
	$buttons[]		=	'<button type="button" onclick="window.location.href=\'' . $blockEmailUrl . '\';" class="btn btn-secondary btn-sm-block blocksButton blocksButtonBlockEmail"><span class="fa fa-at"></span> ' . CBTxt::T( 'Block Email Address' ) . '</button>';
}

if ( $blockDomainUrl ) {
	$buttons[]		=	'<button type="button" onclick="window.location.href=\'' . $blockDomainUrl . '\';" class="btn btn-secondary btn-sm-block blocksButton blocksButtonBlockDomain"><span class="fa fa-globe"></span> ' . CBTxt::T( 'Block Email Domain' ) . '</button>';
}
?>
<div class="blocksTab">
	<form action="<?php echo $tabUrl; ?>" method="post" name="blocksForm" class="m-0 blocksForm">
		<?php if ( $buttons ) { ?>
		<div class="mb-2 blocksHeader">
			<?php echo implode( ' ', $buttons ); ?>
		</div>
		<?php } ?>
		<div class="table-responsive blocksItemsContainer">
			<table class="table table-hover mb-0">
				<thead>
					<tr class="blocksItemsHeader">
						<th><?php echo CBTxt::T( 'Value' ); ?></th>
						<th style="width: 20%;" class="text-center d-none d-sm-table-cell"><?php echo CBTxt::T( 'Type' ); ?></th>
						<th style="width: 30%;" class="d-none d-sm-table-cell"><?php echo CBTxt::T( 'Reason' ); ?></th>
						<th style="width: 25%;" class="d-none d-sm-table-cell"><?php echo CBTxt::T( 'Date' ); ?></th>
						<th style="width: 5%;" class="text-center"><?php echo CBTxt::T( 'Blocked' ); ?></th>
						<th style="width: 1%;">&nbsp;</th>
					</tr>
				</thead>
				<tbody>
				<?php if ( $rows ) {
					foreach ( $rows as $row ) {
						$menuItems		=	'<ul class="list-unstyled dropdown-menu d-block position-relative m-0 blocksMenuItems">'
										.		'<li class="blocksMenuItem"><a href="' . $_CB_framework->pluginClassUrl( 'cbantispam', true, [ 'action' => 'block', 'func' => 'edit', 'id' => $row->getId(), 'return' => $returnUrl ] ) . '" class="dropdown-item"><span class="fa fa-edit"></span> ' . CBTxt::T( 'Edit' ) . '</a></li>'
										.		'<li class="blocksMenuItem"><a href="javascript: void(0);" onclick="cbjQuery.cbconfirm( \'' . addslashes( CBTxt::T( 'Are you sure you want to delete this block?' ) ) . '\' ).done( function() { window.location.href = \'' . addslashes( $_CB_framework->pluginClassUrl( 'cbantispam', true, [ 'action' => 'block', 'func' => 'delete', 'id' => $row->getId(), 'return' => $returnUrl ] ) ) . '\'; })" class="dropdown-item"><span class="fa fa-trash-o"></span> ' . CBTxt::T( 'Delete' ) . '</a></li>'
										.	'</ul>';

						$menuAttr		=	cbTooltip( null, $menuItems, null, 'auto', null, null, null, 'class="btn btn-secondary btn-sm blocksMenu" data-cbtooltip-menu="true" data-cbtooltip-classes="qtip-nostyle" data-cbtooltip-open-classes="active"' );
						?>
						<tr class="blocksItemContainer">
							<td><?php echo $row->getValue(); ?></td>
							<td style="width: 20%;" class="text-center d-none d-sm-table-cell"><?php echo $row->renderType(); ?></td>
							<td style="width: 30%;" class="d-none d-sm-table-cell"><?php echo $row->getReason(); ?></td>
							<td style="width: 25%;" class="d-none d-sm-table-cell">
								<?php if ( $row->getDuration() ) { ?>
									<div class="text-info" title="<?php echo htmlspecialchars( CBTxt::T( 'Start' ) ); ?>"><?php echo cbFormatDate( $row->getDate(), false ); ?></div>
									<div class="text-success" title="<?php echo htmlspecialchars( CBTxt::T( 'End' ) ); ?>"><?php echo cbFormatDate( $row->getExpirationDate(), false ); ?></div>
								<?php } else { ?>
									<?php echo CBTxt::T( 'Forever' ); ?>
								<?php } ?>
							</td>
							<td style="width: 5%;" class="text-center">
								<?php if ( ! $row->isExpired() ) { ?>
									<span class="fa fa-check text-success" title="<?php echo htmlspecialchars( CBTxt::T( 'Blocked' ) ); ?>"></span>
								<?php } else { ?>
									<span class="fa fa-times text-danger" title="<?php echo htmlspecialchars( CBTxt::T( 'Not Blocked' ) ); ?>"></span>
								<?php } ?>
							</td>
							<td style="width: 1%;" class="text-right">
								<button type="button"<?php echo $menuAttr; ?>><span class="fa fa-cog"></span> <span class="fa fa-caret-down"></span></button>
							</td>
						</tr>
					<?php } ?>
				<?php } else { ?>
				<tr class="blocksItemsEmpty">
					<td colspan="5">
						<?php if ( $isOwner ) { ?>
							<?php echo CBTxt::T( 'You have no blocks.' ); ?>
						<?php } else { ?>
							<?php echo CBTxt::T( 'This user has no blocks.' ); ?>
						<?php } ?>
						</td>
					</tr>
				<?php } ?>
				</tbody>
				<?php if ( $hasPaging ) { ?>
				<tfoot>
					<tr class="blocksItemsPaging">
						<td colspan="5" class="text-center">
							<?php echo $pageNav->getListLinks(); ?>
						</td>
					</tr>
				</tfoot>
				<?php } ?>
			</table>
		</div>
		<?php echo $pageNav->getLimitBox( false ); ?>
	</form>
</div>