<?php
/**
* Community Builder (TM)
* @version $Id: $
* @package CommunityBuilder
* @copyright (C) 2004-2022 www.joomlapolis.com / Lightning MultiCom SA - and its licensors, all rights reserved
* @license http://www.gnu.org/licenses/old-licenses/gpl-2.0.html GNU/GPL version 2
*/

use CBLib\Language\CBTxt;
use CB\Database\Table\UserTable;
use CB\Plugin\AntiSpam\Table\WhitelistTable;

if ( ! ( defined( '_VALID_CB' ) || defined( '_JEXEC' ) || defined( '_VALID_MOS' ) ) ) { die( 'Direct Access to this location is not allowed.' ); }

/**
 * @var WhitelistTable[] $rows
 * @var cbPageNav        $pageNav
 * @var bool             $isOwner
 * @var UserTable        $user
 * @var string           $tabUrl
 * @var string           $returnUrl
 * @var string           $whitelistUserUrl
 * @var string           $whitelistIpUrl
 * @var string           $whitelistEmailUrl
 * @var string           $whitelistDomainUrl
 * @var bool             $hasPaging
 */

global $_CB_framework;

$buttons		=	[];

if ( $whitelistUserUrl ) {
	$buttons[]	=	'<button type="button" onclick="window.location.href=\'' . $whitelistUserUrl . '\';" class="btn btn-secondary btn-sm-block whitelistsButton whitelistsButtonWhitelistUser"><span class="fa fa-user"></span> ' . CBTxt::T( 'Whitelist User' ) . '</button>';
}

if ( $whitelistIpUrl ) {
	$buttons[]	=	'<button type="button" onclick="window.location.href=\'' . $whitelistIpUrl . '\';" class="btn btn-secondary btn-sm-block whitelistsButton whitelistsButtonWhitelistIP"><span class="fa fa-flag"></span> ' . CBTxt::T( 'Whitelist IP Address' ) . '</button>';
}

if ( $whitelistEmailUrl ) {
	$buttons[]	=	'<button type="button" onclick="window.location.href=\'' . $whitelistEmailUrl . '\';" class="btn btn-secondary btn-sm-block whitelistsButton whitelistsButtonWhitelistEmail"><span class="fa fa-at"></span> ' . CBTxt::T( 'Whitelist Email Address' ) . '</button>';
}

if ( $whitelistDomainUrl ) {
	$buttons[]	=	'<button type="button" onclick="window.location.href=\'' . $whitelistDomainUrl . '\';" class="btn btn-secondary btn-sm-block whitelistsButton whitelistsButtonWhitelistDomain"><span class="fa fa-globe"></span> ' . CBTxt::T( 'Whitelist Email Domain' ) . '</button>';
}
?>
<div class="whitelistsTab">
	<form action="<?php echo $tabUrl; ?>" method="post" name="whitelistsForm" class="m-0 whitelistsForm">
		<?php if ( $buttons ) { ?>
		<div class="mb-2 whitelistsHeader">
			<?php echo implode( ' ', $buttons ); ?>
		</div>
		<?php } ?>
		<div class="table-responsive whitelistsItemsContainer">
			<table class="table table-hover mb-0">
				<thead>
					<tr class="whitelistsItemsHeader">
						<th><?php echo CBTxt::T( 'Value' ); ?></th>
						<th style="width: 20%;" class="text-center d-none d-sm-table-cell"><?php echo CBTxt::T( 'Type' ); ?></th>
						<th style="width: 30%;" class="d-none d-sm-table-cell"><?php echo CBTxt::T( 'Reason' ); ?></th>
						<th style="width: 25%;" class="d-none d-sm-table-cell"><?php echo CBTxt::T( 'Date' ); ?></th>
						<th style="width: 1%;">&nbsp;</th>
					</tr>
				</thead>
				<tbody>
					<?php if ( $rows ) {
						foreach ( $rows as $row ) {
							$menuItems		=	'<ul class="list-unstyled dropdown-menu d-block position-relative m-0 whitelistsMenuItems">'
											.		'<li class="whitelistsMenuItem"><a href="' . $_CB_framework->pluginClassUrl( 'cbantispam', true, [ 'action' => 'whitelist', 'func' => 'edit', 'id' => $row->getId(), 'return' => $returnUrl ] ) . '" class="dropdown-item"><span class="fa fa-edit"></span> ' . CBTxt::T( 'Edit' ) . '</a></li>'
											.		'<li class="whitelistsMenuItem"><a href="javascript: void(0);" onclick="cbjQuery.cbconfirm( \'' . addslashes( CBTxt::T( 'Are you sure you want to delete this whitelist?' ) ) . '\' ).done( function() { window.location.href = \'' . addslashes( $_CB_framework->pluginClassUrl( 'cbantispam', true, [ 'action' => 'whitelist', 'func' => 'delete', 'id' => $row->getId(), 'return' => $returnUrl ] ) ) . '\'; })" class="dropdown-item"><span class="fa fa-trash-o"></span> ' . CBTxt::T( 'Delete' ) . '</a></li>'
											.	'</ul>';

							$menuAttr		=	cbTooltip( null, $menuItems, null, 'auto', null, null, null, 'class="btn btn-secondary btn-sm whitelistsMenu" data-cbtooltip-menu="true" data-cbtooltip-classes="qtip-nostyle" data-cbtooltip-open-classes="active"' );
							?>
							<tr class="whitelistsItemContainer">
								<td><?php echo $row->getValue(); ?></td>
								<td style="width: 20%;" class="text-center d-none d-sm-table-cell"><?php echo $row->renderType(); ?></td>
								<td style="width: 30%;" class="d-none d-sm-table-cell"><?php echo $row->getReason(); ?></td>
								<td style="width: 25%;" class="d-none d-sm-table-cell"><?php echo cbFormatDate( $row->getDate(), false ); ?></td>
								<td style="width: 1%;" class="text-right">
									<button type="button"<?php echo $menuAttr; ?>><span class="fa fa-cog"></span> <span class="fa fa-caret-down"></span></button>
								</td>
							</tr>
						<?php } ?>
					<?php } else { ?>
					<tr class="whitelistsItemsEmpty">
						<td colspan="4">
						<?php if ( $isOwner ) { ?>
							<?php echo CBTxt::T( 'You have no whitelists.' ); ?>
						<?php } else { ?>
							<?php echo CBTxt::T( 'This user has no whitelists.' ); ?>
						<?php } ?>
						</td>
					</tr>
					<?php } ?>
				</tbody>
				<?php if ( $hasPaging ) { ?>
				<tfoot>
					<tr class="whitelistsItemsPaging">
						<td colspan="4" class="text-center">
							<?php echo $pageNav->getListLinks(); ?>
						</td>
					</tr>
				</tfoot>
				<?php } ?>
			</table>
		</div>
		<?php echo $pageNav->getLimitBox( false ); ?>
	</form>
</div>